@@ -36,11 +36,12 @@ import org.apache.druid.segment.BaseObjectColumnValueSelector;
 import org.apache.druid.segment.ColumnSelectorFactory;
 import org.apache.druid.segment.Cursor;
 import org.apache.druid.segment.CursorBuildSpec;
+import org.apache.druid.segment.CursorFactory;
 import org.apache.druid.segment.CursorHolder;
 import org.apache.druid.segment.DimensionSelector;
 import org.apache.druid.segment.QueryableIndex;
 import org.apache.druid.segment.QueryableIndexSegment;
-import org.apache.druid.segment.ReferenceCountingSegment;
+import org.apache.druid.segment.ReferenceCountedSegmentProvider;
 import org.apache.druid.segment.Segment;
 import org.apache.druid.segment.VirtualColumns;
 import org.apache.druid.segment.column.ValueType;
@@ -193,10 +194,11 @@ public class IndexedTableJoinCursorBenchmark
 
     hashJoinSegment = closer.register(
         new HashJoinSegment(
-            ReferenceCountingSegment.wrapRootGenerationSegment(baseSegment),
+            ReferenceCountedSegmentProvider.wrapRootGenerationSegment(baseSegment).acquireReference().orElseThrow(),
             null,
             clauses,
-            preAnalysis
+            preAnalysis,
+            () -> {}
         )
     );
   }
@@ -233,7 +235,7 @@ public class IndexedTableJoinCursorBenchmark
 
   private CursorHolder makeCursorHolder()
   {
-    return hashJoinSegment.asCursorFactory().makeCursorHolder(CursorBuildSpec.FULL_SCAN);
+    return hashJoinSegment.as(CursorFactory.class).makeCursorHolder(CursorBuildSpec.FULL_SCAN);
   }
 
 
