@@ -33,11 +33,12 @@ import org.apache.druid.query.lookup.LookupExtractorFactoryContainerProvider;
 import org.apache.druid.query.lookup.MapLookupExtractorFactory;
 import org.apache.druid.segment.Cursor;
 import org.apache.druid.segment.CursorBuildSpec;
+import org.apache.druid.segment.CursorFactory;
 import org.apache.druid.segment.CursorHolder;
 import org.apache.druid.segment.DimensionSelector;
 import org.apache.druid.segment.QueryableIndex;
 import org.apache.druid.segment.QueryableIndexSegment;
-import org.apache.druid.segment.ReferenceCountingSegment;
+import org.apache.druid.segment.ReferenceCountedSegmentProvider;
 import org.apache.druid.segment.Segment;
 import org.apache.druid.segment.VirtualColumns;
 import org.apache.druid.segment.column.ColumnConfig;
@@ -151,10 +152,11 @@ public class JoinAndLookupBenchmark
         );
 
     hashJoinLookupStringKeySegment = new HashJoinSegment(
-        ReferenceCountingSegment.wrapRootGenerationSegment(baseSegment),
+        ReferenceCountedSegmentProvider.wrapRootGenerationSegment(baseSegment).acquireReference().orElseThrow(),
         null,
         joinableClausesLookupStringKey,
-        preAnalysisLookupStringKey
+        preAnalysisLookupStringKey,
+        () -> {}
     );
 
     List<JoinableClause> joinableClausesLookupLongKey = ImmutableList.of(
@@ -187,10 +189,11 @@ public class JoinAndLookupBenchmark
         );
 
     hashJoinLookupLongKeySegment = new HashJoinSegment(
-        ReferenceCountingSegment.wrapRootGenerationSegment(baseSegment),
+        ReferenceCountedSegmentProvider.wrapRootGenerationSegment(baseSegment).acquireReference().orElseThrow(),
         null,
         joinableClausesLookupLongKey,
-        preAnalysisLookupLongKey
+        preAnalysisLookupLongKey,
+        () -> {}
     );
 
     List<JoinableClause> joinableClausesIndexedTableStringKey = ImmutableList.of(
@@ -223,10 +226,11 @@ public class JoinAndLookupBenchmark
         );
 
     hashJoinIndexedTableStringKeySegment = new HashJoinSegment(
-        ReferenceCountingSegment.wrapRootGenerationSegment(baseSegment),
+        ReferenceCountedSegmentProvider.wrapRootGenerationSegment(baseSegment).acquireReference().orElseThrow(),
         null,
         joinableClausesIndexedTableStringKey,
-        preAnalysisIndexedStringKey
+        preAnalysisIndexedStringKey,
+        () -> {}
     );
 
     List<JoinableClause> joinableClausesIndexedTableLongKey = ImmutableList.of(
@@ -259,10 +263,11 @@ public class JoinAndLookupBenchmark
         );
 
     hashJoinIndexedTableLongKeySegment = new HashJoinSegment(
-        ReferenceCountingSegment.wrapRootGenerationSegment(baseSegment),
+        ReferenceCountedSegmentProvider.wrapRootGenerationSegment(baseSegment).acquireReference().orElseThrow(),
         null,
         joinableClausesIndexedTableLongKey,
-        preAnalysisIndexedLongKey
+        preAnalysisIndexedLongKey,
+        () -> {}
     );
 
     final Map<String, String> countryCodeToNameMap = JoinTestHelper.createCountryIsoCodeToNameLookup().getMap();
@@ -364,7 +369,7 @@ public class JoinAndLookupBenchmark
   @OutputTimeUnit(TimeUnit.MILLISECONDS)
   public void baseSegment(Blackhole blackhole)
   {
-    try (final CursorHolder cursorHolder = baseSegment.asCursorFactory().makeCursorHolder(CursorBuildSpec.FULL_SCAN)) {
+    try (final CursorHolder cursorHolder = baseSegment.as(CursorFactory.class).makeCursorHolder(CursorBuildSpec.FULL_SCAN)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, "countryIsoCode"));
     }
@@ -379,7 +384,7 @@ public class JoinAndLookupBenchmark
     final CursorBuildSpec buildSpec = CursorBuildSpec.builder()
                                                      .setFilter(filter)
                                                      .build();
-    try (final CursorHolder cursorHolder = baseSegment.asCursorFactory().makeCursorHolder(buildSpec)) {
+    try (final CursorHolder cursorHolder = baseSegment.as(CursorFactory.class).makeCursorHolder(buildSpec)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, "countryIsoCode"));
     }
@@ -390,7 +395,7 @@ public class JoinAndLookupBenchmark
   @OutputTimeUnit(TimeUnit.MILLISECONDS)
   public void joinLookupStringKey(Blackhole blackhole)
   {
-    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.asCursorFactory()
+    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.as(CursorFactory.class)
                                                                          .makeCursorHolder(CursorBuildSpec.FULL_SCAN)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, "c.v"));
@@ -406,7 +411,7 @@ public class JoinAndLookupBenchmark
     final CursorBuildSpec buildSpec = CursorBuildSpec.builder()
                                                      .setFilter(filter)
                                                      .build();
-    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.asCursorFactory()
+    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.as(CursorFactory.class)
                                                                          .makeCursorHolder(buildSpec)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, "c.v"));
@@ -418,7 +423,7 @@ public class JoinAndLookupBenchmark
   @OutputTimeUnit(TimeUnit.MILLISECONDS)
   public void joinLookupLongKey(Blackhole blackhole)
   {
-    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.asCursorFactory()
+    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.as(CursorFactory.class)
                                                                          .makeCursorHolder(CursorBuildSpec.FULL_SCAN)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, "c.v"));
@@ -434,7 +439,7 @@ public class JoinAndLookupBenchmark
     final CursorBuildSpec buildSpec = CursorBuildSpec.builder()
                                                      .setFilter(filter)
                                                      .build();
-    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.asCursorFactory()
+    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.as(CursorFactory.class)
                                                                          .makeCursorHolder(buildSpec)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, "c.v"));
@@ -446,7 +451,7 @@ public class JoinAndLookupBenchmark
   @OutputTimeUnit(TimeUnit.MILLISECONDS)
   public void joinIndexedTableLongKey(Blackhole blackhole)
   {
-    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.asCursorFactory()
+    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.as(CursorFactory.class)
                                                                          .makeCursorHolder(CursorBuildSpec.FULL_SCAN)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, "c.countryName"));
@@ -462,7 +467,7 @@ public class JoinAndLookupBenchmark
     final CursorBuildSpec buildSpec = CursorBuildSpec.builder()
                                                      .setFilter(filter)
                                                      .build();
-    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.asCursorFactory()
+    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.as(CursorFactory.class)
                                                                          .makeCursorHolder(buildSpec)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, "c.countryName"));
@@ -474,7 +479,7 @@ public class JoinAndLookupBenchmark
   @OutputTimeUnit(TimeUnit.MILLISECONDS)
   public void joinIndexedTableStringKey(Blackhole blackhole)
   {
-    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.asCursorFactory()
+    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.as(CursorFactory.class)
                                                                          .makeCursorHolder(CursorBuildSpec.FULL_SCAN)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, "c.countryName"));
@@ -490,7 +495,7 @@ public class JoinAndLookupBenchmark
     final CursorBuildSpec buildSpec = CursorBuildSpec.builder()
                                                      .setFilter(filter)
                                                      .build();
-    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.asCursorFactory()
+    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.as(CursorFactory.class)
                                                                          .makeCursorHolder(buildSpec)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, "c.countryName"));
@@ -505,7 +510,7 @@ public class JoinAndLookupBenchmark
     final CursorBuildSpec buildSpec = CursorBuildSpec.builder()
                                                      .setVirtualColumns(lookupVirtualColumns)
                                                      .build();
-    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.asCursorFactory()
+    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.as(CursorFactory.class)
                                                                          .makeCursorHolder(buildSpec)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, LOOKUP_COUNTRY_CODE_TO_NAME));
@@ -522,7 +527,7 @@ public class JoinAndLookupBenchmark
                                                      .setFilter(filter)
                                                      .setVirtualColumns(lookupVirtualColumns)
                                                      .build();
-    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.asCursorFactory()
+    try (final CursorHolder cursorHolder = hashJoinLookupStringKeySegment.as(CursorFactory.class)
                                                                          .makeCursorHolder(buildSpec)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, LOOKUP_COUNTRY_CODE_TO_NAME));
@@ -537,7 +542,7 @@ public class JoinAndLookupBenchmark
     final CursorBuildSpec buildSpec = CursorBuildSpec.builder()
                                                      .setVirtualColumns(lookupVirtualColumns)
                                                      .build();
-    try (final CursorHolder cursorHolder = baseSegment.asCursorFactory().makeCursorHolder(buildSpec)) {
+    try (final CursorHolder cursorHolder = baseSegment.as(CursorFactory.class).makeCursorHolder(buildSpec)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, LOOKUP_COUNTRY_NUMBER_TO_NAME));
     }
@@ -553,7 +558,7 @@ public class JoinAndLookupBenchmark
                                                      .setVirtualColumns(lookupVirtualColumns)
                                                      .setFilter(filter)
                                                      .build();
-    try (final CursorHolder cursorHolder = baseSegment.asCursorFactory().makeCursorHolder(buildSpec)) {
+    try (final CursorHolder cursorHolder = baseSegment.as(CursorFactory.class).makeCursorHolder(buildSpec)) {
       final Cursor cursor = cursorHolder.asCursor();
       blackhole.consume(getLastValue(cursor, LOOKUP_COUNTRY_NUMBER_TO_NAME));
     }
