@@ -49,7 +49,6 @@ import org.apache.druid.java.util.common.concurrent.Execs;
 import org.apache.druid.java.util.common.guava.Comparators;
 import org.apache.druid.java.util.emitter.EmittingLogger;
 import org.apache.druid.metadata.MetadataSupervisorManager;
-import org.apache.druid.metadata.SqlSegmentsMetadataManager;
 import org.apache.druid.timeline.DataSegment;
 import org.joda.time.Duration;
 import org.joda.time.Interval;
@@ -77,7 +76,6 @@ public class MaterializedViewSupervisor implements Supervisor
 
   private final MetadataSupervisorManager metadataSupervisorManager;
   private final IndexerMetadataStorageCoordinator metadataStorageCoordinator;
-  private final SqlSegmentsMetadataManager sqlSegmentsMetadataManager;
   private final MaterializedViewSupervisorSpec spec;
   private final TaskMaster taskMaster;
   private final TaskStorage taskStorage;
@@ -104,7 +102,6 @@ public class MaterializedViewSupervisor implements Supervisor
       TaskMaster taskMaster,
       TaskStorage taskStorage,
       MetadataSupervisorManager metadataSupervisorManager,
-      SqlSegmentsMetadataManager sqlSegmentsMetadataManager,
       IndexerMetadataStorageCoordinator metadataStorageCoordinator,
       MaterializedViewTaskConfig config,
       MaterializedViewSupervisorSpec spec
@@ -113,7 +110,6 @@ public class MaterializedViewSupervisor implements Supervisor
     this.taskMaster = taskMaster;
     this.taskStorage = taskStorage;
     this.metadataStorageCoordinator = metadataStorageCoordinator;
-    this.sqlSegmentsMetadataManager = sqlSegmentsMetadataManager;
     this.metadataSupervisorManager = metadataSupervisorManager;
     this.config = config;
     this.spec = spec;
@@ -391,7 +387,7 @@ public class MaterializedViewSupervisor implements Supervisor
     // drop derivative segments which interval equals the interval in toDeleteBaseSegments 
     for (Interval interval : toDropInterval.keySet()) {
       for (DataSegment segment : derivativeSegments.get(interval)) {
-        sqlSegmentsMetadataManager.markSegmentAsUnused(segment.getId());
+        metadataStorageCoordinator.markSegmentAsUnused(segment.getId());
       }
     }
     // data of the latest interval will be built firstly.
@@ -500,7 +496,7 @@ public class MaterializedViewSupervisor implements Supervisor
   {
     log.info("Clear all metadata of dataSource %s", dataSource);
     metadataStorageCoordinator.deletePendingSegments(dataSource);
-    sqlSegmentsMetadataManager.markAsUnusedAllSegmentsInDataSource(dataSource);
+    metadataStorageCoordinator.markAllSegmentsAsUnused(dataSource);
     metadataStorageCoordinator.deleteDataSourceMetadata(dataSource);
   }
 
