@@ -20,7 +20,6 @@
 package org.apache.druid.query.aggregation.datasketches.tuple;
 
 import org.apache.datasketches.quantiles.DoublesSketch;
-import org.apache.druid.common.config.NullHandling;
 import org.apache.druid.initialization.DruidModule;
 import org.apache.druid.java.util.common.granularity.Granularities;
 import org.apache.druid.java.util.common.guava.Sequence;
@@ -663,10 +662,10 @@ public class ArrayOfDoublesSketchAggregationTest extends InitializedNullHandling
     List<ResultRow> results = seq.toList();
     Assert.assertEquals(1, results.size());
     ResultRow row = results.get(0);
-    Assert.assertEquals("sketch", NullHandling.replaceWithDefault() ? 40.0 : 30.0, (double) row.get(0), 0);
-    Assert.assertEquals("estimate", NullHandling.replaceWithDefault() ? 40.0 : 30.0, (double) row.get(1), 0);
-    Assert.assertEquals("union", NullHandling.replaceWithDefault() ? 40.0 : 30.0, (double) row.get(3), 0);
-    Assert.assertEquals("intersection", NullHandling.replaceWithDefault() ? 40.0 : 30.0, (double) row.get(4), 0);
+    Assert.assertEquals("sketch", 30.0, (double) row.get(0), 0);
+    Assert.assertEquals("estimate", 30.0, (double) row.get(1), 0);
+    Assert.assertEquals("union", 30.0, (double) row.get(3), 0);
+    Assert.assertEquals("intersection", 30.0, (double) row.get(4), 0);
     Assert.assertEquals("anotb", 0, (double) row.get(5), 0);
 
     Object meansObj = row.get(6); // means
@@ -675,20 +674,20 @@ public class ArrayOfDoublesSketchAggregationTest extends InitializedNullHandling
     Assert.assertEquals(3, means.length);
     Assert.assertEquals(1.0, means[0], 0);
     Assert.assertEquals(2.0, means[1], 0);
-    Assert.assertEquals(NullHandling.replaceWithDefault() ? 2.25 : 3.0, means[2], 0.1);
+    Assert.assertEquals(3.0, means[2], 0.1);
 
     Object obj = row.get(2); // quantiles-sketch
     Assert.assertTrue(obj instanceof DoublesSketch);
     DoublesSketch ds = (DoublesSketch) obj;
-    Assert.assertEquals(NullHandling.replaceWithDefault() ? 40 : 30, ds.getN());
+    Assert.assertEquals(30, ds.getN());
     Assert.assertEquals(2.0, ds.getMinItem(), 0);
     Assert.assertEquals(2.0, ds.getMaxItem(), 0);
 
     Object objSketch2 = row.get(7); // quantiles-sketch-with-nulls
     Assert.assertTrue(objSketch2 instanceof DoublesSketch);
     DoublesSketch ds2 = (DoublesSketch) objSketch2;
-    Assert.assertEquals(NullHandling.replaceWithDefault() ? 40 : 30, ds2.getN());
-    Assert.assertEquals(NullHandling.replaceWithDefault() ? 0.0 : 3.0, ds2.getMinItem(), 0);
+    Assert.assertEquals(30, ds2.getN());
+    Assert.assertEquals(3.0, ds2.getMinItem(), 0);
     Assert.assertEquals(3.0, ds2.getMaxItem(), 0);
   }
 
@@ -1207,12 +1206,12 @@ public class ArrayOfDoublesSketchAggregationTest extends InitializedNullHandling
     List<ResultRow> results = seq.toList();
     Assert.assertEquals(1, results.size());
     ResultRow row = results.get(0);
-    Assert.assertEquals("sketch", NullHandling.replaceWithDefault() ? 40.0 : 30.0, (double) row.get(0), 0);
+    Assert.assertEquals("sketch", 30.0, (double) row.get(0), 0);
     Assert.assertEquals("sketchNoNulls", 40.0, (double) row.get(1), 0);
-    Assert.assertEquals("estimate", NullHandling.replaceWithDefault() ? 40.0 : 30.0, (double) row.get(2), 0);
+    Assert.assertEquals("estimate", 30.0, (double) row.get(2), 0);
     Assert.assertEquals("estimateNoNulls", 40.0, (double) row.get(3), 0);
-    Assert.assertEquals("union", NullHandling.replaceWithDefault() ? 40.0 : 30.0, (double) row.get(5), 0);
-    Assert.assertEquals("intersection", NullHandling.replaceWithDefault() ? 40.0 : 30.0, (double) row.get(6), 0);
+    Assert.assertEquals("union", 30.0, (double) row.get(5), 0);
+    Assert.assertEquals("intersection", 30.0, (double) row.get(6), 0);
     Assert.assertEquals("anotb", 0, (double) row.get(7), 0);
 
     Object meansObj = row.get(8); // means
@@ -1221,20 +1220,20 @@ public class ArrayOfDoublesSketchAggregationTest extends InitializedNullHandling
     Assert.assertEquals(3, means.length);
     Assert.assertEquals(1.0, means[0], 0);
     Assert.assertEquals(2.0, means[1], 0);
-    Assert.assertEquals(NullHandling.replaceWithDefault() ? 2.25 : 3.0, means[2], 0.1);
+    Assert.assertEquals(3.0, means[2], 0.1);
 
     Object obj = row.get(4); // quantiles-sketch
     Assert.assertTrue(obj instanceof DoublesSketch);
     DoublesSketch ds = (DoublesSketch) obj;
-    Assert.assertEquals(NullHandling.replaceWithDefault() ? 40 : 30, ds.getN());
+    Assert.assertEquals(30, ds.getN());
     Assert.assertEquals(2.0, ds.getMinItem(), 0);
     Assert.assertEquals(2.0, ds.getMaxItem(), 0);
 
     Object objSketch2 = row.get(9); // quantiles-sketch-with-nulls
     Assert.assertTrue(objSketch2 instanceof DoublesSketch);
     DoublesSketch ds2 = (DoublesSketch) objSketch2;
-    Assert.assertEquals(NullHandling.replaceWithDefault() ? 40 : 30, ds2.getN());
-    Assert.assertEquals(NullHandling.replaceWithDefault() ? 0.0 : 3.0, ds2.getMinItem(), 0);
+    Assert.assertEquals(30, ds2.getN());
+    Assert.assertEquals(3.0, ds2.getMinItem(), 0);
     Assert.assertEquals(3.0, ds2.getMaxItem(), 0);
 
     Object objSketch3 = row.get(10); // quantiles-sketch-no-nulls
