@@ -22,6 +22,10 @@ package org.apache.druid.indexing.kafka.supervisor;
 import com.fasterxml.jackson.databind.InjectableValues;
 import com.fasterxml.jackson.databind.Module;
 import com.fasterxml.jackson.databind.ObjectMapper;
+import org.apache.druid.data.input.impl.JsonInputFormat;
+import org.apache.druid.error.DruidException;
+import org.apache.druid.error.DruidExceptionMatcher;
+import org.apache.druid.indexer.granularity.UniformGranularitySpec;
 import org.apache.druid.indexing.kafka.KafkaIndexTaskClientFactory;
 import org.apache.druid.indexing.kafka.KafkaIndexTaskModule;
 import org.apache.druid.indexing.overlord.IndexerMetadataStorageCoordinator;
@@ -29,16 +33,26 @@ import org.apache.druid.indexing.overlord.TaskMaster;
 import org.apache.druid.indexing.overlord.TaskStorage;
 import org.apache.druid.indexing.overlord.supervisor.SupervisorStateManagerConfig;
 import org.apache.druid.jackson.DefaultObjectMapper;
+import org.apache.druid.java.util.common.StringUtils;
+import org.apache.druid.java.util.common.granularity.Granularities;
+import org.apache.druid.java.util.common.parsers.JSONPathSpec;
 import org.apache.druid.java.util.emitter.service.ServiceEmitter;
 import org.apache.druid.java.util.metrics.DruidMonitorSchedulerConfig;
 import org.apache.druid.math.expr.ExprMacroTable;
+import org.apache.druid.metadata.TestSupervisorSpec;
+import org.apache.druid.query.aggregation.CountAggregatorFactory;
 import org.apache.druid.query.expression.LookupEnabledTestExprMacroTable;
 import org.apache.druid.segment.incremental.RowIngestionMetersFactory;
+import org.apache.druid.segment.indexing.DataSchema;
 import org.apache.druid.server.metrics.NoopServiceEmitter;
+import org.hamcrest.MatcherAssert;
 import org.junit.Assert;
 import org.junit.Test;
 
 import java.io.IOException;
+import java.util.Map;
+
+import static org.junit.Assert.assertThrows;
 
 public class KafkaSupervisorSpecTest
 {
@@ -578,4 +592,179 @@ public class KafkaSupervisorSpecTest
 
     Assert.assertFalse(runningSpec.isSuspended());
   }
+
+  @Test
+  public void test_validateSpecUpdateTo()
+  {
+    KafkaSupervisorSpec sourceSpec = getSpec("metrics", null);
+
+    // Proposed spec being non-kafka is not allowed
+    TestSupervisorSpec otherSpec = new TestSupervisorSpec("test", new Object());
+    MatcherAssert.assertThat(
+        assertThrows(DruidException.class, () -> sourceSpec.validateSpecUpdateTo(otherSpec)),
+        new DruidExceptionMatcher(
+            DruidException.Persona.USER,
+            DruidException.Category.INVALID_INPUT,
+            "invalidInput"
+        ).expectMessageIs(
+            StringUtils.format("Cannot change spec from type[%s] to type[%s]", sourceSpec.getClass().getSimpleName(), otherSpec.getClass().getSimpleName())
+        )
+    );
+
+    KafkaSupervisorSpec multiTopicProposedSpec = getSpec(null, "metrics-.*");
+    MatcherAssert.assertThat(
+        assertThrows(DruidException.class, () -> sourceSpec.validateSpecUpdateTo(multiTopicProposedSpec)),
+        new DruidExceptionMatcher(
+            DruidException.Persona.USER,
+            DruidException.Category.INVALID_INPUT,
+            "invalidInput"
+        ).expectMessageIs(
+             "Update of the input source stream from [(single-topic) metrics] to [(multi-topic) metrics-.*] is not supported for a running supervisor."
+             + "\nTo perform the update safely, follow these steps:"
+             + "\n(1) Suspend this supervisor, reset its offsets and then terminate it. "
+             + "\n(2) Create a new supervisor with the new input source stream."
+             + "\nNote that doing the reset can cause data duplication or loss if any topic used in the old supervisor is included in the new one too."
+         )
+    );
+
+    KafkaSupervisorSpec singleTopicNewStreamProposedSpec = getSpec("metricsNew", null);
+    MatcherAssert.assertThat(
+        assertThrows(DruidException.class, () -> sourceSpec.validateSpecUpdateTo(singleTopicNewStreamProposedSpec)),
+        new DruidExceptionMatcher(
+            DruidException.Persona.USER,
+            DruidException.Category.INVALID_INPUT,
+            "invalidInput"
+        ).expectMessageIs(
+            "Update of the input source stream from [metrics] to [metricsNew] is not supported for a running supervisor."
+            + "\nTo perform the update safely, follow these steps:"
+            + "\n(1) Suspend this supervisor, reset its offsets and then terminate it. "
+            + "\n(2) Create a new supervisor with the new input source stream."
+            + "\nNote that doing the reset can cause data duplication or loss if any topic used in the old supervisor is included in the new one too."
+        )
+    );
+
+    KafkaSupervisorSpec multiTopicMatchingSourceString = getSpec(null, "metrics");
+    MatcherAssert.assertThat(
+        assertThrows(DruidException.class, () -> sourceSpec.validateSpecUpdateTo(multiTopicMatchingSourceString)),
+        new DruidExceptionMatcher(
+            DruidException.Persona.USER,
+            DruidException.Category.INVALID_INPUT,
+            "invalidInput"
+        ).expectMessageIs(
+            "Update of the input source stream from [(single-topic) metrics] to [(multi-topic) metrics] is not supported for a running supervisor."
+            + "\nTo perform the update safely, follow these steps:"
+            + "\n(1) Suspend this supervisor, reset its offsets and then terminate it. "
+            + "\n(2) Create a new supervisor with the new input source stream."
+            + "\nNote that doing the reset can cause data duplication or loss if any topic used in the old supervisor is included in the new one too."
+        )
+    );
+
+    // test the inverse as well
+    MatcherAssert.assertThat(
+        assertThrows(DruidException.class, () -> multiTopicMatchingSourceString.validateSpecUpdateTo(sourceSpec)),
+        new DruidExceptionMatcher(
+            DruidException.Persona.USER,
+            DruidException.Category.INVALID_INPUT,
+            "invalidInput"
+        ).expectMessageIs(
+            "Update of the input source stream from [(multi-topic) metrics] to [(single-topic) metrics] is not supported for a running supervisor."
+            + "\nTo perform the update safely, follow these steps:"
+            + "\n(1) Suspend this supervisor, reset its offsets and then terminate it. "
+            + "\n(2) Create a new supervisor with the new input source stream."
+            + "\nNote that doing the reset can cause data duplication or loss if any topic used in the old supervisor is included in the new one too."
+        )
+    );
+
+    // Test valid spec update. This spec changes context vs the sourceSpec
+    KafkaSupervisorSpec validDestSpec = new KafkaSupervisorSpec(
+        null,
+        null,
+        DataSchema.builder().withDataSource("testDs").withAggregators(new CountAggregatorFactory("rows")).withGranularity(new UniformGranularitySpec(Granularities.DAY, Granularities.NONE, null)).build(),
+        null,
+        new KafkaSupervisorIOConfig(
+            "metrics",
+            null,
+            new JsonInputFormat(JSONPathSpec.DEFAULT, null, null, null, null),
+            null,
+            null,
+            null,
+            Map.of("bootstrap.servers", "localhost:9092"),
+            null,
+            null,
+            null,
+            null,
+            null,
+            true,
+            null,
+            null,
+            null,
+            null,
+            null,
+            null,
+            null,
+            false
+        ),
+        Map.of(
+            "key1",
+            "value1",
+            "key2",
+            "value2"
+        ),
+        null,
+        null,
+        null,
+        null,
+        null,
+        null,
+        null,
+        null,
+        null,
+        null
+    );
+    sourceSpec.validateSpecUpdateTo(validDestSpec);
+  }
+
+  private KafkaSupervisorSpec getSpec(String topic, String topicPattern)
+  {
+    return new KafkaSupervisorSpec(
+      null,
+      null,
+      DataSchema.builder().withDataSource("testDs").withAggregators(new CountAggregatorFactory("rows")).withGranularity(new UniformGranularitySpec(Granularities.DAY, Granularities.NONE, null)).build(),
+      null,
+      new KafkaSupervisorIOConfig(
+          topic,
+          topicPattern,
+          new JsonInputFormat(JSONPathSpec.DEFAULT, null, null, null, null),
+          null,
+          null,
+          null,
+          Map.of("bootstrap.servers", "localhost:9092"),
+          null,
+          null,
+          null,
+          null,
+          null,
+          true,
+          null,
+          null,
+          null,
+          null,
+          null,
+          null,
+          null,
+          false
+      ),
+      null,
+      null,
+      null,
+      null,
+      null,
+      null,
+      null,
+      null,
+      null,
+      null,
+      null
+  );
+  }
 }
