@@ -41,6 +41,7 @@ import org.apache.druid.frame.util.DurableStorageUtils;
 import org.apache.druid.java.util.common.FileUtils;
 import org.apache.druid.java.util.common.ISE;
 import org.apache.druid.java.util.common.RE;
+import org.apache.druid.java.util.common.Stopwatch;
 import org.apache.druid.java.util.common.StringUtils;
 import org.apache.druid.java.util.common.io.Closer;
 import org.apache.druid.java.util.common.logger.Logger;
@@ -58,7 +59,7 @@ import org.apache.druid.msq.indexing.error.MSQWarningReportSimplePublisher;
 import org.apache.druid.msq.indexing.error.MSQWarnings;
 import org.apache.druid.msq.input.InputSlices;
 import org.apache.druid.msq.input.stage.ReadablePartition;
-import org.apache.druid.msq.kernel.FrameContext;
+import org.apache.druid.msq.kernel.QueryDefinition;
 import org.apache.druid.msq.kernel.StageDefinition;
 import org.apache.druid.msq.kernel.StageId;
 import org.apache.druid.msq.kernel.WorkOrder;
@@ -73,19 +74,24 @@ import org.apache.druid.msq.shuffle.output.StageOutputHolder;
 import org.apache.druid.msq.statistics.ClusterByStatisticsSnapshot;
 import org.apache.druid.msq.statistics.PartialKeyStatisticsInformation;
 import org.apache.druid.msq.util.DecoratedExecutorService;
+import org.apache.druid.msq.util.MSQMetricUtils;
 import org.apache.druid.msq.util.MultiStageQueryContext;
+import org.apache.druid.query.BaseQuery;
+import org.apache.druid.query.DefaultQueryMetrics;
+import org.apache.druid.query.DruidMetrics;
 import org.apache.druid.query.PrioritizedCallable;
 import org.apache.druid.query.PrioritizedRunnable;
 import org.apache.druid.query.QueryContext;
 import org.apache.druid.query.QueryProcessingPool;
 import org.apache.druid.server.DruidNode;
-import org.apache.druid.utils.CloseableUtils;
+import org.joda.time.Interval;
 
 import javax.annotation.Nullable;
 import java.io.Closeable;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.Comparator;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
@@ -96,7 +102,6 @@ import java.util.concurrent.Callable;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.LinkedBlockingDeque;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.function.Consumer;
 import java.util.function.Supplier;
 import java.util.stream.Collectors;
@@ -129,21 +134,11 @@ public class WorkerImpl implements Worker
    */
   private final ConcurrentHashMap<IntObjectPair<StageId>, CounterTracker> stageCounters = new ConcurrentHashMap<>();
 
-  /**
-   * Atomic that is set to true when {@link #run()} starts (or when {@link #stop()} is called before {@link #run()}).
-   */
-  private final AtomicBoolean didRun = new AtomicBoolean();
-
   /**
    * Future that resolves when {@link #run()} completes.
    */
   private final SettableFuture<Void> runFuture = SettableFuture.create();
 
-  /**
-   * Set once in {@link #run} and never reassigned. This is in a field so {@link #doCancel()} can close it.
-   */
-  private volatile ControllerClient controllerClient;
-
   /**
    * Set once in {@link #runInternal} and never reassigned. Used by processing threads so we can contact other workers
    * during a shuffle.
@@ -151,9 +146,9 @@ public class WorkerImpl implements Worker
   private volatile WorkerClient workerClient;
 
   /**
-   * Set to false by {@link #controllerFailed()} as a way of enticing the {@link #runInternal} method to exit promptly.
+   * Stores dimensions for metric emissions.
    */
-  private volatile boolean controllerAlive = true;
+  private final Map<String, Object> queryMetricDimensions = new HashMap<>();
 
   public WorkerImpl(@Nullable final MSQWorkerTask task, final WorkerContext context)
   {
@@ -171,13 +166,10 @@ public class WorkerImpl implements Worker
   @Override
   public void run()
   {
-    if (!didRun.compareAndSet(false, true)) {
-      throw new ISE("already run");
-    }
-
     try (final Closer closer = Closer.create()) {
+      final Stopwatch stopwatch = Stopwatch.createStarted();
       final KernelHolders kernelHolders = KernelHolders.create(context, closer);
-      controllerClient = kernelHolders.getControllerClient();
+      final ControllerClient controllerClient = kernelHolders.getControllerClient();
 
       Throwable t = null;
       Optional<MSQErrorReport> maybeErrorReport;
@@ -197,12 +189,19 @@ public class WorkerImpl implements Worker
         );
       }
 
+      // Report query metrics
+      reportQueryMetrics(maybeErrorReport.isEmpty(), stopwatch.millisElapsed());
+
       if (maybeErrorReport.isPresent()) {
         final MSQErrorReport errorReport = maybeErrorReport.get();
         final String logMessage = MSQTasks.errorReportToLogMessage(errorReport);
         log.warn("%s", logMessage);
 
-        if (controllerAlive) {
+        // Inform controller of any errors that occur, unless we were canceled. This prevents attempting to contact
+        // the controller after cancellation due to controller failure. For situations where the worker is canceled
+        // but the controller is still alive, cancellation propagates to the controller in other ways. (For example,
+        // with single-shot tasks, the controller notices the task fails.)
+        if (!(errorReport.getFault() instanceof CanceledFault)) {
           controllerClient.postWorkerError(errorReport);
         }
 
@@ -222,6 +221,30 @@ public class WorkerImpl implements Worker
     }
   }
 
+  @SuppressWarnings("unchecked")
+  private void reportQueryMetrics(boolean success, long time)
+  {
+    long cpuTimeNs = 0L;
+    for (final CounterTracker tracker : stageCounters.values()) {
+      cpuTimeNs += tracker.totalCpu();
+    }
+
+    final Set<String> datasources = (Set<String>) queryMetricDimensions.get(DruidMetrics.DATASOURCE);
+    final Set<Interval> intervals = (Set<Interval>) queryMetricDimensions.get(DruidMetrics.INTERVAL);
+
+    final MSQMetriceEventBuilder metricBuilder = new MSQMetriceEventBuilder();
+    metricBuilder.setDimension(DruidMetrics.DATASOURCE, DefaultQueryMetrics.getTableNamesAsString(datasources))
+                 .setDimension(DruidMetrics.INTERVAL, DefaultQueryMetrics.getIntervalsAsStringArray(intervals))
+                 .setDimension(DruidMetrics.DURATION, BaseQuery.calculateDuration(intervals))
+                 .setDimension(DruidMetrics.SUCCESS, success)
+                 .setMetric("query/time", time);
+
+    context.emitMetric(metricBuilder);
+
+    metricBuilder.setMetric("query/cpu/time", TimeUnit.NANOSECONDS.toMicros(cpuTimeNs));
+    context.emitMetric(metricBuilder);
+  }
+
   /**
    * Runs worker logic. Returns an empty Optional on success. On failure, returns an error report for errors that
    * happened in other threads; throws exceptions for errors that happened in the main worker loop.
@@ -230,7 +253,6 @@ public class WorkerImpl implements Worker
       throws Exception
   {
     context.registerWorker(this, workerCloser);
-    workerCloser.register(context.dataServerQueryHandlerFactory());
     this.workerClient = workerCloser.register(new ExceptionWrappingWorkerClient(context.makeWorkerClient()));
     final FrameProcessorExecutor workerExec = new FrameProcessorExecutor(makeProcessingPool());
 
@@ -275,7 +297,7 @@ public class WorkerImpl implements Worker
             && kernelHolders.runningKernelCount() < context.maxConcurrentStages()) {
           handleNewWorkOrder(
               kernelHolder,
-              controllerClient,
+              kernelHolders.controllerClient,
               workerExec,
               criticalWarningCodes,
               maxVerboseParseExceptions
@@ -285,7 +307,7 @@ public class WorkerImpl implements Worker
         }
 
         if (kernel.getPhase() == WorkerStagePhase.READING_INPUT
-            && handleReadingInput(kernelHolder, controllerClient)) {
+            && handleReadingInput(kernelHolder, kernelHolders.controllerClient)) {
           didSomething = true;
           logKernelStatus(kernelHolders.getAllKernels());
         }
@@ -297,7 +319,7 @@ public class WorkerImpl implements Worker
         }
 
         if (kernel.getPhase() == WorkerStagePhase.RESULTS_COMPLETE
-            && handleResultsReady(kernelHolder, controllerClient)) {
+            && handleResultsReady(kernelHolder, kernelHolders.controllerClient)) {
           didSomething = true;
           logKernelStatus(kernelHolders.getAllKernels());
         }
@@ -367,6 +389,7 @@ public class WorkerImpl implements Worker
     final WorkerStageKernel kernel = kernelHolder.kernel;
     final WorkOrder workOrder = kernel.getWorkOrder();
     final StageDefinition stageDefinition = workOrder.getStageDefinition();
+    updateMetricDimensions(workOrder.getQueryDefinition());
     final String cancellationId = cancellationIdFor(stageDefinition.getId(), workOrder.getWorkerNumber());
 
     log.info(
@@ -400,9 +423,7 @@ public class WorkerImpl implements Worker
         cancellationId,
         context,
         frameContext,
-        makeRunWorkOrderListener(workOrder, controllerClient, criticalWarningCodes, maxVerboseParseExceptions),
-        MultiStageQueryContext.isReindex(workOrder.getWorkerContext()),
-        MultiStageQueryContext.removeNullBytes(workOrder.getWorkerContext())
+        makeRunWorkOrderListener(workOrder, controllerClient, criticalWarningCodes, maxVerboseParseExceptions)
     );
 
     // Set up processorCloser (called when processing is done).
@@ -414,6 +435,17 @@ public class WorkerImpl implements Worker
     kernelHolder.partitionBoundariesFuture = runWorkOrder.getStagePartitionBoundariesFuture();
   }
 
+  @SuppressWarnings("unchecked")
+  private void updateMetricDimensions(QueryDefinition queryDef)
+  {
+    Set<String> datasourceDim = (Set<String>) queryMetricDimensions.computeIfAbsent(DruidMetrics.DATASOURCE, s -> new HashSet<>());
+    Set<Interval> intervalDim = (Set<Interval>) queryMetricDimensions.computeIfAbsent(DruidMetrics.INTERVAL, s -> new HashSet<>());
+    for (StageDefinition stageDef : queryDef.getStageDefinitions()) {
+      datasourceDim.addAll(stageDef.getDatasources());
+      intervalDim.addAll(MSQMetricUtils.getIntervals(stageDef));
+    }
+  }
+
   /**
    * Handle a kernel in state {@link WorkerStagePhase#READING_INPUT}.
    *
@@ -429,17 +461,15 @@ public class WorkerImpl implements Worker
   {
     final WorkerStageKernel kernel = kernelHolder.kernel;
     if (kernel.hasResultKeyStatisticsSnapshot()) {
-      if (controllerAlive) {
-        PartialKeyStatisticsInformation partialKeyStatisticsInformation =
-            kernel.getResultKeyStatisticsSnapshot()
-                  .partialKeyStatistics();
+      PartialKeyStatisticsInformation partialKeyStatisticsInformation =
+          kernel.getResultKeyStatisticsSnapshot()
+                .partialKeyStatistics();
 
-        controllerClient.postPartialKeyStatistics(
-            kernel.getStageDefinition().getId(),
-            kernel.getWorkOrder().getWorkerNumber(),
-            partialKeyStatisticsInformation
-        );
-      }
+      controllerClient.postPartialKeyStatistics(
+          kernel.getStageDefinition().getId(),
+          kernel.getWorkOrder().getWorkerNumber(),
+          partialKeyStatisticsInformation
+      );
 
       kernel.startPreshuffleWaitingForResultPartitionBoundaries();
       return true;
@@ -448,7 +478,7 @@ public class WorkerImpl implements Worker
                && !kernel.getStageDefinition().mustGatherResultKeyStatistics()) {
       // Skip postDoneReadingInput when context.maxConcurrentStages() == 1, for backwards compatibility.
       // See Javadoc comment on ControllerClient#postDoneReadingInput.
-      if (controllerAlive && context.maxConcurrentStages() > 1) {
+      if (context.maxConcurrentStages() > 1) {
         controllerClient.postDoneReadingInput(
             kernel.getStageDefinition().getId(),
             kernel.getWorkOrder().getWorkerNumber()
@@ -494,7 +524,7 @@ public class WorkerImpl implements Worker
     final boolean didNotPostYet =
         kernel.addPostedResultsComplete(kernel.getStageDefinition().getId(), kernel.getWorkOrder().getWorkerNumber());
 
-    if (controllerAlive && didNotPostYet) {
+    if (didNotPostYet) {
       controllerClient.postResultsComplete(
           kernel.getStageDefinition().getId(),
           kernel.getWorkOrder().getWorkerNumber(),
@@ -518,37 +548,6 @@ public class WorkerImpl implements Worker
     }
   }
 
-  @Override
-  public void stop()
-  {
-    // stopGracefully() is called when the containing process is terminated, or when the task is canceled.
-    log.info("Worker id[%s] canceled.", context.workerId());
-
-    if (didRun.compareAndSet(false, true)) {
-      // run() hasn't been called yet. Set runFuture so awaitStop() still works.
-      runFuture.set(null);
-    } else {
-      doCancel();
-    }
-  }
-
-  @Override
-  public void awaitStop()
-  {
-    FutureUtils.getUnchecked(runFuture, false);
-  }
-
-  @Override
-  public void controllerFailed()
-  {
-    log.info(
-        "Controller task[%s] for worker[%s] failed. Canceling.",
-        task != null ? task.getControllerTaskId() : null,
-        id()
-    );
-    doCancel();
-  }
-
   @Override
   public ListenableFuture<InputStream> readStageOutput(
       final StageId stageId,
@@ -689,6 +688,14 @@ public class WorkerImpl implements Worker
     return retVal;
   }
 
+  /**
+   * Returns the context used to create this worker.
+   */
+  public WorkerContext getWorkerContext()
+  {
+    return context;
+  }
+
   /**
    * Create a {@link RunWorkOrderListener} for {@link RunWorkOrder} that hooks back into the {@link KernelHolders}
    * in the main loop.
@@ -912,7 +919,7 @@ public class WorkerImpl implements Worker
   {
     final CounterSnapshotsTree snapshotsTree = getCounters();
 
-    if (controllerAlive && !snapshotsTree.isEmpty()) {
+    if (!snapshotsTree.isEmpty()) {
       controllerClient.postCounters(id(), snapshotsTree);
     }
   }
@@ -976,36 +983,6 @@ public class WorkerImpl implements Worker
     return StringUtils.format("msq-worker[%s_%s]", stageId, workerNumber);
   }
 
-  /**
-   * Called by {@link #stop()} (task canceled, or containing process shut down) and
-   * {@link #controllerFailed()}.
-   */
-  private void doCancel()
-  {
-    // Set controllerAlive = false so we don't try to contact the controller after being canceled. If it canceled us,
-    // it doesn't need to know that we were canceled. If we were canceled by something else, the controller will
-    // detect this as part of its monitoring of workers.
-    controllerAlive = false;
-
-    // Close controller client to cancel any currently in-flight calls to the controller.
-    if (controllerClient != null) {
-      controllerClient.close();
-    }
-
-    // Close worker client to cancel any currently in-flight calls to other workers.
-    if (workerClient != null) {
-      CloseableUtils.closeAndSuppressExceptions(workerClient, e -> log.warn("Failed to close workerClient"));
-    }
-
-    // Clear the main loop event queue, then throw a CanceledFault into the loop to exit it promptly.
-    kernelManipulationQueue.clear();
-    kernelManipulationQueue.add(
-        kernel -> {
-          throw new MSQException(CanceledFault.INSTANCE);
-        }
-    );
-  }
-
   /**
    * Returns a work order based on the provided "originalWorkOrder", but where {@link WorkOrder#hasOutputChannelMode()}
    * and {@link WorkOrder#hasWorkerContext()} are both true. If the original work order didn't have those fields, they
