@@ -33,25 +33,25 @@ import org.apache.druid.msq.exec.DataServerQueryHandler;
 import org.apache.druid.msq.input.ReadableInput;
 import org.apache.druid.msq.input.table.SegmentWithDescriptor;
 import org.apache.druid.msq.input.table.SegmentsInputSlice;
-import org.apache.druid.segment.ReferenceCountingSegment;
+import org.apache.druid.segment.ReferenceCountedSegmentProvider;
 import org.apache.druid.segment.Segment;
-import org.apache.druid.segment.SegmentReference;
+import org.apache.druid.segment.SegmentMapFunction;
 
 import java.io.IOException;
 import java.util.Collections;
 import java.util.List;
-import java.util.function.Function;
+import java.util.Optional;
 
 public abstract class BaseLeafFrameProcessor implements FrameProcessor<Object>
 {
   private final ReadableInput baseInput;
   private final ResourceHolder<WritableFrameChannel> outputChannelHolder;
   private final ResourceHolder<FrameWriterFactory> frameWriterFactoryHolder;
-  private final Function<SegmentReference, SegmentReference> segmentMapFn;
+  private final SegmentMapFunction segmentMapFn;
 
   protected BaseLeafFrameProcessor(
       final ReadableInput baseInput,
-      final Function<SegmentReference, SegmentReference> segmentMapFn,
+      final SegmentMapFunction segmentMapFn,
       final ResourceHolder<WritableFrameChannel> outputChannelHolder,
       final ResourceHolder<FrameWriterFactory> frameWriterFactoryHolder
   )
@@ -130,8 +130,10 @@ public abstract class BaseLeafFrameProcessor implements FrameProcessor<Object>
    * Helper intended to be used by subclasses. Applies {@link #segmentMapFn}, which applies broadcast joins
    * if applicable to this query.
    */
-  protected SegmentReference mapSegment(final Segment segment)
+  protected Optional<Segment> mapSegment(final Segment segment)
   {
-    return segmentMapFn.apply(ReferenceCountingSegment.wrapRootGenerationSegment(segment));
+    // we use wrapUnmanaged here because segment reference tracking and lifecycle management happens elsewhere, so all
+    // we need to be able to do here is apply a segment map function since we don't care about the provider
+    return segmentMapFn.apply(ReferenceCountedSegmentProvider.wrapUnmanaged(segment));
   }
 }
