@@ -25,6 +25,7 @@ import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Iterables;
 import com.google.common.util.concurrent.Futures;
+import org.apache.druid.indexer.TaskState;
 import org.apache.druid.indexer.report.TaskReport;
 import org.apache.druid.indexing.common.TaskLockType;
 import org.apache.druid.java.util.common.DateTimes;
@@ -33,6 +34,7 @@ import org.apache.druid.java.util.common.ISE;
 import org.apache.druid.java.util.common.StringUtils;
 import org.apache.druid.java.util.common.concurrent.Execs;
 import org.apache.druid.java.util.common.jackson.JacksonUtils;
+import org.apache.druid.java.util.metrics.StubServiceEmitter;
 import org.apache.druid.msq.dart.controller.ControllerHolder;
 import org.apache.druid.msq.dart.controller.DartControllerRegistry;
 import org.apache.druid.msq.dart.controller.sql.DartQueryMaker;
@@ -41,16 +43,22 @@ import org.apache.druid.msq.dart.controller.sql.DartSqlClients;
 import org.apache.druid.msq.dart.controller.sql.DartSqlEngine;
 import org.apache.druid.msq.dart.guice.DartControllerConfig;
 import org.apache.druid.msq.exec.Controller;
+import org.apache.druid.msq.exec.ControllerContext;
+import org.apache.druid.msq.indexing.MSQSpec;
 import org.apache.druid.msq.indexing.error.CanceledFault;
 import org.apache.druid.msq.indexing.error.InvalidNullByteFault;
 import org.apache.druid.msq.indexing.error.MSQErrorReport;
 import org.apache.druid.msq.indexing.error.MSQFaultUtils;
+import org.apache.druid.msq.indexing.report.MSQStatusReport;
 import org.apache.druid.msq.indexing.report.MSQTaskReport;
+import org.apache.druid.msq.kernel.controller.ControllerQueryKernelConfig;
+import org.apache.druid.msq.sql.DartQueryKitSpecFactory;
 import org.apache.druid.msq.test.MSQTestBase;
 import org.apache.druid.msq.test.MSQTestControllerContext;
 import org.apache.druid.query.DefaultQueryConfig;
 import org.apache.druid.query.QueryContext;
 import org.apache.druid.query.QueryContexts;
+import org.apache.druid.query.policy.NoopPolicyEnforcer;
 import org.apache.druid.server.DruidNode;
 import org.apache.druid.server.QueryStackTests;
 import org.apache.druid.server.ResponseContextConfig;
@@ -63,7 +71,6 @@ import org.apache.druid.server.security.AuthConfig;
 import org.apache.druid.server.security.AuthenticationResult;
 import org.apache.druid.server.security.ForbiddenException;
 import org.apache.druid.sql.SqlLifecycleManager;
-import org.apache.druid.sql.SqlStatementFactory;
 import org.apache.druid.sql.SqlToolbox;
 import org.apache.druid.sql.calcite.planner.CalciteRulesManager;
 import org.apache.druid.sql.calcite.planner.CatalogResolver;
@@ -73,10 +80,16 @@ import org.apache.druid.sql.calcite.schema.DruidSchemaCatalog;
 import org.apache.druid.sql.calcite.schema.NoopDruidSchemaManager;
 import org.apache.druid.sql.calcite.util.CalciteTests;
 import org.apache.druid.sql.calcite.util.QueryFrameworkUtils;
+import org.apache.druid.sql.calcite.util.TestTimelineServerView;
 import org.apache.druid.sql.calcite.view.NoopViewManager;
 import org.apache.druid.sql.hook.DruidHookDispatcher;
+import org.apache.druid.sql.http.EngineInfo;
+import org.apache.druid.sql.http.GetQueriesResponse;
 import org.apache.druid.sql.http.ResultFormat;
+import org.apache.druid.sql.http.SqlEngineRegistry;
 import org.apache.druid.sql.http.SqlQuery;
+import org.apache.druid.sql.http.SqlResource;
+import org.apache.druid.sql.http.SupportedEnginesResponse;
 import org.hamcrest.CoreMatchers;
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.Assertions;
@@ -92,6 +105,7 @@ import java.io.IOException;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.UUID;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.ExecutorService;
@@ -101,7 +115,7 @@ import java.util.concurrent.TimeUnit;
 import static org.hamcrest.MatcherAssert.assertThat;
 
 /**
- * Functional test of {@link DartSqlResource}, {@link DartSqlEngine}, and {@link DartQueryMaker}.
+ * Functional test of {@link SqlResource}, {@link DartSqlEngine}, and {@link DartQueryMaker}.
  * Other classes are mocked when possible.
  */
 public class DartSqlResourceTest extends MSQTestBase
@@ -130,21 +144,22 @@ public class DartSqlResourceTest extends MSQTestBase
 
   // Objects created in setUp() below this line.
 
-  private DartSqlResource sqlResource;
+  private SqlResource sqlResource;
   private DartControllerRegistry controllerRegistry;
   private ExecutorService controllerExecutor;
   private AutoCloseable mockCloser;
+  private final StubServiceEmitter serviceEmitter = new StubServiceEmitter();
 
   // Mocks below this line.
 
   /**
-   * Mock for {@link DartSqlClients}, which is used in tests of {@link DartSqlResource#doGetRunningQueries}.
+   * Mock for {@link DartSqlClients}, which is used in tests of {@link SqlResource#doGetRunningQueries}.
    */
   @Mock
   private DartSqlClients dartSqlClients;
 
   /**
-   * Mock for {@link DartSqlClient}, which is used in tests of {@link DartSqlResource#doGetRunningQueries}.
+   * Mock for {@link DartSqlClient}, which is used in tests of {@link SqlResource#doGetRunningQueries}.
    */
   @Mock
   private DartSqlClient dartSqlClient;
@@ -166,32 +181,6 @@ public class DartSqlResourceTest extends MSQTestBase
   {
     mockCloser = MockitoAnnotations.openMocks(this);
 
-    final DartSqlEngine engine = new DartSqlEngine(
-        queryId -> new MSQTestControllerContext(
-            objectMapper,
-            injector,
-            null /* not used in this test */,
-            workerMemoryParameters,
-            loadedSegmentsMetadata,
-            TaskLockType.APPEND,
-            QueryContext.empty()
-        ),
-        controllerRegistry = new DartControllerRegistry()
-        {
-          @Override
-          public void register(ControllerHolder holder)
-          {
-            super.register(holder);
-            controllerRegistered.countDown();
-          }
-        },
-        objectMapper.convertValue(ImmutableMap.of(), DartControllerConfig.class),
-        controllerExecutor = Execs.multiThreaded(
-            MAX_CONTROLLERS,
-            StringUtils.encodeForFormat(getClass().getSimpleName() + "-controller-exec")
-        )
-    );
-
     final DruidSchemaCatalog rootSchema = QueryFrameworkUtils.createMockRootSchema(
         CalciteTests.INJECTOR,
         queryFramework().conglomerate(),
@@ -215,31 +204,75 @@ public class DartSqlResourceTest extends MSQTestBase
         CalciteTests.createJoinableFactoryWrapper(),
         CatalogResolver.NULL_RESOLVER,
         new AuthConfig(),
+        NoopPolicyEnforcer.instance(),
         new DruidHookDispatcher()
     );
 
     final SqlLifecycleManager lifecycleManager = new SqlLifecycleManager();
     final SqlToolbox toolbox = new SqlToolbox(
-        engine,
+        null,
         plannerFactory,
-        new NoopServiceEmitter(),
-        new NoopRequestLogger(),
+        NoopServiceEmitter.instance(),
+        NoopRequestLogger.instance(),
         QueryStackTests.DEFAULT_NOOP_SCHEDULER,
         new DefaultQueryConfig(ImmutableMap.of()),
         lifecycleManager
     );
 
-    sqlResource = new DartSqlResource(
+
+    final DartSqlEngine engine = new DartSqlEngine(
+        new MSQTestControllerContext(
+            "did2",
+            objectMapper,
+            injector,
+            null /* not used in this test */,
+            workerMemoryParameters,
+            loadedSegmentsMetadata,
+            TaskLockType.APPEND,
+            QueryContext.empty(),
+            serviceEmitter
+        ) {
+          @Override
+          public String queryId()
+          {
+            return getQueryContext().getString(QueryContexts.CTX_DART_QUERY_ID);
+          }
+
+          @Override
+          public ControllerQueryKernelConfig queryKernelConfig(MSQSpec querySpec)
+          {
+            return super.queryKernelConfig(querySpec).toBuilder().workerIds(ImmutableList.of("some")).build();
+          }
+        },
+        controllerRegistry = new DartControllerRegistry()
+        {
+          @Override
+          public void register(ControllerHolder holder)
+          {
+            super.register(holder);
+            controllerRegistered.countDown();
+          }
+        },
+        objectMapper.convertValue(ImmutableMap.of(), DartControllerConfig.class),
+        controllerExecutor = Execs.multiThreaded(
+            MAX_CONTROLLERS,
+            StringUtils.encodeForFormat(getClass().getSimpleName() + "-controller-exec")
+        ),
+        new DartQueryKitSpecFactory(new TestTimelineServerView(Collections.emptyList())),
+        new ServerConfig(),
+        new DefaultQueryConfig(ImmutableMap.of("foo", "bar")),
+        toolbox,
+        dartSqlClients
+    );
+
+    sqlResource = new SqlResource(
         objectMapper,
         CalciteTests.TEST_AUTHORIZER_MAPPER,
-        new SqlStatementFactory(toolbox),
-        controllerRegistry,
-        lifecycleManager,
-        dartSqlClients,
         new ServerConfig() /* currently only used for error transform strategy */,
+        lifecycleManager,
+        new SqlEngineRegistry(Set.of(engine)),
         ResponseContextConfig.newConfig(false),
-        SELF_NODE,
-        new DefaultQueryConfig(ImmutableMap.of("foo", "bar"))
+        SELF_NODE
     );
 
     // Setup mocks
@@ -266,12 +299,13 @@ public class DartSqlResourceTest extends MSQTestBase
   @Test
   public void test_getEnabled()
   {
-    final Response response = sqlResource.doGetEnabled(httpServletRequest);
-    Assertions.assertEquals(Response.Status.OK.getStatusCode(), response.getStatus());
+    Response response = sqlResource.getSupportedEngines(httpServletRequest);
+    Set<EngineInfo> supportedEngines = ((SupportedEnginesResponse) response.getEntity()).getEngines();
+    Assertions.assertTrue(supportedEngines.contains(new EngineInfo(DartSqlEngine.NAME)));
   }
 
   /**
-   * Test where a superuser calls {@link DartSqlResource#doGetRunningQueries} with selfOnly enabled.
+   * Test where a superuser calls {@link SqlResource#doGetRunningQueries} with selfOnly enabled.
    */
   @Test
   public void test_getRunningQueries_selfOnly_superUser()
@@ -283,7 +317,7 @@ public class DartSqlResourceTest extends MSQTestBase
 
     Assertions.assertEquals(
         new GetQueriesResponse(Collections.singletonList(DartQueryInfo.fromControllerHolder(holder))),
-        sqlResource.doGetRunningQueries("", httpServletRequest)
+        sqlResource.doGetRunningQueries("", httpServletRequest).getEntity()
     );
 
     controllerRegistry.deregister(holder);
@@ -291,7 +325,7 @@ public class DartSqlResourceTest extends MSQTestBase
 
   /**
    * Test where {@link #REGULAR_USER_NAME} and {@link #DIFFERENT_REGULAR_USER_NAME} issue queries, and
-   * {@link #REGULAR_USER_NAME} calls {@link DartSqlResource#doGetRunningQueries} with selfOnly enabled.
+   * {@link #REGULAR_USER_NAME} calls {@link SqlResource#doGetRunningQueries} with selfOnly enabled.
    */
   @Test
   public void test_getRunningQueries_selfOnly_regularUser()
@@ -307,7 +341,7 @@ public class DartSqlResourceTest extends MSQTestBase
     Assertions.assertEquals(
         new GetQueriesResponse(
             Collections.singletonList(DartQueryInfo.fromControllerHolder(holder).withoutAuthenticationResult())),
-        sqlResource.doGetRunningQueries("", httpServletRequest)
+        sqlResource.doGetRunningQueries("", httpServletRequest).getEntity()
     );
 
     controllerRegistry.deregister(holder);
@@ -315,7 +349,7 @@ public class DartSqlResourceTest extends MSQTestBase
   }
 
   /**
-   * Test where a superuser calls {@link DartSqlResource#doGetRunningQueries} with selfOnly disabled.
+   * Test where a superuser calls {@link SqlResource#doGetRunningQueries} with selfOnly disabled.
    */
   @Test
   public void test_getRunningQueries_global_superUser()
@@ -348,14 +382,14 @@ public class DartSqlResourceTest extends MSQTestBase
                 remoteQueryInfo
             )
         ),
-        sqlResource.doGetRunningQueries(null, httpServletRequest)
+        sqlResource.doGetRunningQueries(null, httpServletRequest).getEntity()
     );
 
     controllerRegistry.deregister(localHolder);
   }
 
   /**
-   * Test where a superuser calls {@link DartSqlResource#doGetRunningQueries} with selfOnly disabled, and where the
+   * Test where a superuser calls {@link SqlResource#doGetRunningQueries} with selfOnly disabled, and where the
    * remote server has a problem.
    */
   @Test
@@ -375,7 +409,7 @@ public class DartSqlResourceTest extends MSQTestBase
     // were able to fetch.)
     Assertions.assertEquals(
         new GetQueriesResponse(ImmutableList.of(DartQueryInfo.fromControllerHolder(localHolder))),
-        sqlResource.doGetRunningQueries(null, httpServletRequest)
+        sqlResource.doGetRunningQueries(null, httpServletRequest).getEntity()
     );
 
     controllerRegistry.deregister(localHolder);
@@ -383,7 +417,7 @@ public class DartSqlResourceTest extends MSQTestBase
 
   /**
    * Test where {@link #REGULAR_USER_NAME} and {@link #DIFFERENT_REGULAR_USER_NAME} issue queries, and
-   * {@link #REGULAR_USER_NAME} calls {@link DartSqlResource#doGetRunningQueries} with selfOnly disabled.
+   * {@link #REGULAR_USER_NAME} calls {@link SqlResource#doGetRunningQueries} with selfOnly disabled.
    */
   @Test
   public void test_getRunningQueries_global_regularUser()
@@ -412,7 +446,7 @@ public class DartSqlResourceTest extends MSQTestBase
     Assertions.assertEquals(
         new GetQueriesResponse(
             ImmutableList.of(DartQueryInfo.fromControllerHolder(localHolder).withoutAuthenticationResult())),
-        sqlResource.doGetRunningQueries(null, httpServletRequest)
+        sqlResource.doGetRunningQueries(null, httpServletRequest).getEntity()
     );
 
     controllerRegistry.deregister(localHolder);
@@ -420,7 +454,7 @@ public class DartSqlResourceTest extends MSQTestBase
 
   /**
    * Test where {@link #REGULAR_USER_NAME} and {@link #DIFFERENT_REGULAR_USER_NAME} issue queries, and
-   * {@link #DIFFERENT_REGULAR_USER_NAME} calls {@link DartSqlResource#doGetRunningQueries} with selfOnly disabled.
+   * {@link #DIFFERENT_REGULAR_USER_NAME} calls {@link SqlResource#doGetRunningQueries} with selfOnly disabled.
    */
   @Test
   public void test_getRunningQueries_global_differentRegularUser()
@@ -448,7 +482,7 @@ public class DartSqlResourceTest extends MSQTestBase
     // The endpoint returns only the query issued by DIFFERENT_REGULAR_USER_NAME.
     Assertions.assertEquals(
         new GetQueriesResponse(ImmutableList.of(remoteQueryInfo.withoutAuthenticationResult())),
-        sqlResource.doGetRunningQueries(null, httpServletRequest)
+        sqlResource.doGetRunningQueries(null, httpServletRequest).getEntity()
     );
 
     controllerRegistry.deregister(holder);
@@ -472,7 +506,7 @@ public class DartSqlResourceTest extends MSQTestBase
         false,
         false,
         false,
-        Collections.emptyMap(),
+        Map.of(QueryContexts.ENGINE, DartSqlEngine.NAME),
         Collections.emptyList()
     );
 
@@ -499,7 +533,7 @@ public class DartSqlResourceTest extends MSQTestBase
         false,
         false,
         false,
-        Collections.emptyMap(),
+        Map.of(QueryContexts.ENGINE, DartSqlEngine.NAME),
         Collections.emptyList()
     );
 
@@ -509,61 +543,6 @@ public class DartSqlResourceTest extends MSQTestBase
     );
   }
 
-  @Test
-  public void test_doPost_regularUser_restricted_throwsForbidden()
-  {
-    final MockAsyncContext asyncContext = new MockAsyncContext();
-    final MockHttpServletResponse asyncResponse = new MockHttpServletResponse();
-    asyncContext.response = asyncResponse;
-
-    Mockito.when(httpServletRequest.getAttribute(AuthConfig.DRUID_AUTHENTICATION_RESULT))
-           .thenReturn(makeAuthenticationResult(REGULAR_USER_NAME));
-    Mockito.when(httpServletRequest.startAsync())
-           .thenReturn(asyncContext);
-
-    final SqlQuery sqlQuery = new SqlQuery(
-        StringUtils.format("SELECT * FROM \"%s\"", CalciteTests.RESTRICTED_DATASOURCE),
-        ResultFormat.ARRAY,
-        false,
-        false,
-        false,
-        Collections.emptyMap(),
-        Collections.emptyList()
-    );
-
-    ForbiddenException e = Assertions.assertThrows(
-        ForbiddenException.class,
-        () -> sqlResource.doPost(sqlQuery, httpServletRequest)
-    );
-    Assertions.assertEquals("Unauthorized", e.getMessage());
-  }
-
-  @Test
-  public void test_doPost_superUser_restricted_throwsServerError()
-  {
-    final MockAsyncContext asyncContext = new MockAsyncContext();
-    final MockHttpServletResponse asyncResponse = new MockHttpServletResponse();
-    asyncContext.response = asyncResponse;
-
-    Mockito.when(httpServletRequest.getAttribute(AuthConfig.DRUID_AUTHENTICATION_RESULT))
-           .thenReturn(makeAuthenticationResult(CalciteTests.TEST_SUPERUSER_NAME));
-    Mockito.when(httpServletRequest.startAsync())
-           .thenReturn(asyncContext);
-
-    final SqlQuery sqlQuery = new SqlQuery(
-        StringUtils.format("SELECT * FROM \"%s\"", CalciteTests.RESTRICTED_DATASOURCE),
-        ResultFormat.ARRAY,
-        false,
-        false,
-        false,
-        Collections.emptyMap(),
-        Collections.emptyList()
-    );
-    Assertions.assertNull(sqlResource.doPost(sqlQuery, httpServletRequest));
-    // Super user can run a dart query, but we don't support it yet.
-    Assertions.assertEquals(Response.Status.INTERNAL_SERVER_ERROR.getStatusCode(), asyncResponse.getStatus());
-  }
-
   @Test
   public void test_doPost_regularUser_runtimeError() throws IOException
   {
@@ -582,7 +561,7 @@ public class DartSqlResourceTest extends MSQTestBase
         false,
         false,
         false,
-        Collections.emptyMap(),
+        Map.of(QueryContexts.ENGINE, DartSqlEngine.NAME),
         Collections.emptyList()
     );
 
@@ -617,7 +596,7 @@ public class DartSqlResourceTest extends MSQTestBase
         false,
         false,
         false,
-        ImmutableMap.of(DartSqlEngine.CTX_FULL_REPORT, true),
+        ImmutableMap.of(QueryContexts.CTX_FULL_REPORT, true, QueryContexts.ENGINE, DartSqlEngine.NAME),
         Collections.emptyList()
     );
 
@@ -640,6 +619,51 @@ public class DartSqlResourceTest extends MSQTestBase
     Assertions.assertArrayEquals(new Object[]{2}, results.get(0));
   }
 
+  @Test
+  public void test_doPost_queryTimeout() throws Exception
+  {
+    final MockAsyncContext asyncContext = new MockAsyncContext();
+    final MockHttpServletResponse asyncResponse = new MockHttpServletResponse();
+    asyncContext.response = asyncResponse;
+
+    Mockito.when(httpServletRequest.getAttribute(AuthConfig.DRUID_AUTHENTICATION_RESULT))
+           .thenReturn(makeAuthenticationResult(REGULAR_USER_NAME));
+    Mockito.when(httpServletRequest.startAsync())
+           .thenReturn(asyncContext);
+
+    final SqlQuery sqlQuery = new SqlQuery(
+        "SELECT 1 + 1",
+        ResultFormat.ARRAY,
+        false,
+        false,
+        false,
+        ImmutableMap.of(
+            QueryContexts.CTX_FULL_REPORT, true,
+            QueryContexts.TIMEOUT_KEY, 1,
+            QueryContexts.ENGINE, DartSqlEngine.NAME
+        ),
+        Collections.emptyList()
+    );
+
+    Assertions.assertNull(sqlResource.doPost(sqlQuery, httpServletRequest));
+    Assertions.assertEquals(Response.Status.OK.getStatusCode(), asyncResponse.getStatus());
+
+    final List<List<TaskReport.ReportMap>> reportMaps = objectMapper.readValue(
+        asyncResponse.baos.toByteArray(),
+        new TypeReference<>()
+        {
+        }
+    );
+
+    Assertions.assertEquals(1, reportMaps.size());
+    final MSQTaskReport report =
+        (MSQTaskReport) Iterables.getOnlyElement(Iterables.getOnlyElement(reportMaps)).get(MSQTaskReport.REPORT_KEY);
+    final MSQStatusReport statusReport = report.getPayload().getStatus();
+
+    Assertions.assertEquals(TaskState.FAILED, statusReport.getStatus());
+    Assertions.assertEquals(CanceledFault.timeout(), statusReport.getErrorReport().getFault());
+  }
+
   @Test
   public void test_doPost_regularUser_runtimeError_fullReport() throws Exception
   {
@@ -658,7 +682,7 @@ public class DartSqlResourceTest extends MSQTestBase
         false,
         false,
         false,
-        ImmutableMap.of(DartSqlEngine.CTX_FULL_REPORT, true),
+        ImmutableMap.of(QueryContexts.CTX_FULL_REPORT, true, QueryContexts.ENGINE, DartSqlEngine.NAME),
         Collections.emptyList()
     );
 
@@ -730,7 +754,11 @@ public class DartSqlResourceTest extends MSQTestBase
         false,
         false,
         false,
-        ImmutableMap.of(QueryContexts.CTX_SQL_QUERY_ID, sqlQueryId, DartSqlEngine.CTX_FULL_REPORT, fullReport),
+        ImmutableMap.of(
+            QueryContexts.CTX_SQL_QUERY_ID, sqlQueryId,
+            QueryContexts.CTX_FULL_REPORT, fullReport,
+            QueryContexts.ENGINE, DartSqlEngine.NAME
+        ),
         Collections.emptyList()
     );
 
@@ -774,7 +802,7 @@ public class DartSqlResourceTest extends MSQTestBase
     Assertions.assertEquals("Canceled", e.get("errorCode"));
     Assertions.assertEquals("CANCELED", e.get("category"));
     Assertions.assertEquals(
-        MSQFaultUtils.generateMessageWithErrorCode(CanceledFault.instance()),
+        MSQFaultUtils.generateMessageWithErrorCode(CanceledFault.userRequest()),
         e.get("errorMessage")
     );
   }
@@ -786,27 +814,28 @@ public class DartSqlResourceTest extends MSQTestBase
            .thenReturn(makeAuthenticationResult(REGULAR_USER_NAME));
 
     final Response cancellationResponse = sqlResource.cancelQuery("nonexistent", httpServletRequest);
-    Assertions.assertEquals(Response.Status.ACCEPTED.getStatusCode(), cancellationResponse.getStatus());
+    Assertions.assertEquals(Response.Status.NOT_FOUND.getStatusCode(), cancellationResponse.getStatus());
   }
 
   /**
    * Add a mock {@link ControllerHolder} to {@link #controllerRegistry}, with a query run by the given user.
-   * Used by methods that test {@link DartSqlResource#doGetRunningQueries}.
+   * Used by methods that test {@link SqlResource#doGetRunningQueries}.
    *
    * @return the mock holder
    */
   private ControllerHolder setUpMockRunningQuery(final String identity)
   {
     final Controller controller = Mockito.mock(Controller.class);
+    final ControllerContext controllerContext = Mockito.mock(ControllerContext.class);
     Mockito.when(controller.queryId()).thenReturn("did_" + identity);
+    Mockito.when(controller.getControllerContext()).thenReturn(controllerContext);
+    Mockito.when(controllerContext.selfNode()).thenReturn(Mockito.mock(DruidNode.class));
 
     final AuthenticationResult authenticationResult = makeAuthenticationResult(identity);
     final ControllerHolder holder = new ControllerHolder(
         controller,
-        null,
         "sid",
         "SELECT 1",
-        "localhost:1001",
         authenticationResult,
         DateTimes.of("2000")
     );
