@@ -22,13 +22,14 @@ package org.apache.druid.msq.exec;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
+import com.google.common.util.concurrent.Futures;
 import org.apache.druid.client.ImmutableSegmentLoadInfo;
 import org.apache.druid.java.util.common.ISE;
 import org.apache.druid.java.util.common.Intervals;
 import org.apache.druid.java.util.common.granularity.Granularities;
 import org.apache.druid.java.util.common.guava.Sequences;
 import org.apache.druid.java.util.common.guava.Yielders;
-import org.apache.druid.msq.indexing.MSQSpec;
+import org.apache.druid.msq.indexing.LegacyMSQSpec;
 import org.apache.druid.msq.indexing.MSQTuningConfig;
 import org.apache.druid.msq.indexing.destination.TaskReportMSQDestination;
 import org.apache.druid.msq.test.MSQTestBase;
@@ -103,25 +104,27 @@ public class MSQLoadedSegmentTests extends MSQTestBase
                                                .build();
 
     doReturn(
-        new DataServerQueryResult<>(
-            ImmutableList.of(
-                Yielders.each(
-                    Sequences.simple(
-                        ImmutableList.of(
-                            new Object[]{1L, "qwe"},
-                            new Object[]{1L, "tyu"}
+        Futures.immediateFuture(
+            new DataServerQueryResult<>(
+                ImmutableList.of(
+                    Yielders.each(
+                        Sequences.simple(
+                            ImmutableList.of(
+                                new Object[]{1L, "qwe"},
+                                new Object[]{1L, "tyu"}
+                            )
                         )
-                    )
-                )),
-            ImmutableList.of(),
-            "foo"
+                    )),
+                ImmutableList.of(),
+                "foo"
+            )
         )).when(dataServerQueryHandler)
           .fetchRowsFromDataServer(any(), any(), any());
 
     testSelectQuery()
         .setSql("select cnt, dim1 from foo")
         .setExpectedMSQSpec(
-            MSQSpec.builder()
+            LegacyMSQSpec.builder()
                    .query(
                        newScanQueryBuilder()
                            .dataSource(CalciteTests.DATASOURCE1)
@@ -164,18 +167,20 @@ public class MSQLoadedSegmentTests extends MSQTestBase
           ScanQuery query = invocationOnMock.getArgument(0);
           ScanQuery.verifyOrderByForNativeExecution(query);
           Assert.assertEquals(Long.MAX_VALUE, query.getScanRowsLimit());
-          return new DataServerQueryResult<>(
-              ImmutableList.of(
-                  Yielders.each(
-                      Sequences.simple(
-                          ImmutableList.of(
-                              new Object[]{1L, "qwe"},
-                              new Object[]{1L, "tyu"}
+          return Futures.immediateFuture(
+              new DataServerQueryResult<>(
+                  ImmutableList.of(
+                      Yielders.each(
+                          Sequences.simple(
+                              ImmutableList.of(
+                                  new Object[]{1L, "qwe"},
+                                  new Object[]{1L, "tyu"}
+                              )
                           )
-                      )
-                  )),
-              ImmutableList.of(),
-              "foo"
+                      )),
+                  ImmutableList.of(),
+                  "foo"
+              )
           );
         }
     )
@@ -185,7 +190,7 @@ public class MSQLoadedSegmentTests extends MSQTestBase
     testSelectQuery()
         .setSql("select cnt, dim1 from foo order by dim1")
         .setExpectedMSQSpec(
-            MSQSpec.builder()
+            LegacyMSQSpec.builder()
                    .query(
                        newScanQueryBuilder()
                            .dataSource(CalciteTests.DATASOURCE1)
@@ -225,17 +230,19 @@ public class MSQLoadedSegmentTests extends MSQTestBase
                                             .build();
 
     doReturn(
-        new DataServerQueryResult<>(
-            ImmutableList.of(
-                Yielders.each(
-                    Sequences.simple(
-                        ImmutableList.of(
-                            ResultRow.of(1L, 2L)
+        Futures.immediateFuture(
+            new DataServerQueryResult<>(
+                ImmutableList.of(
+                    Yielders.each(
+                        Sequences.simple(
+                            ImmutableList.of(
+                                ResultRow.of(1L, 2L)
+                            )
                         )
-                    )
-                )),
-            ImmutableList.of(),
-            "foo"
+                    )),
+                ImmutableList.of(),
+                "foo"
+            )
         )
     )
         .when(dataServerQueryHandler)
@@ -244,7 +251,7 @@ public class MSQLoadedSegmentTests extends MSQTestBase
     testSelectQuery()
         .setSql("select cnt,count(*) as cnt1 from foo group by cnt")
         .setExpectedMSQSpec(
-            MSQSpec.builder()
+            LegacyMSQSpec.builder()
                    .query(GroupByQuery.builder()
                                       .setDataSource(CalciteTests.DATASOURCE1)
                                       .setInterval(querySegmentSpec(Filtration
@@ -285,17 +292,19 @@ public class MSQLoadedSegmentTests extends MSQTestBase
                                             .build();
 
     doReturn(
-        new DataServerQueryResult<>(
-            ImmutableList.of(
-                Yielders.each(
-                    Sequences.simple(
-                        ImmutableList.of(
-                            ResultRow.of(1L, 2L)
+        Futures.immediateFuture(
+            new DataServerQueryResult<>(
+                ImmutableList.of(
+                    Yielders.each(
+                        Sequences.simple(
+                            ImmutableList.of(
+                                ResultRow.of(1L, 2L)
+                            )
                         )
-                    )
-                )),
-            ImmutableList.of(),
-            "foo"
+                    )),
+                ImmutableList.of(),
+                "foo"
+            )
         )
     )
         .when(dataServerQueryHandler)
@@ -304,7 +313,7 @@ public class MSQLoadedSegmentTests extends MSQTestBase
     testSelectQuery()
         .setSql("select cnt,count(*) as cnt1 from foo where (TIMESTAMP '2003-01-01 00:00:00' <= \"__time\" AND \"__time\" < TIMESTAMP '2005-01-01 00:00:00') group by cnt")
         .setExpectedMSQSpec(
-            MSQSpec.builder()
+            LegacyMSQSpec.builder()
                    .query(GroupByQuery.builder()
                                       .setDataSource(CalciteTests.DATASOURCE1)
                                       .setInterval(Intervals.of("2003-01-01T00:00:00.000Z/2005-01-01T00:00:00.000Z"))
@@ -353,7 +362,7 @@ public class MSQLoadedSegmentTests extends MSQTestBase
     testSelectQuery()
         .setSql("select cnt,count(*) as cnt1 from foo where (TIMESTAMP '2003-01-01 00:00:00' <= \"__time\" AND \"__time\" < TIMESTAMP '2005-01-01 00:00:00') group by cnt")
         .setExpectedMSQSpec(
-            MSQSpec.builder()
+            LegacyMSQSpec.builder()
                    .query(GroupByQuery.builder()
                                       .setDataSource(CalciteTests.DATASOURCE1)
                                       .setInterval(Intervals.of("2003-01-01T00:00:00.000Z/2005-01-01T00:00:00.000Z"))
