@@ -50,6 +50,7 @@ import org.apache.druid.msq.indexing.destination.DataSourceMSQDestination;
 import org.apache.druid.msq.kernel.WorkerAssignmentStrategy;
 import org.apache.druid.msq.util.MultiStageQueryContext;
 import org.apache.druid.query.OrderBy;
+import org.apache.druid.query.Query;
 import org.apache.druid.query.aggregation.AggregatorFactory;
 import org.apache.druid.query.aggregation.CountAggregatorFactory;
 import org.apache.druid.query.aggregation.LongSumAggregatorFactory;
@@ -80,6 +81,7 @@ import org.junit.Assert;
 import org.junit.Test;
 
 import javax.annotation.Nullable;
+
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
@@ -396,13 +398,14 @@ public class MSQCompactionRunnerTest
 
     MSQControllerTask msqControllerTask = Iterables.getOnlyElement(msqControllerTasks);
 
-    MSQSpec actualMSQSpec = msqControllerTask.getQuerySpec();
+    LegacyMSQSpec actualMSQSpec = msqControllerTask.getQuerySpec();
 
     Assert.assertEquals(getExpectedTuningConfig(), actualMSQSpec.getTuningConfig());
     Assert.assertEquals(getExpectedDestination(), actualMSQSpec.getDestination());
 
-    Assert.assertTrue(actualMSQSpec.getQuery() instanceof ScanQuery);
-    ScanQuery scanQuery = (ScanQuery) actualMSQSpec.getQuery();
+    Query<?> query = actualMSQSpec.getQuery();
+    Assert.assertTrue(query instanceof ScanQuery);
+    ScanQuery scanQuery = (ScanQuery) query;
 
     List<String> expectedColumns = new ArrayList<>();
     List<ColumnType> expectedColumnTypes = new ArrayList<>();
@@ -477,10 +480,11 @@ public class MSQCompactionRunnerTest
         Collections.singletonMap(COMPACTION_INTERVAL, dataSchema)
     );
 
-    MSQSpec actualMSQSpec = Iterables.getOnlyElement(msqControllerTasks).getQuerySpec();
+    LegacyMSQSpec actualMSQSpec = Iterables.getOnlyElement(msqControllerTasks).getQuerySpec();
 
-    Assert.assertTrue(actualMSQSpec.getQuery() instanceof ScanQuery);
-    ScanQuery scanQuery = (ScanQuery) actualMSQSpec.getQuery();
+    Query<?> query = actualMSQSpec.getQuery();
+    Assert.assertTrue(query instanceof ScanQuery);
+    ScanQuery scanQuery = (ScanQuery) query;
 
     // Dimensions should already list __time and the order should remain intact
     Assert.assertEquals(
@@ -527,13 +531,14 @@ public class MSQCompactionRunnerTest
 
     MSQControllerTask msqControllerTask = Iterables.getOnlyElement(msqControllerTasks);
 
-    MSQSpec actualMSQSpec = msqControllerTask.getQuerySpec();
+    LegacyMSQSpec actualMSQSpec = msqControllerTask.getQuerySpec();
 
     Assert.assertEquals(getExpectedTuningConfig(), actualMSQSpec.getTuningConfig());
     Assert.assertEquals(getExpectedDestination(), actualMSQSpec.getDestination());
 
-    Assert.assertTrue(actualMSQSpec.getQuery() instanceof GroupByQuery);
-    GroupByQuery groupByQuery = (GroupByQuery) actualMSQSpec.getQuery();
+    Query<?> query = actualMSQSpec.getQuery();
+    Assert.assertTrue(query instanceof GroupByQuery);
+    GroupByQuery groupByQuery = (GroupByQuery) query;
 
     Assert.assertEquals(dimFilter, groupByQuery.getFilter());
     Assert.assertEquals(
@@ -612,7 +617,7 @@ public class MSQCompactionRunnerTest
 
     MSQControllerTask msqControllerTask = Iterables.getOnlyElement(msqControllerTasks);
 
-    MSQSpec actualMSQSpec = msqControllerTask.getQuerySpec();
+    LegacyMSQSpec actualMSQSpec = msqControllerTask.getQuerySpec();
 
     Assert.assertEquals(getExpectedTuningConfig(), actualMSQSpec.getTuningConfig());
     Assert.assertEquals(getExpectedDestinationWithProjections(), actualMSQSpec.getDestination());
