@@ -43,8 +43,8 @@ import org.apache.druid.collections.ResourceHolder;
 import org.apache.druid.data.input.impl.DimensionsSpec;
 import org.apache.druid.data.input.impl.LongDimensionSchema;
 import org.apache.druid.data.input.impl.StringDimensionSchema;
-import org.apache.druid.discovery.BrokerClient;
 import org.apache.druid.discovery.NodeRole;
+import org.apache.druid.frame.FrameType;
 import org.apache.druid.frame.channel.FrameChannelSequence;
 import org.apache.druid.frame.processor.Bouncer;
 import org.apache.druid.frame.testutil.FrameTestUtil;
@@ -75,7 +75,6 @@ import org.apache.druid.java.util.common.granularity.Granularity;
 import org.apache.druid.java.util.common.io.Closer;
 import org.apache.druid.java.util.common.logger.Logger;
 import org.apache.druid.java.util.emitter.EmittingLogger;
-import org.apache.druid.java.util.http.client.Request;
 import org.apache.druid.metadata.input.InputSourceModule;
 import org.apache.druid.msq.counters.CounterNames;
 import org.apache.druid.msq.counters.CounterSnapshots;
@@ -93,8 +92,8 @@ import org.apache.druid.msq.guice.MSQIndexingModule;
 import org.apache.druid.msq.guice.MSQSqlModule;
 import org.apache.druid.msq.guice.MultiStageQuery;
 import org.apache.druid.msq.indexing.InputChannelFactory;
+import org.apache.druid.msq.indexing.LegacyMSQSpec;
 import org.apache.druid.msq.indexing.MSQControllerTask;
-import org.apache.druid.msq.indexing.MSQSpec;
 import org.apache.druid.msq.indexing.MSQTuningConfig;
 import org.apache.druid.msq.indexing.destination.DataSourceMSQDestination;
 import org.apache.druid.msq.indexing.destination.SegmentGenerationTerminalStageSpecFactory;
@@ -112,6 +111,7 @@ import org.apache.druid.msq.indexing.report.MSQTaskReportPayload;
 import org.apache.druid.msq.kernel.StageDefinition;
 import org.apache.druid.msq.querykit.DataSegmentProvider;
 import org.apache.druid.msq.shuffle.input.DurableStorageInputChannelFactory;
+import org.apache.druid.msq.sql.MSQTaskQueryKitSpecFactory;
 import org.apache.druid.msq.sql.MSQTaskQueryMaker;
 import org.apache.druid.msq.sql.MSQTaskSqlEngine;
 import org.apache.druid.msq.sql.entity.PageInformation;
@@ -133,6 +133,8 @@ import org.apache.druid.query.groupby.GroupByQueryConfig;
 import org.apache.druid.query.groupby.GroupByQueryRunnerTest;
 import org.apache.druid.query.groupby.GroupingEngine;
 import org.apache.druid.query.groupby.TestGroupByBuffers;
+import org.apache.druid.query.policy.NoopPolicyEnforcer;
+import org.apache.druid.query.policy.PolicyEnforcer;
 import org.apache.druid.rpc.ServiceClientFactory;
 import org.apache.druid.segment.AggregateProjectionMetadata;
 import org.apache.druid.segment.CompleteSegment;
@@ -242,13 +244,10 @@ import static org.apache.druid.sql.calcite.util.CalciteTests.WIKIPEDIA;
 import static org.apache.druid.sql.calcite.util.TestDataBuilder.ROWS1;
 import static org.apache.druid.sql.calcite.util.TestDataBuilder.ROWS2;
 import static org.hamcrest.MatcherAssert.assertThat;
-import static org.mockito.ArgumentMatchers.any;
-import static org.mockito.ArgumentMatchers.anyString;
-import static org.mockito.Mockito.doReturn;
 import static org.mockito.Mockito.mock;
 
 /**
- * Base test runner for running MSQ unit tests. It sets up multi stage query execution environment
+ * Base test runner for running MSQ unit tests. It sets up multi-stage query execution environment
  * and populates data for the datasources. The runner does not go via the HTTP layer for communication between the
  * various MSQ processes.
  * <p>
@@ -276,6 +275,7 @@ public class MSQTestBase extends BaseCalciteQueryTest
                   .put(MSQWarnings.CTX_MAX_PARSE_EXCEPTIONS_ALLOWED, 0)
                   .put(MSQTaskQueryMaker.USER_KEY, CalciteTests.REGULAR_USER_AUTH_RESULT.getIdentity())
                   .put(MultiStageQueryContext.WINDOW_FUNCTION_OPERATOR_TRANSFORMATION, true)
+                  .put(MultiStageQueryContext.CTX_ROW_BASED_FRAME_TYPE, (int) FrameType.latestRowBased().version())
                   .build();
 
   public static final Map<String, Object> SUPERUSER_MSQ_CONTEXT =
@@ -348,6 +348,8 @@ public class MSQTestBase extends BaseCalciteQueryTest
   private TestGroupByBuffers groupByBuffers;
   protected final WorkerMemoryParameters workerMemoryParameters = Mockito.spy(makeTestWorkerMemoryParameters());
   protected static final String TEST_CONTROLLER_TASK_ID = "query-test-query";
+  // Fields in the query context to ignore during assertion.
+  protected Set<String> ignoreFields = Set.of(MultiStageQueryContext.CTX_START_TIME);
 
   protected static class MSQBaseComponentSupplier extends StandardComponentSupplier
   {
@@ -438,7 +440,6 @@ public class MSQTestBase extends BaseCalciteQueryTest
 
     segmentManager = new MSQTestSegmentManager(segmentCacheManager);
 
-    BrokerClient brokerClient = mock(BrokerClient.class);
     List<Module> modules = ImmutableList.of(
         binder -> {
           DruidProcessingConfig druidProcessingConfig = new DruidProcessingConfig()
@@ -538,8 +539,8 @@ public class MSQTestBase extends BaseCalciteQueryTest
         new LookylooModule(),
         new SegmentWranglerModule(),
         new HllSketchModule(),
-        binder -> binder.bind(BrokerClient.class).toInstance(brokerClient),
-        binder -> binder.bind(Bouncer.class).toInstance(new Bouncer(1))
+        binder -> binder.bind(Bouncer.class).toInstance(new Bouncer(1)),
+        binder -> binder.bind(PolicyEnforcer.class).toInstance(NoopPolicyEnforcer.instance())
     );
     // adding node role injection to the modules, since CliPeon would also do that through run method
     injector = new CoreInjectorBuilder(new StartupInjectorBuilder().build(), ImmutableSet.of(NodeRole.PEON))
@@ -552,8 +553,6 @@ public class MSQTestBase extends BaseCalciteQueryTest
     objectMapper.registerModules(sqlModule.getJacksonModules());
     objectMapper.registerModules(BuiltInTypesModule.getJacksonModulesList());
 
-    doReturn(mock(Request.class)).when(brokerClient).makeRequest(any(), anyString());
-
     testTaskActionClient = Mockito.spy(new MSQTestTaskActionClient(objectMapper, injector));
     indexingServiceClient = new MSQTestOverlordServiceClient(
         objectMapper,
@@ -575,10 +574,13 @@ public class MSQTestBase extends BaseCalciteQueryTest
         CatalogResolver.NULL_RESOLVER
     );
 
+
     final SqlEngine engine = new MSQTaskSqlEngine(
         indexingServiceClient,
         qf.queryJsonMapper().copy().registerModules(new MSQSqlModule().getJacksonModules()),
-        new SegmentGenerationTerminalStageSpecFactory()
+        new SegmentGenerationTerminalStageSpecFactory(),
+        injector.getInstance(MSQTaskQueryKitSpecFactory.class),
+        null
     );
 
     PlannerFactory plannerFactory = new PlannerFactory(
@@ -593,10 +595,11 @@ public class MSQTestBase extends BaseCalciteQueryTest
         CalciteTests.createJoinableFactoryWrapper(),
         catalogResolver,
         new AuthConfig(),
+        NoopPolicyEnforcer.instance(),
         new DruidHookDispatcher()
     );
 
-    sqlStatementFactory = CalciteTests.createSqlStatementFactory(engine, plannerFactory);
+    sqlStatementFactory = QueryFrameworkUtils.createSqlMultiStatementFactory(engine, plannerFactory);
 
     authorizerMapper = CalciteTests.TEST_EXTERNAL_AUTHORIZER_MAPPER;
 
@@ -640,11 +643,12 @@ public class MSQTestBase extends BaseCalciteQueryTest
 
   private DataServerQueryHandlerFactory getTestDataServerQueryHandlerFactory()
   {
-    DataServerQueryHandlerFactory mockFactory = Mockito.mock(DataServerQueryHandlerFactory.class);
-    doReturn(dataServerQueryHandler)
-        .when(mockFactory)
-        .createDataServerQueryHandler(anyString(), any(), any());
-    return mockFactory;
+    return (inputNumber, dataSourceName, channelCounters, dataServerRequestDescriptor) -> dataServerQueryHandler;
+  }
+
+  protected List<Number> getEmittedMetrics(String metricName, Map<String, Object> dimensionFilters)
+  {
+    return indexingServiceClient.getEmittedMetrics(metricName, dimensionFilters);
   }
 
   @Nonnull
@@ -724,15 +728,14 @@ public class MSQTestBase extends BaseCalciteQueryTest
 
         @Nullable
         @Override
-        public QueryableIndex asQueryableIndex()
+        public <T> T as(@Nonnull Class<T> clazz)
         {
-          return index;
-        }
-
-        @Override
-        public CursorFactory asCursorFactory()
-        {
-          return new QueryableIndexCursorFactory(index);
+          if (CursorFactory.class.equals(clazz)) {
+            return (T) new QueryableIndexCursorFactory(index);
+          } else if (QueryableIndex.class.equals(clazz)) {
+            return (T) index;
+          }
+          return null;
         }
 
         @Override
@@ -807,12 +810,12 @@ public class MSQTestBase extends BaseCalciteQueryTest
   )
   {
     final DirectStatement stmt = sqlStatementFactory.directStatement(
-        new SqlQueryPlus(
-            query,
-            context,
-            parameters,
-            authenticationResult
-        )
+        SqlQueryPlus.builder()
+                    .sql(query)
+                    .context(context)
+                    .parameters(parameters)
+                    .auth(authenticationResult)
+                    .build()
     );
 
     final List<Object[]> sequence = stmt.execute().getResults().toList();
@@ -860,12 +863,21 @@ public class MSQTestBase extends BaseCalciteQueryTest
     return payload.getStatus().getErrorReport();
   }
 
-  private void assertMSQSpec(MSQSpec expectedMSQSpec, MSQSpec querySpecForTask)
+  private void assertMSQSpec(LegacyMSQSpec expectedMSQSpec, LegacyMSQSpec querySpecForTask)
   {
-    Assert.assertEquals(expectedMSQSpec.getQuery(), querySpecForTask.getQuery());
+    final Map<String, Object> ignoredContext = new HashMap<>();
+    final Map<String, Object> actualQueryContext = querySpecForTask.getQuery().getContext();
+    for (String ignoredField : ignoreFields) {
+      if (actualQueryContext.containsKey(ignoredField)) {
+        ignoredContext.put(ignoredField, actualQueryContext.get(ignoredField));
+      }
+    }
+
+    Assert.assertEquals(expectedMSQSpec.getQuery().withOverriddenContext(ignoredContext), querySpecForTask.getQuery());
     Assert.assertEquals(expectedMSQSpec.getAssignmentStrategy(), querySpecForTask.getAssignmentStrategy());
     Assert.assertEquals(expectedMSQSpec.getColumnMappings(), querySpecForTask.getColumnMappings());
     Assert.assertEquals(expectedMSQSpec.getDestination(), querySpecForTask.getDestination());
+
   }
 
   private void assertTuningConfig(
@@ -909,7 +921,7 @@ public class MSQTestBase extends BaseCalciteQueryTest
     protected Map<String, Object> queryContext = DEFAULT_MSQ_CONTEXT;
     protected List<TypedValue> dynamicParameters = new ArrayList<>();
     protected List<MSQResultsReport.ColumnAndType> expectedRowSignature = null;
-    protected MSQSpec expectedMSQSpec = null;
+    protected LegacyMSQSpec expectedMSQSpec = null;
     protected MSQTuningConfig expectedTuningConfig = null;
     protected Set<SegmentId> expectedSegments = null;
     protected CompactionState expectedLastCompactionState = null;
@@ -925,6 +937,7 @@ public class MSQTestBase extends BaseCalciteQueryTest
     protected Map<Integer, Integer> expectedStageVsWorkerCount = new HashMap<>();
     protected final Map<Integer, Map<Integer, Map<String, CounterSnapshotMatcher>>>
         expectedStageWorkerChannelToCounters = new HashMap<>();
+    protected Map<String, Object> expectedMetricDimensions = null;
 
     private boolean hasRun = false;
 
@@ -1001,7 +1014,7 @@ public class MSQTestBase extends BaseCalciteQueryTest
       return asBuilder();
     }
 
-    public Builder setExpectedMSQSpec(MSQSpec expectedMSQSpec)
+    public Builder setExpectedMSQSpec(LegacyMSQSpec expectedMSQSpec)
     {
       this.expectedMSQSpec = expectedMSQSpec;
       return asBuilder();
@@ -1074,6 +1087,12 @@ public class MSQTestBase extends BaseCalciteQueryTest
       return asBuilder();
     }
 
+    public Builder setExpectedMetricDimensions(Map<String, Object> expectedMetricDimensions)
+    {
+      this.expectedMetricDimensions = expectedMetricDimensions;
+      return asBuilder();
+    }
+
     @SuppressWarnings("unchecked")
     private Builder asBuilder()
     {
@@ -1094,6 +1113,31 @@ public class MSQTestBase extends BaseCalciteQueryTest
       assertThat(e, expectedValidationErrorMatcher);
     }
 
+    protected void verifyMetrics()
+    {
+      if (expectedMetricDimensions != null) {
+        Map<String, Object> controllerDims = new HashMap<>(expectedMetricDimensions);
+        controllerDims.put(MSQTestOverlordServiceClient.TEST_METRIC_DIMENSION, MSQTestOverlordServiceClient.METRIC_CONTROLLER_TASK_TYPE);
+
+        Map<String, Object> workerDims = new HashMap<>(expectedMetricDimensions);
+        workerDims.put(MSQTestOverlordServiceClient.TEST_METRIC_DIMENSION, MSQTestOverlordServiceClient.METRIC_WORKER_TASK_TYPE);
+
+        // Since the time could vary, it can't be asserted, but the dimensions are asserted by using them as a filter.
+        // The value should be greater than 0 as a basic sanity check.
+        List<Number> metric = getEmittedMetrics("query/time", controllerDims);
+        Assert.assertEquals(1, metric.size());
+        Assert.assertTrue(metric.get(0).longValue() > 0);
+
+        metric = getEmittedMetrics("query/time", workerDims);
+        Assert.assertEquals(1, metric.size());
+        Assert.assertTrue(metric.get(0).longValue() > 0);
+
+        metric = getEmittedMetrics("query/cpu/time", workerDims);
+        Assert.assertEquals(1, metric.size());
+        Assert.assertTrue(metric.get(0).longValue() > 0);
+      }
+    }
+
     protected void verifyLookupLoadingInfoInTaskContext(Map<String, Object> context)
     {
       LookupLoadingSpec specFromContext = LookupLoadingSpec.createFromContext(context, LookupLoadingSpec.ALL);
@@ -1277,9 +1321,10 @@ public class MSQTestBase extends BaseCalciteQueryTest
         MSQTaskReportPayload reportPayload = getPayloadOrThrow(controllerId);
         verifyWorkerCount(reportPayload.getCounters());
         verifyCounters(reportPayload.getCounters());
+        verifyMetrics();
 
         MSQControllerTask msqControllerTask = indexingServiceClient.getMSQControllerTask(controllerId);
-        MSQSpec foundSpec = msqControllerTask.getQuerySpec();
+        LegacyMSQSpec foundSpec = msqControllerTask.getQuerySpec();
         verifyLookupLoadingInfoInTaskContext(msqControllerTask.getContext());
         log.info(
             "found generated segments: %s",
@@ -1402,8 +1447,8 @@ public class MSQTestBase extends BaseCalciteQueryTest
 
         // Assert on the tombstone intervals
         // Tombstone segments are only published, but since they do not have any data, they are not pushed by the
-        // SegmentGeneratorFrameProcessorFactory. We can get the tombstone segment ids published by taking a set
-        // difference of all the segments published with the segments that are created by the SegmentGeneratorFrameProcessorFactory
+        // SegmentGeneratorStageProcessor. We can get the tombstone segment ids published by taking a set
+        // difference of all the segments published with the segments that are created by the SegmentGeneratorStageProcessor
         if (!testTaskActionClient.getPublishedSegments().isEmpty()) {
           if (expectedLastCompactionState != null) {
             CompactionState compactionState = testTaskActionClient.getPublishedSegments().stream().findFirst().get()
@@ -1485,6 +1530,7 @@ public class MSQTestBase extends BaseCalciteQueryTest
             expectedExecutionErrorMatcher
         );
       }
+      verifyMetrics();
     }
   }
 
@@ -1497,7 +1543,7 @@ public class MSQTestBase extends BaseCalciteQueryTest
 
     // Made the visibility public to aid adding ut's easily with minimum parameters to set.
     @Nullable
-    public Pair<MSQSpec, Pair<List<MSQResultsReport.ColumnAndType>, List<Object[]>>> runQueryWithResult()
+    public Pair<LegacyMSQSpec, Pair<List<MSQResultsReport.ColumnAndType>, List<Object[]>>> runQueryWithResult()
     {
       readyToRun();
       Preconditions.checkArgument(
@@ -1541,7 +1587,7 @@ public class MSQTestBase extends BaseCalciteQueryTest
           MSQControllerTask msqControllerTask = indexingServiceClient.getMSQControllerTask(controllerId);
           verifyLookupLoadingInfoInTaskContext(msqControllerTask.getContext());
 
-          final MSQSpec spec = msqControllerTask.getQuerySpec();
+          final LegacyMSQSpec spec = msqControllerTask.getQuerySpec();
           final List<Object[]> rows;
 
           if (spec.getDestination() instanceof TaskReportMSQDestination) {
@@ -1618,7 +1664,7 @@ public class MSQTestBase extends BaseCalciteQueryTest
         Preconditions.checkArgument(expectedRowSignature != null, "Row signature cannot be null");
         Preconditions.checkArgument(expectedMSQSpec != null, "MultiStageQuery Query spec cannot be null ");
       }
-      Pair<MSQSpec, Pair<List<MSQResultsReport.ColumnAndType>, List<Object[]>>> specAndResults = runQueryWithResult();
+      Pair<LegacyMSQSpec, Pair<List<MSQResultsReport.ColumnAndType>, List<Object[]>>> specAndResults = runQueryWithResult();
 
       if (specAndResults == null) { // A fault was expected and the assertion has been done in the runQueryWithResult
         return;
@@ -1627,6 +1673,7 @@ public class MSQTestBase extends BaseCalciteQueryTest
       Assert.assertEquals(expectedRowSignature, specAndResults.rhs.lhs);
       assertResultsEquals(sql != null ? sql : taskSpec.toString(), expectedResultRows, specAndResults.rhs.rhs);
       assertMSQSpec(expectedMSQSpec, specAndResults.lhs);
+      verifyMetrics();
     }
 
     public void verifyExecutionError()
@@ -1635,6 +1682,7 @@ public class MSQTestBase extends BaseCalciteQueryTest
       if (runQueryWithResult() != null) {
         throw new ISE("Query %s did not throw an exception", sql != null ? sql : taskSpec);
       }
+      verifyMetrics();
     }
   }
 
