@@ -81,7 +81,7 @@ public abstract class HadoopTask extends AbstractBatchIndexTask
   // This is only used for classpath isolation in the runTask isolation stuff, so it shooouuullldddd be ok.
   /** {@link #buildClassLoader(TaskToolbox)} has outdated javadocs referencing this field, TODO update */
   @SuppressWarnings("unused")
-  protected static final Predicate<URL> IS_DRUID_URL = new Predicate<URL>()
+  protected static final Predicate<URL> IS_DRUID_URL = new Predicate<>()
   {
     @Override
     public boolean apply(@Nullable URL input)
@@ -173,16 +173,7 @@ public abstract class HadoopTask extends AbstractBatchIndexTask
       localClassLoaderURLs.addAll(Arrays.asList(hadoopLoader.getURLs()));
     }
 
-    ClassLoader parent = null;
-    if (JvmUtils.isIsJava9Compatible()) {
-      try {
-        // See also https://docs.oracle.com/en/java/javase/11/migrate/index.html#JSMIG-GUID-A868D0B9-026F-4D46-B979-901834343F9E
-        parent = (ClassLoader) ClassLoader.class.getMethod("getPlatformClassLoader").invoke(null);
-      }
-      catch (IllegalAccessException | InvocationTargetException | NoSuchMethodException e) {
-        throw new RuntimeException(e);
-      }
-    }
+    ClassLoader parent = ClassLoader.getPlatformClassLoader();
     final ClassLoader classLoader = new URLClassLoader(
         localClassLoaderURLs.toArray(new URL[0]),
         parent
