@@ -57,6 +57,7 @@ public class TaskLockHelper
   private final Map<Interval, OverwritingRootGenerationPartitions> overwritingRootGenPartitions = new HashMap<>();
   private final Set<DataSegment> lockedExistingSegments = new HashSet<>();
   private final boolean useSegmentLock;
+  private final TaskLockType taskLockType;
 
   @Nullable
   private Granularity knownSegmentGranularity;
@@ -90,9 +91,10 @@ public class TaskLockHelper
     }
   }
 
-  public TaskLockHelper(boolean useSegmentLock)
+  public TaskLockHelper(boolean useSegmentLock, TaskLockType taskLockType)
   {
     this.useSegmentLock = useSegmentLock;
+    this.taskLockType = taskLockType;
   }
 
   public boolean isUseSegmentLock()
@@ -105,9 +107,9 @@ public class TaskLockHelper
     return useSegmentLock ? LockGranularity.SEGMENT : LockGranularity.TIME_CHUNK;
   }
 
-  public boolean hasLockedExistingSegments()
+  public TaskLockType getLockTypeToUse()
   {
-    return !lockedExistingSegments.isEmpty();
+    return taskLockType;
   }
 
   public boolean hasOverwritingRootGenerationPartition(Interval interval)
@@ -125,6 +127,9 @@ public class TaskLockHelper
     return overwritingRootGenPartitions.get(interval);
   }
 
+  /**
+   * Verify and lock existing segments when using a SegmentLock
+   */
   boolean verifyAndLockExistingSegments(TaskActionClient actionClient, List<DataSegment> segments)
       throws IOException
   {
