@@ -25,17 +25,19 @@ import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Lists;
 import com.google.common.io.Files;
 import org.apache.druid.client.indexing.ClientCompactionTaskGranularitySpec;
-import org.apache.druid.client.indexing.ClientCompactionTaskTransformSpec;
 import org.apache.druid.data.input.InputSplit;
 import org.apache.druid.data.input.SegmentsSplitHintSpec;
+import org.apache.druid.data.input.impl.AggregateProjectionSpec;
 import org.apache.druid.data.input.impl.CsvInputFormat;
 import org.apache.druid.data.input.impl.DimensionSchema;
 import org.apache.druid.data.input.impl.DimensionsSpec;
 import org.apache.druid.data.input.impl.LocalInputSource;
+import org.apache.druid.data.input.impl.LongDimensionSchema;
 import org.apache.druid.data.input.impl.StringDimensionSchema;
 import org.apache.druid.data.input.impl.TimestampSpec;
 import org.apache.druid.indexer.TaskState;
 import org.apache.druid.indexer.TaskStatus;
+import org.apache.druid.indexer.granularity.UniformGranularitySpec;
 import org.apache.druid.indexer.partitions.DimensionRangePartitionsSpec;
 import org.apache.druid.indexer.partitions.DynamicPartitionsSpec;
 import org.apache.druid.indexer.partitions.HashedPartitionsSpec;
@@ -61,9 +63,10 @@ import org.apache.druid.query.aggregation.LongSumAggregatorFactory;
 import org.apache.druid.query.filter.SelectorDimFilter;
 import org.apache.druid.segment.DataSegmentsWithSchemas;
 import org.apache.druid.segment.SegmentUtils;
+import org.apache.druid.segment.VirtualColumns;
 import org.apache.druid.segment.indexing.DataSchema;
-import org.apache.druid.segment.indexing.granularity.UniformGranularitySpec;
 import org.apache.druid.segment.loading.NoopSegmentCacheManager;
+import org.apache.druid.segment.transform.CompactionTransformSpec;
 import org.apache.druid.server.security.Action;
 import org.apache.druid.server.security.Resource;
 import org.apache.druid.server.security.ResourceAction;
@@ -94,7 +97,6 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
-import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
@@ -116,6 +118,23 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
   private static final String DATA_SOURCE = "test";
   private static final Interval INTERVAL_TO_INDEX = Intervals.of("2014-01-01/2014-01-02");
 
+  private static final AggregateProjectionSpec PROJECTION_SPEC = new AggregateProjectionSpec(
+      "projection1",
+      VirtualColumns.create(
+          Granularities.toVirtualColumn(
+              Granularities.HOUR,
+              Granularities.GRANULARITY_VIRTUAL_COLUMN_NAME
+          )
+      ),
+      ImmutableList.of(
+          new LongDimensionSchema(Granularities.GRANULARITY_VIRTUAL_COLUMN_NAME),
+          new StringDimensionSchema("dim", DimensionSchema.MultiValueHandling.ARRAY, null)
+      ),
+      new AggregatorFactory[]{
+          new LongSumAggregatorFactory("val", "val")
+      }
+  );
+
   private final LockGranularity lockGranularity;
 
   // Whether the currently-running test case is allowed to fetch segments in the primary logic of the CompactionTask.
@@ -152,7 +171,7 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
   }
 
   @Test
-  public void testRunParallelWithDynamicPartitioningMatchCompactionState() throws Exception
+  public void testRunParallelWithDynamicPartitioningMatchCompactionState()
   {
     allowSegmentFetchesByCompactionTask = true;
     runIndexTask(null, true);
@@ -176,10 +195,7 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
           segment.getShardSpec().getClass()
       );
       // Expect compaction state to exist as store compaction state by default
-      Map<String, String> expectedLongSumMetric = new HashMap<>();
-      expectedLongSumMetric.put("type", "longSum");
-      expectedLongSumMetric.put("name", "val");
-      expectedLongSumMetric.put("fieldName", "val");
+      LongSumAggregatorFactory expectedLongSumMetric = new LongSumAggregatorFactory("val", "val");
       CompactionState expectedState = new CompactionState(
           new DynamicPartitionsSpec(null, Long.MAX_VALUE),
           new DimensionsSpec(
@@ -190,18 +206,14 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
           ),
           ImmutableList.of(expectedLongSumMetric),
           null,
-          compactionTask.getTuningConfig().getIndexSpec().asMap(getObjectMapper()),
-          getObjectMapper().readValue(
-              getObjectMapper().writeValueAsString(
-                  new UniformGranularitySpec(
-                      Granularities.HOUR,
-                      Granularities.MINUTE,
-                      true,
-                      ImmutableList.of(segment.getInterval())
-                  )
-              ),
-              Map.class
-          )
+          compactionTask.getTuningConfig().getIndexSpec(),
+          new UniformGranularitySpec(
+              Granularities.HOUR,
+              Granularities.MINUTE,
+              true,
+              ImmutableList.of(segment.getInterval())
+          ),
+          null
       );
       Assert.assertEquals("Compaction state for " + segment.getId(), expectedState, segment.getLastCompactionState());
     }
@@ -230,10 +242,7 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
     final Set<DataSegment> compactedSegments = dataSegmentsWithSchemas.getSegments();
     for (DataSegment segment : compactedSegments) {
       // Expect compaction state to exist as store compaction state by default
-      Map<String, String> expectedLongSumMetric = new HashMap<>();
-      expectedLongSumMetric.put("type", "longSum");
-      expectedLongSumMetric.put("name", "val");
-      expectedLongSumMetric.put("fieldName", "val");
+      LongSumAggregatorFactory expectedLongSumMetric = new LongSumAggregatorFactory("val", "val");
       Assert.assertSame(HashBasedNumberedShardSpec.class, segment.getShardSpec().getClass());
       CompactionState expectedState = new CompactionState(
           new HashedPartitionsSpec(null, 3, null),
@@ -245,18 +254,14 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
           ),
           ImmutableList.of(expectedLongSumMetric),
           null,
-          compactionTask.getTuningConfig().getIndexSpec().asMap(getObjectMapper()),
-          getObjectMapper().readValue(
-              getObjectMapper().writeValueAsString(
-                  new UniformGranularitySpec(
-                      Granularities.HOUR,
-                      Granularities.MINUTE,
-                      true,
-                      ImmutableList.of(segment.getInterval())
-                  )
-              ),
-              Map.class
-          )
+          compactionTask.getTuningConfig().getIndexSpec(),
+          new UniformGranularitySpec(
+              Granularities.HOUR,
+              Granularities.MINUTE,
+              true,
+              ImmutableList.of(segment.getInterval())
+          ),
+          null
       );
       Assert.assertEquals("Compaction state for " + segment.getId(), expectedState, segment.getLastCompactionState());
     }
@@ -300,10 +305,7 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
     final Set<DataSegment> compactedSegments = dataSegmentsWithSchemas.getSegments();
     for (DataSegment segment : compactedSegments) {
       // Expect compaction state to exist as store compaction state by default
-      Map<String, String> expectedLongSumMetric = new HashMap<>();
-      expectedLongSumMetric.put("type", "longSum");
-      expectedLongSumMetric.put("name", "val");
-      expectedLongSumMetric.put("fieldName", "val");
+      LongSumAggregatorFactory expectedLongSumMetric = new LongSumAggregatorFactory("val", "val");
       Assert.assertSame(SingleDimensionShardSpec.class, segment.getShardSpec().getClass());
       CompactionState expectedState = new CompactionState(
           new SingleDimensionPartitionsSpec(7, null, "dim", false),
@@ -315,18 +317,14 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
           ),
           ImmutableList.of(expectedLongSumMetric),
           null,
-          compactionTask.getTuningConfig().getIndexSpec().asMap(getObjectMapper()),
-          getObjectMapper().readValue(
-              getObjectMapper().writeValueAsString(
-                  new UniformGranularitySpec(
-                      Granularities.HOUR,
-                      Granularities.MINUTE,
-                      true,
-                      ImmutableList.of(segment.getInterval())
-                  )
-              ),
-              Map.class
-          )
+          compactionTask.getTuningConfig().getIndexSpec(),
+          new UniformGranularitySpec(
+              Granularities.HOUR,
+              Granularities.MINUTE,
+              true,
+              ImmutableList.of(segment.getInterval())
+          ),
+          null
       );
       Assert.assertEquals("Compaction state for " + segment.getId(), expectedState, segment.getLastCompactionState());
     }
@@ -365,30 +363,23 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
     final Set<DataSegment> compactedSegments = dataSegmentsWithSchemas.getSegments();
     for (DataSegment segment : compactedSegments) {
       // Expect compaction state to exist as store compaction state by default
-      Map<String, String> expectedLongSumMetric = new HashMap<>();
-      expectedLongSumMetric.put("type", "longSum");
-      expectedLongSumMetric.put("name", "val");
-      expectedLongSumMetric.put("fieldName", "val");
+      LongSumAggregatorFactory expectedLongSumMetric = new LongSumAggregatorFactory("val", "val");
       Assert.assertSame(SingleDimensionShardSpec.class, segment.getShardSpec().getClass());
       CompactionState expectedState = new CompactionState(
           new SingleDimensionPartitionsSpec(7, null, "dim", false),
           new DimensionsSpec(DimensionsSpec.getDefaultSchemas(ImmutableList.of("ts", "dim"))),
           ImmutableList.of(expectedLongSumMetric),
           null,
-          compactionTask.getTuningConfig().getIndexSpec().asMap(getObjectMapper()),
-          getObjectMapper().readValue(
-              getObjectMapper().writeValueAsString(
-                  new UniformGranularitySpec(
-                      Granularities.HOUR,
-                      Granularities.MINUTE,
-                      true,
+          compactionTask.getTuningConfig().getIndexSpec(),
+          new UniformGranularitySpec(
+              Granularities.HOUR,
+              Granularities.MINUTE,
+              true,
 
-                      // Umbrella interval for all segments, since CompactionTasks generated a single granularitySpec.
-                      ImmutableList.of(Intervals.of("2014-01-01/2014-01-01T03:00:00"))
-                  )
-              ),
-              Map.class
-          )
+              // Umbrella interval for all segments, since CompactionTasks generated a single granularitySpec.
+              ImmutableList.of(Intervals.of("2014-01-01/2014-01-01T03:00:00"))
+          ),
+          null
       );
       Assert.assertEquals("Compaction state for " + segment.getId(), expectedState, segment.getLastCompactionState());
     }
@@ -420,10 +411,7 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
     final Set<DataSegment> compactedSegments = dataSegmentsWithSchemas.getSegments();
     for (DataSegment segment : compactedSegments) {
       // Expect compaction state to exist as store compaction state by default
-      Map<String, String> expectedLongSumMetric = new HashMap<>();
-      expectedLongSumMetric.put("type", "longSum");
-      expectedLongSumMetric.put("name", "val");
-      expectedLongSumMetric.put("fieldName", "val");
+      LongSumAggregatorFactory expectedLongSumMetric = new LongSumAggregatorFactory("val", "val");
       Assert.assertSame(DimensionRangeShardSpec.class, segment.getShardSpec().getClass());
       CompactionState expectedState = new CompactionState(
           new DimensionRangePartitionsSpec(7, null, Arrays.asList("dim1", "dim2"), false),
@@ -435,18 +423,14 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
           ),
           ImmutableList.of(expectedLongSumMetric),
           null,
-          compactionTask.getTuningConfig().getIndexSpec().asMap(getObjectMapper()),
-          getObjectMapper().readValue(
-              getObjectMapper().writeValueAsString(
-                  new UniformGranularitySpec(
-                      Granularities.HOUR,
-                      Granularities.MINUTE,
-                      true,
-                      ImmutableList.of(segment.getInterval())
-                  )
-              ),
-              Map.class
-          )
+          compactionTask.getTuningConfig().getIndexSpec(),
+          new UniformGranularitySpec(
+              Granularities.HOUR,
+              Granularities.MINUTE,
+              true,
+              ImmutableList.of(segment.getInterval())
+          ),
+          null
       );
       Assert.assertEquals("Compaction state for " + segment.getId(), expectedState, segment.getLastCompactionState());
     }
@@ -475,10 +459,7 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
     final Set<DataSegment> compactedSegments = dataSegmentsWithSchemas.getSegments();
     for (DataSegment segment : compactedSegments) {
       // Expect compaction state to exist as store compaction state by default
-      Map<String, String> expectedLongSumMetric = new HashMap<>();
-      expectedLongSumMetric.put("type", "longSum");
-      expectedLongSumMetric.put("name", "val");
-      expectedLongSumMetric.put("fieldName", "val");
+      LongSumAggregatorFactory expectedLongSumMetric = new LongSumAggregatorFactory("val", "val");
       Assert.assertSame(SingleDimensionShardSpec.class, segment.getShardSpec().getClass());
       CompactionState expectedState = new CompactionState(
           new SingleDimensionPartitionsSpec(7, null, "dim", false),
@@ -490,18 +471,14 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
           ),
           ImmutableList.of(expectedLongSumMetric),
           null,
-          compactionTask.getTuningConfig().getIndexSpec().asMap(getObjectMapper()),
-          getObjectMapper().readValue(
-              getObjectMapper().writeValueAsString(
-                  new UniformGranularitySpec(
-                      Granularities.HOUR,
-                      Granularities.MINUTE,
-                      true,
-                      ImmutableList.of(segment.getInterval())
-                  )
-              ),
-              Map.class
-          )
+          compactionTask.getTuningConfig().getIndexSpec(),
+          new UniformGranularitySpec(
+              Granularities.HOUR,
+              Granularities.MINUTE,
+              true,
+              ImmutableList.of(segment.getInterval())
+          ),
+          null
       );
       Assert.assertEquals("Compaction state for " + segment.getId(), expectedState, segment.getLastCompactionState());
     }
@@ -533,10 +510,7 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
     final Set<DataSegment> compactedSegments = dataSegmentsWithSchemas.getSegments();
     for (DataSegment segment : compactedSegments) {
       // Expect compaction state to exist as store compaction state by default
-      Map<String, String> expectedLongSumMetric = new HashMap<>();
-      expectedLongSumMetric.put("type", "longSum");
-      expectedLongSumMetric.put("name", "val");
-      expectedLongSumMetric.put("fieldName", "val");
+      LongSumAggregatorFactory expectedLongSumMetric = new LongSumAggregatorFactory("val", "val");
       Assert.assertSame(DimensionRangeShardSpec.class, segment.getShardSpec().getClass());
       CompactionState expectedState = new CompactionState(
           new DimensionRangePartitionsSpec(7, null, Arrays.asList("dim1", "dim2"), false),
@@ -548,18 +522,14 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
           ),
           ImmutableList.of(expectedLongSumMetric),
           null,
-          compactionTask.getTuningConfig().getIndexSpec().asMap(getObjectMapper()),
-          getObjectMapper().readValue(
-              getObjectMapper().writeValueAsString(
-                  new UniformGranularitySpec(
-                      Granularities.HOUR,
-                      Granularities.MINUTE,
-                      true,
-                      ImmutableList.of(segment.getInterval())
-                  )
-              ),
-              Map.class
-          )
+          compactionTask.getTuningConfig().getIndexSpec(),
+          new UniformGranularitySpec(
+              Granularities.HOUR,
+              Granularities.MINUTE,
+              true,
+              ImmutableList.of(segment.getInterval())
+          ),
+          null
       );
       Assert.assertEquals("Compaction state for " + segment.getId(), expectedState, segment.getLastCompactionState());
     }
@@ -608,7 +578,7 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
     final CompactionTask compactionTask = builder
         .inputSpec(new CompactionIntervalSpec(INTERVAL_TO_INDEX, null))
         .tuningConfig(AbstractParallelIndexSupervisorTaskTest.DEFAULT_TUNING_CONFIG_FOR_PARALLEL_INDEXING)
-        .transformSpec(new ClientCompactionTaskTransformSpec(new SelectorDimFilter("dim", "a", null)))
+        .transformSpec(new CompactionTransformSpec(new SelectorDimFilter("dim", "a", null)))
         .build();
 
     final DataSegmentsWithSchemas dataSegmentsWithSchemas = runTask(compactionTask);
@@ -622,10 +592,7 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
           lockGranularity == LockGranularity.TIME_CHUNK ? NumberedShardSpec.class : NumberedOverwriteShardSpec.class,
           segment.getShardSpec().getClass()
       );
-      Map<String, String> expectedLongSumMetric = new HashMap<>();
-      expectedLongSumMetric.put("type", "longSum");
-      expectedLongSumMetric.put("name", "val");
-      expectedLongSumMetric.put("fieldName", "val");
+      LongSumAggregatorFactory expectedLongSumMetric = new LongSumAggregatorFactory("val", "val");
       CompactionState expectedState = new CompactionState(
           new DynamicPartitionsSpec(null, Long.MAX_VALUE),
           new DimensionsSpec(
@@ -635,22 +602,15 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
               )
           ),
           ImmutableList.of(expectedLongSumMetric),
-          getObjectMapper().readValue(
-              getObjectMapper().writeValueAsString(compactionTask.getTransformSpec()),
-              Map.class
+          compactionTask.getTransformSpec(),
+          compactionTask.getTuningConfig().getIndexSpec(),
+          new UniformGranularitySpec(
+              Granularities.HOUR,
+              Granularities.MINUTE,
+              true,
+              ImmutableList.of(segment.getInterval())
           ),
-          compactionTask.getTuningConfig().getIndexSpec().asMap(getObjectMapper()),
-          getObjectMapper().readValue(
-              getObjectMapper().writeValueAsString(
-                  new UniformGranularitySpec(
-                      Granularities.HOUR,
-                      Granularities.MINUTE,
-                      true,
-                      ImmutableList.of(segment.getInterval())
-                  )
-              ),
-              Map.class
-          )
+          null
       );
       Assert.assertEquals("Compaction state for " + segment.getId(), expectedState, segment.getLastCompactionState());
     }
@@ -686,13 +646,8 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
           lockGranularity == LockGranularity.TIME_CHUNK ? NumberedShardSpec.class : NumberedOverwriteShardSpec.class,
           segment.getShardSpec().getClass()
       );
-      Map<String, String> expectedCountMetric = new HashMap<>();
-      expectedCountMetric.put("type", "count");
-      expectedCountMetric.put("name", "cnt");
-      Map<String, String> expectedLongSumMetric = new HashMap<>();
-      expectedLongSumMetric.put("type", "longSum");
-      expectedLongSumMetric.put("name", "val");
-      expectedLongSumMetric.put("fieldName", "val");
+      CountAggregatorFactory expectedCountMetric = new CountAggregatorFactory("cnt");
+      LongSumAggregatorFactory expectedLongSumMetric = new LongSumAggregatorFactory("val", "val");
       CompactionState expectedState = new CompactionState(
           new DynamicPartitionsSpec(null, Long.MAX_VALUE),
           new DimensionsSpec(
@@ -702,22 +657,15 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
               )
           ),
           ImmutableList.of(expectedCountMetric, expectedLongSumMetric),
-          getObjectMapper().readValue(
-              getObjectMapper().writeValueAsString(compactionTask.getTransformSpec()),
-              Map.class
+          compactionTask.getTransformSpec(),
+          compactionTask.getTuningConfig().getIndexSpec(),
+          new UniformGranularitySpec(
+              Granularities.HOUR,
+              Granularities.MINUTE,
+              true,
+              ImmutableList.of(segment.getInterval())
           ),
-          compactionTask.getTuningConfig().getIndexSpec().asMap(getObjectMapper()),
-          getObjectMapper().readValue(
-              getObjectMapper().writeValueAsString(
-                  new UniformGranularitySpec(
-                      Granularities.HOUR,
-                      Granularities.MINUTE,
-                      true,
-                      ImmutableList.of(segment.getInterval())
-                  )
-              ),
-              Map.class
-          )
+          null
       );
       Assert.assertEquals("Compaction state for " + segment.getId(), expectedState, segment.getLastCompactionState());
     }
@@ -948,6 +896,120 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
     Assert.assertEquals(3, minuteSegmentCount);
   }
 
+
+
+  @Test
+  public void testRunParallelWithProjections()
+  {
+    allowSegmentFetchesByCompactionTask = true;
+    runIndexTaskWithProjections(null, true);
+
+    final Builder builder = new Builder(
+        DATA_SOURCE,
+        getSegmentCacheManagerFactory()
+    );
+    final CompactionTask compactionTask = builder
+        .inputSpec(new CompactionIntervalSpec(INTERVAL_TO_INDEX, null))
+        .tuningConfig(AbstractParallelIndexSupervisorTaskTest.DEFAULT_TUNING_CONFIG_FOR_PARALLEL_INDEXING)
+        .build();
+
+    final DataSegmentsWithSchemas dataSegmentsWithSchemas = runTask(compactionTask);
+    verifySchema(dataSegmentsWithSchemas);
+    final Set<DataSegment> compactedSegments = dataSegmentsWithSchemas.getSegments();
+
+    for (DataSegment segment : compactedSegments) {
+      Assert.assertSame(
+          lockGranularity == LockGranularity.TIME_CHUNK ? NumberedShardSpec.class : NumberedOverwriteShardSpec.class,
+          segment.getShardSpec().getClass()
+      );
+      // Expect compaction state to exist as store compaction state by default
+      CompactionState expectedState = new CompactionState(
+          new DynamicPartitionsSpec(null, Long.MAX_VALUE),
+          new DimensionsSpec(
+              ImmutableList.of(
+                  new StringDimensionSchema("ts", DimensionSchema.MultiValueHandling.ARRAY, null),
+                  new StringDimensionSchema("dim", DimensionSchema.MultiValueHandling.ARRAY, null),
+                  new LongDimensionSchema("val")
+              )
+          ),
+          Collections.emptyList(),
+          null,
+          compactionTask.getTuningConfig().getIndexSpec(),
+          new UniformGranularitySpec(
+              Granularities.HOUR,
+              Granularities.MINUTE,
+              true,
+              ImmutableList.of(segment.getInterval())
+          ),
+          ImmutableList.of(PROJECTION_SPEC)
+      );
+      Assert.assertEquals("Compaction state for " + segment.getId(), expectedState, segment.getLastCompactionState());
+    }
+  }
+
+  @Test
+  public void testRunParallelAddProjections()
+  {
+    allowSegmentFetchesByCompactionTask = true;
+    runIndexTaskWithProjections(null, true);
+
+    final Builder builder = new Builder(
+        DATA_SOURCE,
+        getSegmentCacheManagerFactory()
+    );
+    final AggregateProjectionSpec addProjection = new AggregateProjectionSpec(
+        "projection2",
+        VirtualColumns.EMPTY,
+        null,
+        new AggregatorFactory[]{
+            new LongSumAggregatorFactory("val", "val")
+        }
+    );
+    final CompactionTask compactionTask = builder
+        .inputSpec(new CompactionIntervalSpec(INTERVAL_TO_INDEX, null))
+        .tuningConfig(AbstractParallelIndexSupervisorTaskTest.DEFAULT_TUNING_CONFIG_FOR_PARALLEL_INDEXING)
+        .projections(
+            ImmutableList.of(
+                PROJECTION_SPEC,
+                addProjection
+            )
+        )
+        .build();
+
+    final DataSegmentsWithSchemas dataSegmentsWithSchemas = runTask(compactionTask);
+    verifySchema(dataSegmentsWithSchemas);
+    final Set<DataSegment> compactedSegments = dataSegmentsWithSchemas.getSegments();
+
+    for (DataSegment segment : compactedSegments) {
+      Assert.assertSame(
+          lockGranularity == LockGranularity.TIME_CHUNK ? NumberedShardSpec.class : NumberedOverwriteShardSpec.class,
+          segment.getShardSpec().getClass()
+      );
+      // Expect compaction state to exist as store compaction state by default
+      CompactionState expectedState = new CompactionState(
+          new DynamicPartitionsSpec(null, Long.MAX_VALUE),
+          new DimensionsSpec(
+              ImmutableList.of(
+                  new StringDimensionSchema("ts", DimensionSchema.MultiValueHandling.ARRAY, null),
+                  new StringDimensionSchema("dim", DimensionSchema.MultiValueHandling.ARRAY, null),
+                  new LongDimensionSchema("val")
+              )
+          ),
+          Collections.emptyList(),
+          null,
+          compactionTask.getTuningConfig().getIndexSpec(),
+          new UniformGranularitySpec(
+              Granularities.HOUR,
+              Granularities.MINUTE,
+              true,
+              ImmutableList.of(segment.getInterval())
+          ),
+          ImmutableList.of(PROJECTION_SPEC, addProjection)
+      );
+      Assert.assertEquals("Compaction state for " + segment.getId(), expectedState, segment.getLastCompactionState());
+    }
+  }
+
   @Override
   protected TaskToolbox createTaskToolbox(Task task, TaskActionClient actionClient) throws IOException
   {
@@ -1003,10 +1065,79 @@ public class CompactionTaskParallelRunTest extends AbstractParallelIndexSupervis
 
     Assert.assertEquals(
         Collections.singleton(
-            new ResourceAction(new Resource(
-                LocalInputSource.TYPE_KEY,
-                ResourceType.EXTERNAL
-            ), Action.READ)),
+            new ResourceAction(
+                new Resource(
+                    LocalInputSource.TYPE_KEY,
+                    ResourceType.EXTERNAL
+                ), Action.READ
+            )),
+        indexTask.getInputSourceResources()
+    );
+
+    final DataSegmentsWithSchemas dataSegmentsWithSchemas = runTask(indexTask);
+    verifySchema(dataSegmentsWithSchemas);
+  }
+
+  private void runIndexTaskWithProjections(@Nullable PartitionsSpec partitionsSpec, boolean appendToExisting)
+  {
+    ParallelIndexIOConfig ioConfig = new ParallelIndexIOConfig(
+        new LocalInputSource(inputDir, "druid*"),
+        new CsvInputFormat(
+            Arrays.asList("ts", "dim", "val"),
+            "|",
+            null,
+            false,
+            0,
+            null
+        ),
+        appendToExisting,
+        null
+    );
+    ParallelIndexTuningConfig tuningConfig = newTuningConfig(partitionsSpec, 2, !appendToExisting);
+    ParallelIndexSupervisorTask indexTask = new ParallelIndexSupervisorTask(
+        null,
+        null,
+        null,
+        new ParallelIndexIngestionSpec(
+            DataSchema.builder()
+                      .withDataSource(DATA_SOURCE)
+                      .withTimestamp(new TimestampSpec("ts", "auto", null))
+                      .withDimensions(
+                          DimensionsSpec.builder()
+                                        .setDimensions(
+                                            ImmutableList.of(
+                                                new StringDimensionSchema("ts"),
+                                                new StringDimensionSchema("dim"),
+                                                new LongDimensionSchema("val")
+                                            )
+                                        )
+                                        .build()
+                      )
+                      .withProjections(
+                          ImmutableList.of(PROJECTION_SPEC)
+                      )
+                      .withGranularity(
+                          new UniformGranularitySpec(
+                              Granularities.HOUR,
+                              Granularities.MINUTE,
+                              ImmutableList.of(INTERVAL_TO_INDEX)
+                          )
+                      )
+                      .build(),
+            ioConfig,
+            tuningConfig
+        ),
+        null
+    );
+
+    Assert.assertEquals(
+        Collections.singleton(
+            new ResourceAction(
+                new Resource(
+                    LocalInputSource.TYPE_KEY,
+                    ResourceType.EXTERNAL
+                ), Action.READ
+            )),
         indexTask.getInputSourceResources()
     );
 
