@@ -62,7 +62,7 @@ public class TaskLockBoxConcurrencyTest
   private final ObjectMapper objectMapper = new DefaultObjectMapper();
   private ExecutorService service;
   private TaskStorage taskStorage;
-  private TaskLockbox lockbox;
+  private GlobalTaskLockbox lockbox;
   private SegmentSchemaManager segmentSchemaManager;
 
   @Before
@@ -81,7 +81,7 @@ public class TaskLockBoxConcurrencyTest
     );
 
     segmentSchemaManager = new SegmentSchemaManager(derby.metadataTablesConfigSupplier().get(), objectMapper, derbyConnector);
-    lockbox = new TaskLockbox(
+    lockbox = new GlobalTaskLockbox(
         taskStorage,
         new IndexerSQLMetadataStorageCoordinator(
             new SqlSegmentMetadataTransactionFactory(
@@ -99,6 +99,7 @@ public class TaskLockBoxConcurrencyTest
             CentralizedDatasourceSchemaConfig.create()
         )
     );
+    lockbox.syncFromStorage();
     service = Execs.multiThreaded(2, "TaskLockBoxConcurrencyTest-%d");
   }
 
