@@ -44,6 +44,7 @@ import org.apache.druid.java.util.common.logger.Logger;
 import javax.annotation.Nullable;
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -134,7 +135,7 @@ public class FrameProcessorExecutor
           logProcessorStatusString(processor, finished, allWritabilityFutures);
 
           if (!writabilityFuturesToWaitFor.isEmpty()) {
-            runProcessorAfterFutureResolves(Futures.allAsList(writabilityFuturesToWaitFor));
+            runProcessorAfterFutureResolves(Futures.allAsList(writabilityFuturesToWaitFor), false);
             return;
           }
 
@@ -150,13 +151,17 @@ public class FrameProcessorExecutor
 
           if (result.isReturn()) {
             succeed(result.value());
+          } else if (result.hasAwaitableFutures()) {
+            runProcessorAfterFutureResolves(Futures.allAsList(result.awaitableFutures()), true);
           } else {
+            assert result.hasAwaitableChannels();
+
             // Don't retain a reference to this set: it may be mutated the next time the processor runs.
-            final IntSet await = result.awaitSet();
+            final IntSet await = result.awaitableChannels();
 
             if (await.isEmpty()) {
               exec.execute(ExecutorRunnable.this);
-            } else if (result.isAwaitAll() || await.size() == 1) {
+            } else if (result.isAwaitAllChannels() || await.size() == 1) {
               final List<ListenableFuture<?>> readabilityFutures = new ArrayList<>();
 
               for (final int channelNumber : await) {
@@ -169,11 +174,11 @@ public class FrameProcessorExecutor
               if (readabilityFutures.isEmpty()) {
                 exec.execute(ExecutorRunnable.this);
               } else {
-                runProcessorAfterFutureResolves(Futures.allAsList(readabilityFutures));
+                runProcessorAfterFutureResolves(Futures.allAsList(readabilityFutures), false);
               }
             } else {
               // Await any.
-              runProcessorAfterFutureResolves(awaitAnyWidget.awaitAny(await));
+              runProcessorAfterFutureResolves(awaitAnyWidget.awaitAny(await), false);
             }
           }
         }
@@ -222,6 +227,7 @@ public class FrameProcessorExecutor
           }
         }
 
+        final String threadName = Thread.currentThread().getName();
         boolean canceled = false;
         Either<Throwable, ReturnOrAwait<T>> retVal;
 
@@ -230,6 +236,11 @@ public class FrameProcessorExecutor
             throw new InterruptedException();
           }
 
+          if (cancellationId != null) {
+            // Set the thread name to something involving the cancellationId, to make thread dumps more useful.
+            Thread.currentThread().setName(threadName + "-" + cancellationId);
+          }
+
           retVal = Either.value(processor.runIncrementally(readableInputs));
         }
         catch (Throwable e) {
@@ -253,6 +264,9 @@ public class FrameProcessorExecutor
                 canceled = true;
               }
             }
+
+            // Restore original thread name.
+            Thread.currentThread().setName(threadName);
           }
         }
 
@@ -263,13 +277,23 @@ public class FrameProcessorExecutor
         }
       }
 
-      private <V> void runProcessorAfterFutureResolves(final ListenableFuture<V> future)
+      /**
+       * Schedule this processor to run after the provided future resolves.
+       *
+       * @param future       the future
+       * @param failOnCancel whether the processor should be {@link #fail(Throwable)} if the future is itself canceled.
+       *                     This is true for futures provided by {@link ReturnOrAwait#awaitAllFutures(Collection)},
+       *                     because the processor has declared it wants to wait for them; if they are canceled
+       *                     the processor must fail. It is false for other futures, which the processor was not
+       *                     directly waiting for.
+       */
+      private <V> void runProcessorAfterFutureResolves(final ListenableFuture<V> future, final boolean failOnCancel)
       {
         final ListenableFuture<V> cancelableFuture = registerCancelableFuture(future, false, cancellationId);
 
         Futures.addCallback(
             cancelableFuture,
-            new FutureCallback<V>()
+            new FutureCallback<>()
             {
               @Override
               public void onSuccess(final V ignored)
@@ -285,8 +309,7 @@ public class FrameProcessorExecutor
               @Override
               public void onFailure(Throwable t)
               {
-                // Ignore cancellation.
-                if (!cancelableFuture.isCancelled()) {
+                if (failOnCancel || !cancelableFuture.isCancelled()) {
                   fail(t);
                 }
               }
