@@ -34,7 +34,6 @@ import org.apache.druid.java.util.common.logger.Logger;
 import org.checkerframework.checker.nullness.qual.MonotonicNonNull;
 
 import javax.inject.Inject;
-
 import java.io.File;
 import java.io.FilenameFilter;
 import java.io.IOException;
@@ -404,12 +403,23 @@ public class ExtensionsLoader
 
   private class ServiceLoadingFromExtensions<T>
   {
+    private final boolean isEmbeddedTest;
     private final Class<T> serviceClass;
     private final List<T> implsToLoad = new ArrayList<>();
     private final Set<String> implClassNamesToLoad = new HashSet<>();
 
     private ServiceLoadingFromExtensions(Class<T> serviceClass)
     {
+      this.isEmbeddedTest = extensionsConfig.getModulesForEmbeddedTest() != null;
+      if (isEmbeddedTest) {
+        log.warn(
+            "Running service in embedded testing mode with allowed modules[%s]."
+            + " This is an unsafe test-only mode and must never be used in a production cluster."
+            + " Remove property 'druid.extensions.modulesForEmbeddedTest' to disable embedded testing mode.",
+            extensionsConfig.getModulesForEmbeddedTest()
+        );
+      }
+
       this.serviceClass = serviceClass;
       if (extensionsConfig.searchCurrentClassloader()) {
         addAllFromCurrentClassLoader();
@@ -459,6 +469,11 @@ public class ExtensionsLoader
             + "is it a local or anonymous class?",
             serviceImpl.getClass().getName()
         );
+      } else if (isEmbeddedTest && !extensionsConfig.getModulesForEmbeddedTest().contains(serviceImplName)) {
+        log.debug(
+            "Skipping extension[%s] as it is not listed in config[%s]",
+            serviceImplName, extensionsConfig.getModulesForEmbeddedTest()
+        );
       } else if (!implClassNamesToLoad.contains(serviceImplName)) {
         log.debug(
             "Adding implementation %s for class %s from %s extension",
