@@ -21,23 +21,19 @@ package org.apache.druid.query;
 
 import org.apache.druid.frame.Frame;
 import org.apache.druid.frame.read.FrameReader;
-import org.apache.druid.java.util.common.IAE;
 import org.apache.druid.java.util.common.guava.Sequence;
 import org.apache.druid.java.util.common.guava.Sequences;
-import org.apache.druid.query.planning.DataSourceAnalysis;
 import org.apache.druid.segment.BaseObjectColumnValueSelector;
 import org.apache.druid.segment.ColumnSelectorFactory;
 import org.apache.druid.segment.Cursor;
 import org.apache.druid.segment.CursorBuildSpec;
 import org.apache.druid.segment.CursorHolder;
-import org.apache.druid.segment.SegmentReference;
 import org.apache.druid.segment.column.RowSignature;
 
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
-import java.util.function.Function;
 import java.util.stream.Collectors;
 
 /**
@@ -51,9 +47,8 @@ import java.util.stream.Collectors;
  * {@link #rowSignature}. For frames that donot contain the columns present in the {@link #rowSignature}, they are
  * populated with {@code null}.
  */
-public class FrameBasedInlineDataSource implements DataSource
+public class FrameBasedInlineDataSource extends LeafDataSource
 {
-
   final List<FrameSignaturePair> frames;
   final RowSignature rowSignature;
 
@@ -135,22 +130,6 @@ public class FrameBasedInlineDataSource implements DataSource
     return Collections.emptySet();
   }
 
-  @Override
-  public List<DataSource> getChildren()
-  {
-    return Collections.emptyList();
-  }
-
-  @Override
-  public DataSource withChildren(List<DataSource> children)
-  {
-    if (!children.isEmpty()) {
-      throw new IAE("Cannot accept children");
-    }
-
-    return this;
-  }
-
   @Override
   public boolean isCacheable(boolean isBroker)
   {
@@ -164,32 +143,15 @@ public class FrameBasedInlineDataSource implements DataSource
   }
 
   @Override
-  public boolean isConcrete()
+  public boolean isProcessable()
   {
     return true;
   }
 
-  @Override
-  public Function<SegmentReference, SegmentReference> createSegmentMapFunction(Query query)
-  {
-    return Function.identity();
-  }
-
-  @Override
-  public DataSource withUpdatedDataSource(DataSource newSource)
-  {
-    return newSource;
-  }
-
   @Override
   public byte[] getCacheKey()
   {
     return null;
   }
 
-  @Override
-  public DataSourceAnalysis getAnalysis()
-  {
-    return new DataSourceAnalysis(this, null, null, Collections.emptyList(), null);
-  }
 }
