@@ -46,6 +46,7 @@ import org.apache.druid.query.metadata.metadata.ColumnAnalysis;
 import org.apache.druid.query.metadata.metadata.ColumnIncluderator;
 import org.apache.druid.query.metadata.metadata.SegmentAnalysis;
 import org.apache.druid.query.metadata.metadata.SegmentMetadataQuery;
+import org.apache.druid.segment.AggregateProjectionMetadata;
 import org.apache.druid.segment.Metadata;
 import org.apache.druid.segment.PhysicalSegmentInspector;
 import org.apache.druid.segment.Segment;
@@ -57,10 +58,12 @@ import java.util.HashMap;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.concurrent.CancellationException;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
+import java.util.stream.Collectors;
 
 public class SegmentMetadataQueryRunnerFactory implements QueryRunnerFactory<SegmentAnalysis, SegmentMetadataQuery>
 {
@@ -83,7 +86,7 @@ public class SegmentMetadataQueryRunnerFactory implements QueryRunnerFactory<Seg
   @Override
   public QueryRunner<SegmentAnalysis> createRunner(final Segment segment)
   {
-    return new QueryRunner<SegmentAnalysis>()
+    return new QueryRunner<>()
     {
       @Override
       public Sequence<SegmentAnalysis> run(QueryPlus<SegmentAnalysis> inQ, ResponseContext responseContext)
@@ -132,6 +135,20 @@ public class SegmentMetadataQueryRunnerFactory implements QueryRunnerFactory<Seg
           aggregators = null;
         }
 
+        final Map<String, AggregateProjectionMetadata> projectionsMap;
+        if (updatedQuery.hasProjections()
+            && ((metadata = Objects.isNull(metadata) ? getMetadata(segment) : metadata)) != null
+            && metadata.getProjections() != null) {
+          projectionsMap = metadata.getProjections()
+                                   .stream()
+                                   .collect(Collectors.toUnmodifiableMap(
+                                       projectionMetadata -> projectionMetadata.getSchema().getName(),
+                                       p -> p
+                                   ));
+        } else {
+          projectionsMap = null;
+        }
+
         final TimestampSpec timestampSpec;
         if (updatedQuery.hasTimestampSpec()) {
           if (metadata == null) {
@@ -174,6 +191,7 @@ public class SegmentMetadataQueryRunnerFactory implements QueryRunnerFactory<Seg
                     totalSize,
                     numRows,
                     aggregators,
+                    projectionsMap,
                     timestampSpec,
                     queryGranularity,
                     rollup
@@ -190,15 +208,15 @@ public class SegmentMetadataQueryRunnerFactory implements QueryRunnerFactory<Seg
       Iterable<QueryRunner<SegmentAnalysis>> queryRunners
   )
   {
-    return new ConcatQueryRunner<SegmentAnalysis>(
+    return new ConcatQueryRunner<>(
         Sequences.map(
             Sequences.simple(queryRunners),
-            new Function<QueryRunner<SegmentAnalysis>, QueryRunner<SegmentAnalysis>>()
+            new Function<>()
             {
               @Override
               public QueryRunner<SegmentAnalysis> apply(final QueryRunner<SegmentAnalysis> input)
               {
-                return new QueryRunner<SegmentAnalysis>()
+                return new QueryRunner<>()
                 {
                   @Override
                   public Sequence<SegmentAnalysis> run(
@@ -210,7 +228,7 @@ public class SegmentMetadataQueryRunnerFactory implements QueryRunnerFactory<Seg
                     final int priority = query.context().getPriority();
                     final QueryPlus<SegmentAnalysis> threadSafeQueryPlus = queryPlus.withoutThreadUnsafeState();
                     ListenableFuture<Sequence<SegmentAnalysis>> future = queryProcessingPool.submitRunnerTask(
-                        new AbstractPrioritizedQueryRunnerCallable<Sequence<SegmentAnalysis>, SegmentAnalysis>(priority, input)
+                        new AbstractPrioritizedQueryRunnerCallable<>(priority, input)
                         {
                           @Override
                           public Sequence<SegmentAnalysis> call()
@@ -239,7 +257,10 @@ public class SegmentMetadataQueryRunnerFactory implements QueryRunnerFactory<Seg
                     catch (TimeoutException e) {
                       log.info("Query timeout, cancelling pending results for query id [%s]", query.getId());
                       future.cancel(true);
-                      throw new QueryTimeoutException(StringUtils.nonStrictFormat("Query [%s] timed out", query.getId()));
+                      throw new QueryTimeoutException(StringUtils.nonStrictFormat(
+                          "Query [%s] timed out",
+                          query.getId()
+                      ));
                     }
                     catch (ExecutionException e) {
                       throw new RuntimeException(e);
