@@ -217,6 +217,12 @@ public class DefaultSearchQueryMetrics implements SearchQueryMetrics
     return delegateQueryMetrics.reportSegmentAndCacheTime(timeNs);
   }
 
+  @Override
+  public QueryMetrics reportResultCachePoll(boolean hit)
+  {
+    return delegateQueryMetrics.reportResultCachePoll(hit);
+  }
+
   @Override
   public QueryMetrics reportCpuTime(long timeNs)
   {
