@@ -45,11 +45,11 @@ import org.joda.time.Interval;
 import javax.annotation.Nullable;
 import java.util.List;
 import java.util.Map;
-import java.util.stream.Collectors;
+
 
 /**
  * Metadata of Druid's data segment. An immutable object.
- *
+ * <p>
  * DataSegment's equality ({@link #equals}/{@link #hashCode}) and {@link #compareTo} methods consider only the
  * {@link SegmentId} of the segment.
  */
@@ -86,6 +86,7 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
   private static final Interner<String> STRING_INTERNER = Interners.newWeakInterner();
   private static final Interner<List<String>> DIMENSIONS_INTERNER = Interners.newWeakInterner();
   private static final Interner<List<String>> METRICS_INTERNER = Interners.newWeakInterner();
+  private static final Interner<List<String>> PROJECTIONS_INTERNER = Interners.newWeakInterner();
   private static final Interner<CompactionState> COMPACTION_STATE_INTERNER = Interners.newWeakInterner();
   private static final Map<String, Object> PRUNED_LOAD_SPEC = ImmutableMap.of(
       "load spec is pruned, because it's not needed on Brokers, but eats a lot of heap space",
@@ -98,6 +99,7 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
   private final Map<String, Object> loadSpec;
   private final List<String> dimensions;
   private final List<String> metrics;
+  private final List<String> projections;
   private final ShardSpec shardSpec;
 
   /**
@@ -111,7 +113,10 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
   private final CompactionState lastCompactionState;
   private final long size;
 
-  @VisibleForTesting
+  /**
+   * @deprecated use {@link #builder(SegmentId)} or {@link #builder(DataSegment)} instead.
+   */
+  @Deprecated
   public DataSegment(
       SegmentId segmentId,
       Map<String, Object> loadSpec,
@@ -130,13 +135,19 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
         loadSpec,
         dimensions,
         metrics,
+        null,
         shardSpec,
         lastCompactionState,
         binaryVersion,
-        size
+        size,
+        PruneSpecsHolder.DEFAULT
     );
   }
 
+  /**
+   * @deprecated use {@link #builder(SegmentId)} or {@link #builder(DataSegment)} instead.
+   */
+  @Deprecated
   public DataSegment(
       String dataSource,
       Interval interval,
@@ -156,13 +167,19 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
         loadSpec,
         dimensions,
         metrics,
+        null,
         shardSpec,
         null,
         binaryVersion,
-        size
+        size,
+        PruneSpecsHolder.DEFAULT
     );
   }
 
+  /**
+   * @deprecated use {@link #builder(SegmentId)} or {@link #builder(DataSegment)} instead.
+   */
+  @Deprecated
   public DataSegment(
       String dataSource,
       Interval interval,
@@ -183,6 +200,7 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
         loadSpec,
         dimensions,
         metrics,
+        null,
         shardSpec,
         lastCompactionState,
         binaryVersion,
@@ -198,14 +216,11 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
       @JsonProperty("version") String version,
       // use `Map` *NOT* `LoadSpec` because we want to do lazy materialization to prevent dependency pollution
       @JsonProperty("loadSpec") @Nullable Map<String, Object> loadSpec,
-      @JsonProperty("dimensions")
-      @JsonDeserialize(using = CommaListJoinDeserializer.class)
-      @Nullable
-          List<String> dimensions,
-      @JsonProperty("metrics")
-      @JsonDeserialize(using = CommaListJoinDeserializer.class)
-      @Nullable
-          List<String> metrics,
+      @JsonProperty("dimensions") @JsonDeserialize(using = CommaListJoinDeserializer.class) @Nullable
+      List<String> dimensions,
+      @JsonProperty("metrics") @JsonDeserialize(using = CommaListJoinDeserializer.class) @Nullable List<String> metrics,
+      @JsonProperty("projections") @JsonDeserialize(using = CommaListJoinDeserializer.class) @Nullable
+      List<String> projections,
       @JsonProperty("shardSpec") @Nullable ShardSpec shardSpec,
       @JsonProperty("lastCompactionState") @Nullable CompactionState lastCompactionState,
       @JsonProperty("binaryVersion") Integer binaryVersion,
@@ -216,10 +231,11 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
     this.id = SegmentId.of(dataSource, interval, version, shardSpec);
     // prune loadspec if needed
     this.loadSpec = pruneSpecsHolder.pruneLoadSpec ? PRUNED_LOAD_SPEC : prepareLoadSpec(loadSpec);
-    // Deduplicating dimensions and metrics lists as a whole because they are very likely the same for the same
-    // dataSource
-    this.dimensions = prepareDimensionsOrMetrics(dimensions, DIMENSIONS_INTERNER);
-    this.metrics = prepareDimensionsOrMetrics(metrics, METRICS_INTERNER);
+    this.dimensions = dimensions == null ? ImmutableList.of() : prepareWithInterner(dimensions, DIMENSIONS_INTERNER);
+    this.metrics = metrics == null ? ImmutableList.of() : prepareWithInterner(metrics, METRICS_INTERNER);
+    // A null value for projections means that this segment is not aware of projections (launched in druid 32).
+    // An empty list means that this segment is projection-aware, but has no projections.
+    this.projections = projections == null ? null : prepareWithInterner(projections, PROJECTIONS_INTERNER);
     this.shardSpec = (shardSpec == null) ? new NumberedShardSpec(0, 1) : shardSpec;
     this.lastCompactionState = pruneSpecsHolder.pruneLastCompactionState
                                ? null
@@ -229,46 +245,6 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
     this.size = size;
   }
 
-  @Nullable
-  private Map<String, Object> prepareLoadSpec(@Nullable Map<String, Object> loadSpec)
-  {
-    if (loadSpec == null) {
-      return null;
-    }
-    // Load spec is just of 3 entries on average; HashMap/LinkedHashMap consumes much more memory than ArrayMap
-    Map<String, Object> result = new Object2ObjectArrayMap<>(loadSpec.size());
-    for (Map.Entry<String, Object> e : loadSpec.entrySet()) {
-      result.put(STRING_INTERNER.intern(e.getKey()), e.getValue());
-    }
-    return result;
-  }
-
-  @Nullable
-  private CompactionState prepareCompactionState(@Nullable CompactionState lastCompactionState)
-  {
-    if (lastCompactionState == null) {
-      return null;
-    }
-    return COMPACTION_STATE_INTERNER.intern(lastCompactionState);
-  }
-
-  private List<String> prepareDimensionsOrMetrics(@Nullable List<String> list, Interner<List<String>> interner)
-  {
-    if (list == null) {
-      return ImmutableList.of();
-    } else {
-      List<String> result = list
-          .stream()
-          .filter(s -> !Strings.isNullOrEmpty(s))
-          // dimensions & metrics are stored as canonical string values to decrease memory required for storing
-          // large numbers of segments.
-          .map(STRING_INTERNER::intern)
-          // TODO replace with ImmutableList.toImmutableList() when updated to Guava 21+
-          .collect(Collectors.collectingAndThen(Collectors.toList(), ImmutableList::copyOf));
-      return interner.intern(result);
-    }
-  }
-
   /**
    * Get dataSource
    *
@@ -314,6 +290,15 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
     return metrics;
   }
 
+  @Nullable
+  @JsonProperty
+  @JsonInclude(JsonInclude.Include.NON_NULL)
+  @JsonSerialize(using = CommaListJoinSerializer.class)
+  public List<String> getProjections()
+  {
+    return projections;
+  }
+
   @JsonProperty
   public ShardSpec getShardSpec()
   {
@@ -416,6 +401,11 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
     return builder(this).metrics(metrics).build();
   }
 
+  public DataSegment withProjections(List<String> projections)
+  {
+    return builder(this).projections(projections).build();
+  }
+
   public DataSegment withShardSpec(ShardSpec newSpec)
   {
     return builder(this).shardSpec(newSpec).build();
@@ -471,17 +461,65 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
            ", loadSpec=" + loadSpec +
            ", dimensions=" + dimensions +
            ", metrics=" + metrics +
+           ", projections=" + projections +
            ", shardSpec=" + shardSpec +
            ", lastCompactionState=" + lastCompactionState +
            ", size=" + size +
            '}';
   }
 
+
+  @Nullable
+  private static Map<String, Object> prepareLoadSpec(@Nullable Map<String, Object> loadSpec)
+  {
+    if (loadSpec == null) {
+      return null;
+    }
+    // Load spec is just of 3 entries on average; HashMap/LinkedHashMap consumes much more memory than ArrayMap
+    Map<String, Object> result = new Object2ObjectArrayMap<>(loadSpec.size());
+    for (Map.Entry<String, Object> e : loadSpec.entrySet()) {
+      result.put(STRING_INTERNER.intern(e.getKey()), e.getValue());
+    }
+    return result;
+  }
+
+  @Nullable
+  private static CompactionState prepareCompactionState(@Nullable CompactionState lastCompactionState)
+  {
+    if (lastCompactionState == null) {
+      return null;
+    }
+    return COMPACTION_STATE_INTERNER.intern(lastCompactionState);
+  }
+
+  /**
+   * Returns a list of strings with all empty strings removed and all strings interned.
+   * <p>
+   * The dimensions, metrics, and projections are stored as canonical string values to decrease memory required for
+   * storing large numbers of segments.
+   */
+  private static List<String> prepareWithInterner(List<String> list, Interner<List<String>> interner)
+  {
+    return interner.intern(list.stream()
+                               .filter(s -> !Strings.isNullOrEmpty(s))
+                               .map(STRING_INTERNER::intern)
+                               .collect(ImmutableList.toImmutableList()));
+  }
+
+  /**
+   * @deprecated use {@link #builder(SegmentId)} or {@link #builder(DataSegment)} instead.
+   */
+  @Deprecated
   public static Builder builder()
   {
     return new Builder();
   }
 
+  public static Builder builder(SegmentId segmentId)
+  {
+    return new Builder(segmentId);
+  }
+
   public static Builder builder(DataSegment segment)
   {
     return new Builder(segment);
@@ -495,21 +533,39 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
     private Map<String, Object> loadSpec;
     private List<String> dimensions;
     private List<String> metrics;
+    private List<String> projections;
     private ShardSpec shardSpec;
     private CompactionState lastCompactionState;
     private Integer binaryVersion;
     private long size;
 
-    public Builder()
+    /**
+     * @deprecated use {@link #Builder(SegmentId)} or {@link #Builder(DataSegment)} instead.
+     */
+    @Deprecated
+    private Builder()
     {
       this.loadSpec = ImmutableMap.of();
       this.dimensions = ImmutableList.of();
       this.metrics = ImmutableList.of();
+      // By default, segment is not projection-aware.
+      this.projections = null;
       this.shardSpec = new NumberedShardSpec(0, 1);
       this.size = -1;
     }
 
-    public Builder(DataSegment segment)
+    private Builder(SegmentId segmentId)
+    {
+      this.dataSource = segmentId.getDataSource();
+      this.interval = segmentId.getInterval();
+      this.version = segmentId.getVersion();
+      this.shardSpec = new NumberedShardSpec(0, 1);
+      this.binaryVersion = 0;
+      this.size = 0;
+      this.lastCompactionState = null;
+    }
+
+    private Builder(DataSegment segment)
     {
       this.dataSource = segment.getDataSource();
       this.interval = segment.getInterval();
@@ -517,6 +573,7 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
       this.loadSpec = segment.getLoadSpec();
       this.dimensions = segment.getDimensions();
       this.metrics = segment.getMetrics();
+      this.projections = segment.getProjections();
       this.shardSpec = segment.getShardSpec();
       this.lastCompactionState = segment.getLastCompactionState();
       this.binaryVersion = segment.getBinaryVersion();
@@ -559,6 +616,12 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
       return this;
     }
 
+    public Builder projections(List<String> projections)
+    {
+      this.projections = projections;
+      return this;
+    }
+
     public Builder shardSpec(ShardSpec shardSpec)
     {
       this.shardSpec = shardSpec;
@@ -598,10 +661,12 @@ public class DataSegment implements Comparable<DataSegment>, Overshadowable<Data
           loadSpec,
           dimensions,
           metrics,
+          projections,
           shardSpec,
           lastCompactionState,
           binaryVersion,
-          size
+          size,
+          PruneSpecsHolder.DEFAULT
       );
     }
   }
