@@ -30,18 +30,19 @@ import org.apache.druid.query.aggregation.LongSumAggregatorFactory;
 import org.apache.druid.segment.TestHelper;
 import org.apache.druid.segment.VirtualColumns;
 import org.apache.druid.testing.InitializedNullHandlingTest;
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.Test;
 
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.List;
 
-public class AggregateProjectionSpecTest extends InitializedNullHandlingTest
+class AggregateProjectionSpecTest extends InitializedNullHandlingTest
 {
   private static final ObjectMapper JSON_MAPPER = TestHelper.makeJsonMapper();
 
   @Test
-  public void testSerde() throws JsonProcessingException
+  void testSerde() throws JsonProcessingException
   {
     AggregateProjectionSpec spec = new AggregateProjectionSpec(
         "some_projection",
@@ -60,13 +61,40 @@ public class AggregateProjectionSpecTest extends InitializedNullHandlingTest
             new LongSumAggregatorFactory("e", "e")
         }
     );
-    Assert.assertEquals(spec, JSON_MAPPER.readValue(JSON_MAPPER.writeValueAsString(spec), AggregateProjectionSpec.class));
+    Assertions.assertEquals(spec, JSON_MAPPER.readValue(JSON_MAPPER.writeValueAsString(spec), AggregateProjectionSpec.class));
   }
 
   @Test
-  public void testInvalidGrouping()
+  void testMissingName()
   {
-    Throwable t = Assert.assertThrows(
+    Throwable t = Assertions.assertThrows(
+        DruidException.class,
+        () -> new AggregateProjectionSpec(
+            null,
+            VirtualColumns.EMPTY,
+            List.of(new StringDimensionSchema("string")),
+            null
+        )
+    );
+
+    Assertions.assertEquals("projection name cannot be null or empty", t.getMessage());
+
+    t = Assertions.assertThrows(
+        DruidException.class,
+        () -> new AggregateProjectionSpec(
+            "",
+            VirtualColumns.EMPTY,
+            List.of(new StringDimensionSchema("string")),
+            null
+        )
+    );
+    Assertions.assertEquals("projection name cannot be null or empty", t.getMessage());
+  }
+
+  @Test
+  void testInvalidGrouping()
+  {
+    Throwable t = Assertions.assertThrows(
         DruidException.class,
         () -> new AggregateProjectionSpec(
             "other_projection",
@@ -75,9 +103,9 @@ public class AggregateProjectionSpecTest extends InitializedNullHandlingTest
             null
         )
     );
-    Assert.assertEquals("groupingColumns and aggregators must not both be null or empty", t.getMessage());
+    Assertions.assertEquals("projection[other_projection] groupingColumns and aggregators must not both be null or empty", t.getMessage());
 
-    t = Assert.assertThrows(
+    t = Assertions.assertThrows(
         DruidException.class,
         () -> new AggregateProjectionSpec(
             "other_projection",
@@ -86,11 +114,11 @@ public class AggregateProjectionSpecTest extends InitializedNullHandlingTest
             null
         )
     );
-    Assert.assertEquals("groupingColumns and aggregators must not both be null or empty", t.getMessage());
+    Assertions.assertEquals("projection[other_projection] groupingColumns and aggregators must not both be null or empty", t.getMessage());
   }
 
   @Test
-  public void testEqualsAndHashcode()
+  void testEqualsAndHashcode()
   {
     EqualsVerifier.forClass(AggregateProjectionSpec.class)
                   .usingGetClass()
