@@ -31,6 +31,7 @@ import org.apache.druid.java.util.common.granularity.Granularity;
 import org.apache.druid.java.util.common.guava.MergeSequence;
 import org.apache.druid.java.util.common.guava.Sequence;
 import org.apache.druid.java.util.common.guava.Sequences;
+import org.apache.druid.java.util.common.io.Closer;
 import org.apache.druid.js.JavaScriptConfig;
 import org.apache.druid.math.expr.ExprMacroTable;
 import org.apache.druid.query.aggregation.AggregatorFactory;
@@ -53,6 +54,8 @@ import org.apache.druid.query.aggregation.post.ConstantPostAggregator;
 import org.apache.druid.query.aggregation.post.FieldAccessPostAggregator;
 import org.apache.druid.query.dimension.DefaultDimensionSpec;
 import org.apache.druid.query.filter.SelectorDimFilter;
+import org.apache.druid.query.planning.ExecutionVertex;
+import org.apache.druid.query.policy.NoopPolicyEnforcer;
 import org.apache.druid.query.spec.MultipleIntervalSegmentSpec;
 import org.apache.druid.query.spec.QuerySegmentSpec;
 import org.apache.druid.query.spec.SpecificSegmentSpec;
@@ -61,14 +64,14 @@ import org.apache.druid.query.timeseries.TimeseriesQueryQueryToolChest;
 import org.apache.druid.query.timeseries.TimeseriesQueryRunnerFactory;
 import org.apache.druid.segment.IncrementalIndexSegment;
 import org.apache.druid.segment.QueryableIndexSegment;
-import org.apache.druid.segment.ReferenceCountingSegment;
+import org.apache.druid.segment.ReferenceCountedSegmentProvider;
 import org.apache.druid.segment.Segment;
-import org.apache.druid.segment.SegmentReference;
 import org.apache.druid.segment.TestIndex;
 import org.apache.druid.segment.virtual.ExpressionVirtualColumn;
 import org.apache.druid.timeline.SegmentId;
 import org.apache.druid.timeline.TimelineObjectHolder;
 import org.apache.druid.timeline.VersionedIntervalTimeline;
+import org.apache.druid.utils.CloseableUtils;
 import org.joda.time.DateTime;
 import org.joda.time.Interval;
 
@@ -79,6 +82,7 @@ import java.util.Iterator;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.function.BiFunction;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
@@ -556,16 +560,15 @@ public class QueryRunnerTestHelper
       final String runnerName
   )
   {
-    final DataSource base = query.getDataSource();
-
-    final SegmentReference segmentReference = base.createSegmentMapFunction(query)
-                                                  .apply(ReferenceCountingSegment.wrapRootGenerationSegment(adapter));
-    return makeQueryRunner(factory, segmentReference, runnerName);
+    ExecutionVertex ev = ExecutionVertex.of(query);
+    final Optional<Segment> segmentReference = ev.createSegmentMapFunction(NoopPolicyEnforcer.instance())
+                                                 .apply(ReferenceCountedSegmentProvider.wrapRootGenerationSegment(adapter));
+    return makeQueryRunner(factory, segmentReference.orElseThrow(), runnerName);
   }
 
   @SuppressWarnings({"rawtypes", "unchecked"})
   public static <T> QueryRunner<T> makeFilteringQueryRunner(
-      final VersionedIntervalTimeline<String, ReferenceCountingSegment> timeline,
+      final VersionedIntervalTimeline<String, ReferenceCountedSegmentProvider> timeline,
       final QueryRunnerFactory<T, Query<T>> factory
   )
   {
@@ -579,25 +582,33 @@ public class QueryRunnerTestHelper
                 segments.addAll(timeline.lookup(interval));
               }
               List<Sequence<T>> sequences = new ArrayList<>();
-              for (TimelineObjectHolder<String, ReferenceCountingSegment> holder : toolChest.filterSegments(
-                  query,
-                  segments
-              )) {
-                Segment segment = holder.getObject().getChunk(0).getObject();
-                QueryPlus queryPlusRunning = queryPlus.withQuery(
-                    queryPlus.getQuery().withQuerySegmentSpec(
-                        new SpecificSegmentSpec(
-                            new SegmentDescriptor(
-                                holder.getInterval(),
-                                holder.getVersion(),
-                                0
-                            )
-                        )
-                    )
+              final Closer closer = Closer.create();
+              try {
+                for (TimelineObjectHolder<String, ReferenceCountedSegmentProvider> holder : toolChest.filterSegments(
+                    query,
+                    segments
+                )) {
+                  final SegmentDescriptor descriptor = new SegmentDescriptor(
+                      holder.getInterval(),
+                      holder.getVersion(),
+                      0
+                  );
+                  final QueryPlus queryPlusRunning = queryPlus.withQuery(
+                      queryPlus.getQuery().withQuerySegmentSpec(new SpecificSegmentSpec(descriptor))
+                  );
+                  final QueryRunner<?> runner = factory.createRunner(
+                      closer.register(holder.getObject().getChunk(0).getObject().acquireReference().orElseThrow())
+                  );
+                  sequences.add(runner.run(queryPlusRunning, responseContext));
+                }
+                return Sequences.withBaggage(
+                    new MergeSequence<>(query.getResultOrdering(), Sequences.simple(sequences)),
+                    closer
                 );
-                sequences.add(factory.createRunner(segment).run(queryPlusRunning, responseContext));
               }
-              return new MergeSequence<>(query.getResultOrdering(), Sequences.simple(sequences));
+              catch (Throwable t) {
+                throw CloseableUtils.closeAndWrapInCatch(t, closer);
+              }
             },
             toolChest
         )
