@@ -197,6 +197,21 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
               new LongLastAggregatorFactory("_c_last", "c", null)
           }
       ),
+      new AggregateProjectionSpec(
+          "b_hourly_c_sum_non_time_ordered",
+          VirtualColumns.create(
+              Granularities.toVirtualColumn(Granularities.HOUR, "__gran")
+          ),
+          Arrays.asList(
+              new StringDimensionSchema("b"),
+              new LongDimensionSchema("__gran")
+          ),
+          new AggregatorFactory[]{
+              new CountAggregatorFactory("chocula"),
+              new LongSumAggregatorFactory("_c_sum", "c"),
+              new LongLastAggregatorFactory("_c_last", "c", null)
+          }
+      ),
       new AggregateProjectionSpec(
           "bf_daily_c_sum",
           VirtualColumns.create(
@@ -212,7 +227,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
           }
       ),
       new AggregateProjectionSpec(
-          "ab_daily",
+          "ab",
           null,
           Arrays.asList(
               new StringDimensionSchema("a"),
@@ -221,7 +236,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
           null
       ),
       new AggregateProjectionSpec(
-          "abfoo_daily",
+          "abfoo",
           VirtualColumns.create(
               new ExpressionVirtualColumn(
                   "bfoo",
@@ -278,7 +293,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
           }
       ),
       new AggregateProjectionSpec(
-          "afoo_daily",
+          "afoo",
           VirtualColumns.create(
               new ExpressionVirtualColumn(
                   "afoo",
@@ -322,7 +337,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
                         ))
                         .collect(Collectors.toList());
 
-  @Parameterized.Parameters(name = "name: {0}, sortByDim: {5}, autoSchema: {6}")
+  @Parameterized.Parameters(name = "name: {0}, segmentTimeOrdered: {5}, autoSchema: {6}")
   public static Collection<?> constructorFeeder()
   {
     final List<Object[]> constructors = new ArrayList<>();
@@ -405,7 +420,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
                   new IncrementalIndexTimeBoundaryInspector(index),
                   new IncrementalIndexCursorFactory(rollupIndex),
                   new IncrementalIndexTimeBoundaryInspector(rollupIndex),
-                  sortByDim,
+                  !sortByDim,
                   autoSchema
               });
             } else {
@@ -419,7 +434,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
                   QueryableIndexTimeBoundaryInspector.create(index),
                   new QueryableIndexCursorFactory(rollupIndex),
                   QueryableIndexTimeBoundaryInspector.create(rollupIndex),
-                  sortByDim,
+                  !sortByDim,
                   autoSchema
               });
             }
@@ -446,7 +461,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
   private final TimeseriesQueryEngine timeseriesEngine;
 
   private final NonBlockingPool<ByteBuffer> nonBlockingPool;
-  public final boolean sortByDim;
+  public final boolean segmentSortedByTime;
   public final boolean autoSchema;
 
   @Rule
@@ -458,7 +473,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
       TimeBoundaryInspector projectionsTimeBoundaryInspector,
       CursorFactory rollupProjectionsCursorFactory,
       TimeBoundaryInspector rollupProjectionsTimeBoundaryInspector,
-      boolean sortByDim,
+      boolean segmentSortedByTime,
       boolean autoSchema
   )
   {
@@ -466,7 +481,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
     this.projectionsTimeBoundaryInspector = projectionsTimeBoundaryInspector;
     this.rollupProjectionsCursorFactory = rollupProjectionsCursorFactory;
     this.rollupProjectionsTimeBoundaryInspector = rollupProjectionsTimeBoundaryInspector;
-    this.sortByDim = sortByDim;
+    this.segmentSortedByTime = segmentSortedByTime;
     this.autoSchema = autoSchema;
     this.nonBlockingPool = closer.closeLater(
         new CloseableStupidPool<>(
@@ -584,7 +599,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
                             10
                         )
                     )
-                    .setContext(ImmutableMap.of(QueryContexts.USE_PROJECTION, "abfoo_daily"))
+                    .setContext(ImmutableMap.of(QueryContexts.USE_PROJECTION, "abfoo"))
                     .build();
 
     final CursorBuildSpec buildSpec = GroupingEngine.makeCursorBuildSpec(query, null);
@@ -964,16 +979,16 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
                     .setDataSource("test")
                     .setInterval(Intervals.ETERNITY)
                     .addAggregator(new LongSumAggregatorFactory("c_sum", "c"));
-    if (sortByDim) {
+    if (segmentSortedByTime) {
+      queryBuilder.addDimension("a")
+                  .setGranularity(Granularities.MINUTE);
+    } else {
       queryBuilder.setVirtualColumns(Granularities.toVirtualColumn(Granularities.MINUTE, "__gran"))
                   .setDimensions(
                       DefaultDimensionSpec.of("__gran", ColumnType.LONG),
                       DefaultDimensionSpec.of("a")
                   )
                   .setGranularity(Granularities.ALL);
-    } else {
-      queryBuilder.addDimension("a")
-                  .setGranularity(Granularities.MINUTE);
     }
     final GroupByQuery query = queryBuilder.build();
     final CursorBuildSpec buildSpec = GroupingEngine.makeCursorBuildSpec(query, null);
@@ -998,7 +1013,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
     final List<ResultRow> results = resultRows.toList();
     Assert.assertEquals(8, results.size());
 
-    if (sortByDim && projectionsCursorFactory instanceof QueryableIndexCursorFactory) {
+    if (!segmentSortedByTime && projectionsCursorFactory instanceof QueryableIndexCursorFactory) {
       // this sorts funny when not time ordered
       Set<Object[]> resultsInNoParticularOrder = makeArrayResultSet();
       resultsInNoParticularOrder.addAll(
@@ -1053,16 +1068,16 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
                     .setDataSource("test")
                     .setInterval(Intervals.ETERNITY)
                     .addAggregator(new LongSumAggregatorFactory("c_sum", "c"));
-    if (sortByDim) {
+    if (segmentSortedByTime) {
+      queryBuilder.addDimension("a")
+                  .setGranularity(Granularities.HOUR);
+    } else {
       queryBuilder.setGranularity(Granularities.ALL)
                   .setDimensions(
                       DefaultDimensionSpec.of("__gran", ColumnType.LONG),
                       DefaultDimensionSpec.of("a")
                   )
                   .setVirtualColumns(Granularities.toVirtualColumn(Granularities.HOUR, "__gran"));
-    } else {
-      queryBuilder.addDimension("a")
-                  .setGranularity(Granularities.HOUR);
     }
     final GroupByQuery query = queryBuilder.build();
     final CursorBuildSpec buildSpec = GroupingEngine.makeCursorBuildSpec(query, null);
@@ -1086,7 +1101,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
 
     final List<ResultRow> results = resultRows.toList();
     Assert.assertEquals(3, results.size());
-    if (sortByDim && projectionsCursorFactory instanceof QueryableIndexCursorFactory) {
+    if (!segmentSortedByTime && projectionsCursorFactory instanceof QueryableIndexCursorFactory) {
       Set<Object[]> resultsInNoParticularOrder = makeArrayResultSet(
           new Object[]{TIMESTAMP.getMillis(), "a", 4L},
           new Object[]{TIMESTAMP.getMillis(), "b", 12L},
@@ -1102,6 +1117,115 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
     }
   }
 
+  @Test
+  public void testQueryGranularityFitsProjectionGranularityNotTimeOrdered()
+  {
+    final GroupByQuery.Builder queryBuilder =
+        GroupByQuery.builder()
+                    .setDataSource("test")
+                    .setInterval(Intervals.ETERNITY)
+                    .addAggregator(new LongSumAggregatorFactory("c_sum", "c"));
+    if (segmentSortedByTime) {
+      queryBuilder.addDimension("b")
+                  .setGranularity(Granularities.HOUR);
+    } else {
+      queryBuilder.setGranularity(Granularities.ALL)
+                  .setDimensions(
+                      DefaultDimensionSpec.of("__gran", ColumnType.LONG),
+                      DefaultDimensionSpec.of("b")
+                  )
+                  .setVirtualColumns(Granularities.toVirtualColumn(Granularities.HOUR, "__gran"));
+    }
+    final GroupByQuery query = queryBuilder.build();
+    final CursorBuildSpec buildSpec = GroupingEngine.makeCursorBuildSpec(query, null);
+    try (final CursorHolder cursorHolder = projectionsCursorFactory.makeCursorHolder(buildSpec)) {
+      final Cursor cursor = cursorHolder.asCursor();
+      int rowCount = 0;
+      while (!cursor.isDone()) {
+        rowCount++;
+        cursor.advance();
+      }
+      Assert.assertEquals(segmentSortedByTime ? 8 : 5, rowCount);
+    }
+
+    final Sequence<ResultRow> resultRows = groupingEngine.process(
+        query,
+        projectionsCursorFactory,
+        projectionsTimeBoundaryInspector,
+        nonBlockingPool,
+        null
+    );
+
+    final List<ResultRow> results = resultRows.toList();
+    Assert.assertEquals(5, results.size());
+    Set<Object[]> resultsInNoParticularOrder = makeArrayResultSet(
+        new Object[]{TIMESTAMP.getMillis(), "aa", 8L},
+        new Object[]{TIMESTAMP.getMillis(), "bb", 6L},
+        new Object[]{TIMESTAMP.getMillis(), "cc", 2L},
+        new Object[]{TIMESTAMP.plusHours(1).getMillis(), "aa", 1L},
+        new Object[]{TIMESTAMP.plusHours(1).getMillis(), "dd", 2L}
+    );
+    for (ResultRow row : results) {
+      Assert.assertTrue("missing row" + row.toString(), resultsInNoParticularOrder.contains(row.getArray()));
+    }
+  }
+
+
+  @Test
+  public void testQueryGranularityLargerProjectionGranularity()
+  {
+    final GroupByQuery.Builder queryBuilder =
+        GroupByQuery.builder()
+                    .setDataSource("test")
+                    .setInterval(Intervals.ETERNITY)
+                    .addAggregator(new LongSumAggregatorFactory("c_sum", "c"));
+    if (segmentSortedByTime) {
+      queryBuilder.addDimension("a")
+                  .setGranularity(Granularities.DAY);
+    } else {
+      queryBuilder.setGranularity(Granularities.ALL)
+                  .setDimensions(
+                      DefaultDimensionSpec.of("__gran", ColumnType.LONG),
+                      DefaultDimensionSpec.of("a")
+                  )
+                  .setVirtualColumns(Granularities.toVirtualColumn(Granularities.DAY, "__gran"));
+    }
+    final GroupByQuery query = queryBuilder.build();
+    final CursorBuildSpec buildSpec = GroupingEngine.makeCursorBuildSpec(query, null);
+    try (final CursorHolder cursorHolder = projectionsCursorFactory.makeCursorHolder(buildSpec)) {
+      final Cursor cursor = cursorHolder.asCursor();
+      int rowCount = 0;
+      while (!cursor.isDone()) {
+        rowCount++;
+        cursor.advance();
+      }
+      Assert.assertEquals(3, rowCount);
+    }
+
+    final Sequence<ResultRow> resultRows = groupingEngine.process(
+        query,
+        projectionsCursorFactory,
+        projectionsTimeBoundaryInspector,
+        nonBlockingPool,
+        null
+    );
+
+    final List<ResultRow> results = resultRows.toList();
+    Assert.assertEquals(2, results.size());
+    if (!segmentSortedByTime && projectionsCursorFactory instanceof QueryableIndexCursorFactory) {
+      Set<Object[]> resultsInNoParticularOrder = makeArrayResultSet(
+          new Object[]{TIMESTAMP.getMillis(), "a", 7L},
+          new Object[]{TIMESTAMP.getMillis(), "b", 12L}
+      );
+      for (ResultRow row : results) {
+        Assert.assertTrue(resultsInNoParticularOrder.contains(row.getArray()));
+      }
+    } else {
+      Assert.assertArrayEquals(new Object[]{TIMESTAMP.getMillis(), "a", 7L}, results.get(0).getArray());
+      Assert.assertArrayEquals(new Object[]{TIMESTAMP.getMillis(), "b", 12L}, results.get(1).getArray());
+    }
+  }
+
   @Test
   public void testProjectionSelectionMissingAggregatorButHasAggregatorInput()
   {
@@ -1327,7 +1451,7 @@ public class CursorFactoryProjectionTest extends InitializedNullHandlingTest
   @Test
   public void testTimeseriesQueryGranularityFinerThanProjectionGranularity()
   {
-    Assume.assumeFalse(sortByDim);
+    Assume.assumeTrue(segmentSortedByTime);
     final TimeseriesQuery query = Druids.newTimeseriesQueryBuilder()
                                         .dataSource("test")
                                         .intervals(ImmutableList.of(Intervals.ETERNITY))
