@@ -40,12 +40,10 @@ import org.apache.druid.segment.column.ColumnConfig;
 import org.apache.druid.segment.column.RowSignature;
 import org.apache.druid.segment.incremental.IncrementalIndex;
 import org.apache.druid.segment.incremental.IncrementalIndexSchema;
-import org.apache.druid.segment.incremental.IndexSizeExceededException;
 import org.apache.druid.segment.incremental.OnheapIncrementalIndex;
 import org.apache.druid.segment.transform.TransformSpec;
 import org.apache.druid.segment.writeout.OffHeapMemorySegmentWriteOutMediumFactory;
 import org.apache.druid.segment.writeout.SegmentWriteOutMediumFactory;
-import org.apache.druid.timeline.SegmentId;
 import org.joda.time.Interval;
 
 import javax.annotation.Nonnull;
@@ -406,7 +404,6 @@ public class IndexBuilder
   public RowBasedSegment<InputRow> buildRowBasedSegmentWithoutTypeSignature()
   {
     return new RowBasedSegment<>(
-        SegmentId.dummy("IndexBuilder"),
         Sequences.simple(rows),
         RowAdapters.standardRow(),
         RowSignature.empty()
@@ -420,7 +417,6 @@ public class IndexBuilder
       final RowSignature signature = new QueryableIndexCursorFactory(index).getRowSignature();
 
       return new RowBasedSegment<>(
-          SegmentId.dummy("IndexBuilder"),
           Sequences.simple(rows),
           RowAdapters.standardRow(),
           signature
@@ -434,8 +430,7 @@ public class IndexBuilder
     try (final QueryableIndex index = buildMMappedIndex()) {
       return FrameTestUtil.cursorFactoryToFrameSegment(
           new QueryableIndexCursorFactory(index),
-          frameType,
-          SegmentId.dummy("IndexBuilder")
+          frameType
       );
     }
   }
@@ -453,13 +448,8 @@ public class IndexBuilder
         .build();
 
     while (rows.hasNext()) {
-      try {
-        InputRow row = rows.next();
-        incrementalIndex.add(row);
-      }
-      catch (IndexSizeExceededException e) {
-        throw new RuntimeException(e);
-      }
+      InputRow row = rows.next();
+      incrementalIndex.add(row);
     }
     return incrementalIndex;
   }
