@@ -145,7 +145,7 @@ public class SpatialFilterTest extends InitializedNullHandlingTest
     );
   }
 
-  private static IncrementalIndex makeIncrementalIndex() throws IOException
+  private static IncrementalIndex makeIncrementalIndex()
   {
     IncrementalIndex theIndex = new OnheapIncrementalIndex.Builder()
         .setIndexSchema(
