@@ -25,7 +25,6 @@ import com.google.common.util.concurrent.Futures;
 import com.google.common.util.concurrent.ListenableFuture;
 import com.google.common.util.concurrent.ListeningExecutorService;
 import com.google.common.util.concurrent.MoreExecutors;
-import org.apache.druid.common.config.NullHandling;
 import org.apache.druid.guice.BuiltInTypesModule;
 import org.apache.druid.java.util.common.concurrent.Execs;
 import org.apache.druid.java.util.common.io.Closer;
@@ -219,30 +218,35 @@ public class ScalarLongColumnSupplierTest extends InitializedNullHandlingTest
 
     final int threads = 10;
     ListeningExecutorService executorService = MoreExecutors.listeningDecorator(
-        Execs.multiThreaded(threads, "StandardNestedColumnSupplierTest-%d")
+        Execs.multiThreaded(threads, "ScalarLongColumnSupplierTest-%d")
     );
-    Collection<ListenableFuture<?>> futures = new ArrayList<>(threads);
-    final CountDownLatch threadsStartLatch = new CountDownLatch(1);
-    for (int i = 0; i < threads; ++i) {
-      futures.add(
-          executorService.submit(() -> {
-            try {
-              threadsStartLatch.await();
-              for (int iter = 0; iter < 5000; iter++) {
-                try (ScalarLongColumn column = (ScalarLongColumn) supplier.get()) {
-                  smokeTest(supplier, column);
+    try {
+      Collection<ListenableFuture<?>> futures = new ArrayList<>(threads);
+      final CountDownLatch threadsStartLatch = new CountDownLatch(1);
+      for (int i = 0; i < threads; ++i) {
+        futures.add(
+            executorService.submit(() -> {
+              try {
+                threadsStartLatch.await();
+                for (int iter = 0; iter < 5000; iter++) {
+                  try (ScalarLongColumn column = (ScalarLongColumn) supplier.get()) {
+                    smokeTest(supplier, column);
+                  }
                 }
               }
-            }
-            catch (Throwable ex) {
-              failureReason.set(ex.getMessage());
-            }
-          })
-      );
+              catch (Throwable ex) {
+                failureReason.set(ex.getMessage());
+              }
+            })
+        );
+      }
+      threadsStartLatch.countDown();
+      Futures.allAsList(futures).get();
+      Assert.assertEquals(expectedReason, failureReason.get());
+    }
+    finally {
+      executorService.shutdownNow();
     }
-    threadsStartLatch.countDown();
-    Futures.allAsList(futures).get();
-    Assert.assertEquals(expectedReason, failureReason.get());
   }
 
   private void smokeTest(ScalarLongColumnAndIndexSupplier supplier, ScalarLongColumn column)
@@ -277,7 +281,7 @@ public class ScalarLongColumnSupplierTest extends InitializedNullHandlingTest
         Assert.assertEquals(row.doubleValue(), vectorValueSelector.getDoubleVector()[0], 0.0);
         Assert.assertEquals(row.floatValue(), vectorValueSelector.getFloatVector()[0], 0.0);
         boolean[] nullVector = vectorValueSelector.getNullVector();
-        if (NullHandling.sqlCompatible() && nullVector != null) {
+        if (nullVector != null) {
           Assert.assertFalse(nullVector[0]);
         } else {
           Assert.assertNull(nullVector);
@@ -301,30 +305,14 @@ public class ScalarLongColumnSupplierTest extends InitializedNullHandlingTest
         Assert.assertFalse(nullValueIndex.get().computeBitmapResult(resultFactory, false).get(i));
 
       } else {
-        if (NullHandling.sqlCompatible()) {
-          Assert.assertNull(valueSelector.getObject());
-          Assert.assertTrue(valueSelector.isNull());
-          Assert.assertTrue(vectorValueSelector.getNullVector()[0]);
-          Assert.assertTrue(valueSetIndex.forValue(null).computeBitmapResult(resultFactory, false).get(i));
-          Assert.assertTrue(nullValueIndex.get().computeBitmapResult(resultFactory, false).get(i));
-          Assert.assertTrue(predicateIndex.forPredicate(new SelectorPredicateFactory(null))
-                                          .computeBitmapResult(resultFactory, false)
-                                          .get(i));
-        } else {
-          Assert.assertEquals(NullHandling.defaultLongValue(), valueSelector.getObject());
-          Assert.assertFalse(valueSelector.isNull());
-          Assert.assertNull(vectorValueSelector.getNullVector());
-          Assert.assertFalse(valueSetIndex.forValue(null).computeBitmapResult(resultFactory, false).get(i));
-          Assert.assertFalse(nullValueIndex.get().computeBitmapResult(resultFactory, false).get(i));
-          Assert.assertFalse(predicateIndex.forPredicate(new SelectorPredicateFactory(null))
-                                           .computeBitmapResult(resultFactory, false)
-                                           .get(i));
-          final String defaultString = String.valueOf(NullHandling.defaultLongValue());
-          Assert.assertTrue(valueSetIndex.forValue(defaultString).computeBitmapResult(resultFactory, false).get(i));
-          Assert.assertTrue(predicateIndex.forPredicate(new SelectorPredicateFactory(defaultString))
-                                          .computeBitmapResult(resultFactory, false)
-                                          .get(i));
-        }
+        Assert.assertNull(valueSelector.getObject());
+        Assert.assertTrue(valueSelector.isNull());
+        Assert.assertTrue(vectorValueSelector.getNullVector()[0]);
+        Assert.assertTrue(valueSetIndex.forValue(null).computeBitmapResult(resultFactory, false).get(i));
+        Assert.assertTrue(nullValueIndex.get().computeBitmapResult(resultFactory, false).get(i));
+        Assert.assertTrue(predicateIndex.forPredicate(new SelectorPredicateFactory(null))
+                                        .computeBitmapResult(resultFactory, false)
+                                        .get(i));
 
         Assert.assertFalse(valueSetIndex.forValue(NO_MATCH).computeBitmapResult(resultFactory, false).get(i));
         Assert.assertFalse(valueSetIndex.forValue(NO_MATCH).computeBitmapResult(resultFactory, false).get(i));
