@@ -26,7 +26,7 @@ import org.apache.druid.query.DataSource;
 import org.apache.druid.query.GlobalTableDataSource;
 import org.apache.druid.segment.join.table.IndexedTable;
 import org.apache.druid.segment.join.table.IndexedTableJoinable;
-import org.apache.druid.segment.join.table.ReferenceCountingIndexedTable;
+import org.apache.druid.segment.join.table.ReferenceCountedIndexedTableProvider;
 import org.apache.druid.server.SegmentManager;
 
 import java.util.Iterator;
@@ -70,11 +70,11 @@ public class BroadcastTableJoinableFactory implements JoinableFactory
     return getOnlyIndexedTable(dataSource).filter(IndexedTable::isCacheable).map(IndexedTable::computeCacheKey);
   }
 
-  private Optional<ReferenceCountingIndexedTable> getOnlyIndexedTable(DataSource dataSource)
+  private Optional<ReferenceCountedIndexedTableProvider> getOnlyIndexedTable(DataSource dataSource)
   {
     GlobalTableDataSource broadcastDataSource = (GlobalTableDataSource) dataSource;
     return segmentManager.getIndexedTables(broadcastDataSource).flatMap(tables -> {
-      Iterator<ReferenceCountingIndexedTable> tableIterator = tables.iterator();
+      Iterator<ReferenceCountedIndexedTableProvider> tableIterator = tables.iterator();
       if (!tableIterator.hasNext()) {
         return Optional.empty();
       }
