@@ -48,8 +48,7 @@ import org.apache.druid.query.QuerySegmentWalker;
 import org.apache.druid.query.QueryTimeoutException;
 import org.apache.druid.query.QueryToolChest;
 import org.apache.druid.query.context.ResponseContext;
-import org.apache.druid.query.metadata.metadata.SegmentMetadataQuery;
-import org.apache.druid.server.QueryResource.ResourceIOReaderWriter;
+import org.apache.druid.query.policy.PolicyEnforcer;
 import org.apache.druid.server.log.RequestLogger;
 import org.apache.druid.server.security.Action;
 import org.apache.druid.server.security.AuthConfig;
@@ -99,6 +98,7 @@ public class QueryLifecycle
   private final AuthorizerMapper authorizerMapper;
   private final DefaultQueryConfig defaultQueryConfig;
   private final AuthConfig authConfig;
+  private final PolicyEnforcer policyEnforcer;
   private final long startMs;
   private final long startNs;
 
@@ -120,6 +120,7 @@ public class QueryLifecycle
       final AuthorizerMapper authorizerMapper,
       final DefaultQueryConfig defaultQueryConfig,
       final AuthConfig authConfig,
+      final PolicyEnforcer policyEnforcer,
       final long startMs,
       final long startNs
   )
@@ -132,6 +133,7 @@ public class QueryLifecycle
     this.authorizerMapper = authorizerMapper;
     this.defaultQueryConfig = defaultQueryConfig;
     this.authConfig = authConfig;
+    this.policyEnforcer = policyEnforcer;
     this.startMs = startMs;
     this.startNs = startNs;
   }
@@ -321,12 +323,11 @@ public class QueryLifecycle
       transition(State.AUTHORIZING, State.UNAUTHORIZED);
     } else {
       transition(State.AUTHORIZING, State.AUTHORIZED);
-      if (this.baseQuery instanceof SegmentMetadataQuery && authorizationResult.allowAccessWithNoRestriction()) {
-        // skip restrictions mapping for SegmentMetadataQuery from user with no restriction
-      } else {
-        this.baseQuery = this.baseQuery.withDataSource(this.baseQuery.getDataSource()
-                                                                     .withPolicies(authorizationResult.getPolicyMap()));
-      }
+      this.baseQuery = this.baseQuery.withDataSource(baseQuery.getDataSource()
+                                                              .withPolicies(
+                                                                  authorizationResult.getPolicyMap(),
+                                                                  policyEnforcer
+                                                              ));
     }
 
     this.authenticationResult = authenticationResult;
@@ -479,7 +480,7 @@ public class QueryLifecycle
            || (!shouldFinalize && queryContext.isSerializeDateTimeAsLongInner(false));
   }
 
-  public ObjectMapper newOutputWriter(ResourceIOReaderWriter ioReaderWriter)
+  public ObjectMapper newOutputWriter(ResourceIOReaderWriterFactory.ResourceIOReaderWriter ioReaderWriter)
   {
     return ioReaderWriter.getResponseWriter().newOutputWriter(
         getToolChest(),
