@@ -30,7 +30,6 @@ import org.apache.druid.initialization.DruidModule;
 import org.apache.druid.java.util.common.StringUtils;
 import org.apache.druid.server.http.security.ConfigResourceFilter;
 import org.apache.druid.server.http.security.StateResourceFilter;
-import org.apache.druid.utils.JvmUtils;
 import org.apache.druid.utils.RuntimeInfo;
 
 import javax.annotation.Nonnull;
@@ -41,7 +40,6 @@ import javax.ws.rs.Path;
 import javax.ws.rs.Produces;
 import javax.ws.rs.core.Context;
 import javax.ws.rs.core.MediaType;
-
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
@@ -59,17 +57,20 @@ public class StatusResource
   private final Properties properties;
   private final DruidServerConfig druidServerConfig;
   private final ExtensionsLoader extnLoader;
+  private final RuntimeInfo runtimeInfo;
 
   @Inject
   public StatusResource(
       final Properties properties,
       final DruidServerConfig druidServerConfig,
-      final ExtensionsLoader extnLoader
+      final ExtensionsLoader extnLoader,
+      final RuntimeInfo runtimeInfo
   )
   {
     this.properties = properties;
     this.druidServerConfig = druidServerConfig;
     this.extnLoader = extnLoader;
+    this.runtimeInfo = runtimeInfo;
   }
 
   @GET
@@ -115,7 +116,7 @@ public class StatusResource
       @Context final HttpServletRequest req
   )
   {
-    return new Status(extnLoader.getLoadedModules());
+    return new Status(extnLoader.getLoadedModules(), runtimeInfo);
   }
 
   /**
@@ -136,11 +137,11 @@ public class StatusResource
     final List<ModuleVersion> modules;
     final Memory memory;
 
-    public Status(Collection<DruidModule> modules)
+    public Status(Collection<DruidModule> modules, RuntimeInfo runtimeInfo)
     {
       this.version = getDruidVersion();
       this.modules = getExtensionVersions(modules);
-      this.memory = new Memory(JvmUtils.getRuntimeInfo());
+      this.memory = new Memory(runtimeInfo);
     }
 
     private String getDruidVersion()
