@@ -57,7 +57,7 @@ public class HttpInputSourceDefnTest extends BaseExternTableTest
   {
     mapper.setInjectableValues(new InjectableValues.Std().addValue(
         HttpInputSourceConfig.class,
-        new HttpInputSourceConfig(HttpInputSourceConfig.DEFAULT_ALLOWED_PROTOCOLS)
+        new HttpInputSourceConfig(HttpInputSourceConfig.DEFAULT_ALLOWED_PROTOCOLS, null)
     ));
   }
 
@@ -99,7 +99,8 @@ public class HttpInputSourceDefnTest extends BaseExternTableTest
         null,
         null,
         null,
-        new HttpInputSourceConfig(null)
+        null,
+        new HttpInputSourceConfig(null, null)
     );
     TableMetadata table = TableBuilder.external("foo")
         .inputSource(toMap(inputSource))
@@ -119,7 +120,8 @@ public class HttpInputSourceDefnTest extends BaseExternTableTest
         null,
         null,
         null,
-        new HttpInputSourceConfig(null)
+        null,
+        new HttpInputSourceConfig(null, null)
     );
     TableMetadata table = TableBuilder.external("foo")
         .inputSource(toMap(inputSource))
@@ -150,7 +152,8 @@ public class HttpInputSourceDefnTest extends BaseExternTableTest
         null,
         null,
         null,
-        new HttpInputSourceConfig(null)
+        null,
+        new HttpInputSourceConfig(null, null)
     );
     TableMetadata table = TableBuilder.external("foo")
         .inputSource(toMap(inputSource))
@@ -216,7 +219,8 @@ public class HttpInputSourceDefnTest extends BaseExternTableTest
         "bob",
         new DefaultPasswordProvider("secret"),
         null,
-        new HttpInputSourceConfig(null)
+        null,
+        new HttpInputSourceConfig(null, null)
     );
     TableMetadata table = TableBuilder.external("foo")
         .inputSource(toMap(inputSource))
@@ -272,11 +276,12 @@ public class HttpInputSourceDefnTest extends BaseExternTableTest
 
     // Get the partial table function
     TableFunction fn = externDefn.tableFn(resolved);
-    assertEquals(4, fn.parameters().size());
+    assertEquals(5, fn.parameters().size());
     assertTrue(hasParam(fn, HttpInputSourceDefn.URIS_PARAMETER));
     assertTrue(hasParam(fn, HttpInputSourceDefn.USER_PARAMETER));
     assertTrue(hasParam(fn, HttpInputSourceDefn.PASSWORD_PARAMETER));
     assertTrue(hasParam(fn, HttpInputSourceDefn.PASSWORD_ENV_VAR_PARAMETER));
+    assertTrue(hasParam(fn, HttpInputSourceDefn.HEADERS));
 
     // Convert to an external table.
     ExternalTableSpec externSpec = fn.apply(
@@ -320,8 +325,9 @@ public class HttpInputSourceDefnTest extends BaseExternTableTest
 
     // Get the partial table function
     TableFunction fn = externDefn.tableFn(resolved);
-    assertEquals(1, fn.parameters().size());
+    assertEquals(2, fn.parameters().size());
     assertTrue(hasParam(fn, HttpInputSourceDefn.URIS_PARAMETER));
+    assertTrue(hasParam(fn, HttpInputSourceDefn.HEADERS));
 
     // Convert to an external table.
     ExternalTableSpec externSpec = fn.apply(
@@ -344,7 +350,8 @@ public class HttpInputSourceDefnTest extends BaseExternTableTest
         "bob",
         new DefaultPasswordProvider("secret"),
         null,
-        new HttpInputSourceConfig(null)
+         null,
+        new HttpInputSourceConfig(null, null)
     );
     TableMetadata table = TableBuilder.external("foo")
         .inputSource(httpToMap(inputSource))
@@ -382,7 +389,8 @@ public class HttpInputSourceDefnTest extends BaseExternTableTest
         "bob",
         new EnvironmentVariablePasswordProvider("SECRET"),
         null,
-        new HttpInputSourceConfig(null)
+        null,
+        new HttpInputSourceConfig(null, null)
     );
     TableMetadata table = TableBuilder.external("foo")
         .inputSource(toMap(inputSource))
@@ -415,7 +423,8 @@ public class HttpInputSourceDefnTest extends BaseExternTableTest
         "bob",
         new DefaultPasswordProvider("secret"),
         null,
-        new HttpInputSourceConfig(null)
+        null,
+        new HttpInputSourceConfig(null, null)
     );
     TableMetadata table = TableBuilder.external("foo")
         .inputSource(httpToMap(inputSource))
@@ -484,7 +493,8 @@ public class HttpInputSourceDefnTest extends BaseExternTableTest
         "bob",
         new EnvironmentVariablePasswordProvider("SECRET"),
         null,
-        new HttpInputSourceConfig(null)
+        null,
+        new HttpInputSourceConfig(null, null)
     );
     TableMetadata table = TableBuilder.external("foo")
         .inputSource(toMap(inputSource))
@@ -518,7 +528,6 @@ public class HttpInputSourceDefnTest extends BaseExternTableTest
       assertEquals("secret", ((DefaultPasswordProvider) sourceSpec.getHttpAuthenticationPasswordProvider()).getPassword());
     }
     assertEquals("http://foo.com/my.csv", sourceSpec.getUris().get(0).toString());
-
     // Just a sanity check: details of CSV conversion are tested elsewhere.
     CsvInputFormat csvFormat = (CsvInputFormat) externSpec.inputFormat;
     assertEquals(Arrays.asList("x", "y"), csvFormat.getColumns());
