@@ -31,9 +31,7 @@ import org.apache.druid.java.util.common.Intervals;
 import org.apache.druid.java.util.common.StringUtils;
 import org.apache.druid.java.util.common.concurrent.Execs;
 import org.apache.druid.java.util.emitter.EmittingLogger;
-import org.apache.druid.segment.CursorFactory;
 import org.apache.druid.segment.PhysicalSegmentInspector;
-import org.apache.druid.segment.QueryableIndex;
 import org.apache.druid.segment.Segment;
 import org.apache.druid.segment.SegmentLazyLoadFailCallback;
 import org.apache.druid.segment.TestIndex;
@@ -60,7 +58,6 @@ import org.junit.Test;
 import org.junit.rules.TemporaryFolder;
 import org.mockito.Mockito;
 
-import javax.annotation.Nullable;
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
@@ -261,19 +258,6 @@ public class SegmentManagerThreadSafetyTest
           return segment.getInterval();
         }
 
-        @Nullable
-        @Override
-        public QueryableIndex asQueryableIndex()
-        {
-          throw new UnsupportedOperationException();
-        }
-
-        @Override
-        public CursorFactory asCursorFactory()
-        {
-          throw new UnsupportedOperationException();
-        }
-
         @Override
         public <T> T as(Class<T> clazz)
         {
