@@ -20,7 +20,9 @@
 package org.apache.druid.server.http;
 
 import com.fasterxml.jackson.databind.ObjectMapper;
+import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
+import nl.jqno.equalsverifier.EqualsVerifier;
 import org.apache.druid.segment.TestHelper;
 import org.apache.druid.server.coordinator.CoordinatorDynamicConfig;
 import org.apache.druid.utils.JvmUtils;
@@ -28,6 +30,7 @@ import org.junit.Assert;
 import org.junit.Test;
 
 import javax.annotation.Nullable;
+import java.util.Map;
 import java.util.Set;
 
 public class CoordinatorDynamicConfigTest
@@ -52,7 +55,8 @@ public class CoordinatorDynamicConfigTest
                      + "  \"decommissioningNodes\": [\"host1\", \"host2\"],\n"
                      + "  \"pauseCoordination\": false,\n"
                      + "  \"replicateAfterLoadTimeout\": false,\n"
-                     + "  \"turboLoadingNodes\":[\"host1\", \"host3\"]\n"
+                     + "  \"turboLoadingNodes\":[\"host1\", \"host3\"],\n"
+                     + "  \"cloneServers\":{\"host5\": \"host6\"}\n"
                      + "}\n";
 
     CoordinatorDynamicConfig actual = mapper.readValue(
@@ -67,6 +71,7 @@ public class CoordinatorDynamicConfigTest
     ImmutableSet<String> decommissioning = ImmutableSet.of("host1", "host2");
     ImmutableSet<String> whitelist = ImmutableSet.of("test1", "test2");
     ImmutableSet<String> turboLoadingNodes = ImmutableSet.of("host1", "host3");
+    ImmutableMap<String, String> cloneServers = ImmutableMap.of("host5", "host6");
     assertConfig(
         actual,
         1,
@@ -81,7 +86,8 @@ public class CoordinatorDynamicConfigTest
         decommissioning,
         false,
         false,
-        turboLoadingNodes
+        turboLoadingNodes,
+        cloneServers
     );
 
     actual = CoordinatorDynamicConfig.builder().withDecommissioningNodes(ImmutableSet.of("host1")).build(actual);
@@ -99,7 +105,8 @@ public class CoordinatorDynamicConfigTest
         ImmutableSet.of("host1"),
         false,
         false,
-        turboLoadingNodes
+        turboLoadingNodes,
+        cloneServers
     );
 
     actual = CoordinatorDynamicConfig.builder().build(actual);
@@ -117,7 +124,8 @@ public class CoordinatorDynamicConfigTest
         ImmutableSet.of("host1"),
         false,
         false,
-        turboLoadingNodes
+        turboLoadingNodes,
+        cloneServers
     );
 
     actual = CoordinatorDynamicConfig.builder().withPauseCoordination(true).build(actual);
@@ -135,7 +143,8 @@ public class CoordinatorDynamicConfigTest
         ImmutableSet.of("host1"),
         true,
         false,
-        turboLoadingNodes
+        turboLoadingNodes,
+        cloneServers
     );
 
     actual = CoordinatorDynamicConfig.builder().withReplicateAfterLoadTimeout(true).build(actual);
@@ -153,7 +162,8 @@ public class CoordinatorDynamicConfigTest
         ImmutableSet.of("host1"),
         true,
         true,
-        turboLoadingNodes
+        turboLoadingNodes,
+        cloneServers
     );
 
     actual = CoordinatorDynamicConfig.builder().build(actual);
@@ -171,7 +181,8 @@ public class CoordinatorDynamicConfigTest
         ImmutableSet.of("host1"),
         true,
         true,
-        turboLoadingNodes
+        turboLoadingNodes,
+        cloneServers
     );
 
     actual = CoordinatorDynamicConfig.builder().withKillTaskSlotRatio(0.1).build(actual);
@@ -189,7 +200,8 @@ public class CoordinatorDynamicConfigTest
         ImmutableSet.of("host1"),
         true,
         true,
-        turboLoadingNodes
+        turboLoadingNodes,
+        cloneServers
     );
 
     actual = CoordinatorDynamicConfig.builder().withMaxKillTaskSlots(5).build(actual);
@@ -207,7 +219,29 @@ public class CoordinatorDynamicConfigTest
         ImmutableSet.of("host1"),
         true,
         true,
-        turboLoadingNodes
+        turboLoadingNodes,
+        cloneServers
+    );
+
+    actual = CoordinatorDynamicConfig.builder()
+                                     .withTurboLoadingNodes(ImmutableSet.of("host3"))
+                                     .withCloneServers(ImmutableMap.of("host3", "host4")).build(actual);
+    assertConfig(
+        actual,
+        1,
+        1,
+        1,
+        1,
+        2,
+        whitelist,
+        0.1,
+        5,
+        1,
+        ImmutableSet.of("host1"),
+        true,
+        true,
+        ImmutableSet.of("host3"),
+        ImmutableMap.of("host3", "host4")
     );
   }
 
@@ -244,7 +278,8 @@ public class CoordinatorDynamicConfigTest
         false,
         false,
         null,
-        ImmutableSet.of("host1")
+        ImmutableSet.of("host1"),
+        null
     );
     Assert.assertTrue(config.getSpecificDataSourcesToKillUnusedSegmentsIn().isEmpty());
   }
@@ -269,7 +304,8 @@ public class CoordinatorDynamicConfigTest
         false,
         false,
         null,
-        ImmutableSet.of("host1")
+        ImmutableSet.of("host1"),
+        null
     );
     Assert.assertEquals(ImmutableSet.of("test1"), config.getSpecificDataSourcesToKillUnusedSegmentsIn());
   }
@@ -285,7 +321,8 @@ public class CoordinatorDynamicConfigTest
                      + "  \"balancerComputeThreads\": 2, \n"
                      + "  \"killDataSourceWhitelist\": [\"test1\",\"test2\"],\n"
                      + "  \"maxSegmentsInNodeLoadingQueue\": 1,\n"
-                     + "  \"turboLoadingNodes\": [\"host3\",\"host4\"]\n"
+                     + "  \"turboLoadingNodes\": [\"host3\",\"host4\"],\n"
+                     + "  \"cloneServers\": {\"host3\":\"host4\", \"host5\":\"host6\"}\n"
                      + "}\n";
 
     CoordinatorDynamicConfig actual = mapper.readValue(
@@ -300,6 +337,7 @@ public class CoordinatorDynamicConfigTest
     ImmutableSet<String> decommissioning = ImmutableSet.of();
     ImmutableSet<String> whitelist = ImmutableSet.of("test1", "test2");
     ImmutableSet<String> turboLoading = ImmutableSet.of("host3", "host4");
+    ImmutableMap<String, String> cloneServers = ImmutableMap.of("host3", "host4", "host5", "host6");
     assertConfig(
         actual,
         1,
@@ -314,7 +352,8 @@ public class CoordinatorDynamicConfigTest
         decommissioning,
         false,
         false,
-        turboLoading
+        turboLoading,
+        cloneServers
     );
 
     actual = CoordinatorDynamicConfig.builder().withDecommissioningNodes(ImmutableSet.of("host1")).build(actual);
@@ -332,7 +371,8 @@ public class CoordinatorDynamicConfigTest
         ImmutableSet.of("host1"),
         false,
         false,
-        turboLoading
+        turboLoading,
+        cloneServers
     );
 
     actual = CoordinatorDynamicConfig.builder().build(actual);
@@ -350,7 +390,8 @@ public class CoordinatorDynamicConfigTest
         ImmutableSet.of("host1"),
         false,
         false,
-        turboLoading
+        turboLoading,
+        cloneServers
     );
   }
 
@@ -390,7 +431,8 @@ public class CoordinatorDynamicConfigTest
         ImmutableSet.of(),
         false,
         false,
-        ImmutableSet.of()
+        ImmutableSet.of(),
+        ImmutableMap.of()
     );
   }
 
@@ -430,7 +472,8 @@ public class CoordinatorDynamicConfigTest
         decommissioning,
         false,
         false,
-        ImmutableSet.of()
+        ImmutableSet.of(),
+        ImmutableMap.of()
     );
   }
 
@@ -466,7 +509,8 @@ public class CoordinatorDynamicConfigTest
         ImmutableSet.of(),
         false,
         false,
-        ImmutableSet.of()
+        ImmutableSet.of(),
+        ImmutableMap.of()
     );
   }
 
@@ -489,7 +533,8 @@ public class CoordinatorDynamicConfigTest
         emptyList,
         false,
         false,
-        ImmutableSet.of()
+        ImmutableSet.of(),
+        ImmutableMap.of()
     );
   }
 
@@ -515,7 +560,8 @@ public class CoordinatorDynamicConfigTest
         ImmutableSet.of(),
         false,
         false,
-        ImmutableSet.of()
+        ImmutableSet.of(),
+        ImmutableMap.of()
     );
   }
 
@@ -568,7 +614,8 @@ public class CoordinatorDynamicConfigTest
       Set<String> decommissioningNodes,
       boolean pauseCoordination,
       boolean replicateAfterLoadTimeout,
-      Set<String> turboLoadingNodes
+      Set<String> turboLoadingNodes,
+      Map<String, String> cloneServers
   )
   {
     Assert.assertEquals(
@@ -590,10 +637,20 @@ public class CoordinatorDynamicConfigTest
     Assert.assertEquals(pauseCoordination, config.getPauseCoordination());
     Assert.assertEquals(replicateAfterLoadTimeout, config.getReplicateAfterLoadTimeout());
     Assert.assertEquals(turboLoadingNodes, config.getTurboLoadingNodes());
+    Assert.assertEquals(cloneServers, config.getCloneServers());
   }
 
   private static int getDefaultNumBalancerThreads()
   {
     return Math.max(1, JvmUtils.getRuntimeInfo().getAvailableProcessors() / 2);
   }
+
+  @Test
+  public void testEquals()
+  {
+    EqualsVerifier.forClass(CoordinatorDynamicConfig.class)
+                  .withIgnoredFields("validDebugDimensions")
+                  .usingGetClass()
+                  .verify();
+  }
 }
