@@ -30,6 +30,9 @@ public interface TestDataSet
 
   public static final MapBasedTestDataset NUMFOO = new NumFoo();
   public static final MapBasedTestDataset BROADCAST = new NumFoo("broadcast");
+  public static final MapBasedTestDataset RESTRICTED_BROADCAST = new NumFoo("restrictedBroadcastDatasource_m1_is_6");
+
+  public static final MapBasedTestDataset NUMBERS = new Numbers();
 
   String getName();
 
