@@ -22,7 +22,7 @@ package org.apache.druid.sql;
 import org.apache.druid.server.security.AuthorizationResult;
 import org.apache.druid.server.security.AuthorizationUtils;
 import org.apache.druid.server.security.ResourceAction;
-import org.apache.druid.sql.http.SqlQuery;
+import org.apache.druid.sql.calcite.planner.DruidPlanner;
 
 import javax.servlet.http.HttpServletRequest;
 import java.util.Set;
@@ -44,20 +44,30 @@ public class HttpStatement extends DirectStatement
 
   public HttpStatement(
       final SqlToolbox lifecycleToolbox,
-      final SqlQuery sqlQuery,
+      final SqlQueryPlus sqlQueryPlus,
       final HttpServletRequest req
   )
   {
     super(
         lifecycleToolbox,
-        SqlQueryPlus.builder(sqlQuery)
-                    .auth(AuthorizationUtils.authenticationResultFromRequest(req))
-                    .build(),
+        sqlQueryPlus,
         req.getRemoteAddr()
     );
     this.req = req;
   }
 
+  @Override
+  protected DruidPlanner createPlanner()
+  {
+    return sqlToolbox.plannerFactory.createPlanner(
+        sqlToolbox.engine,
+        queryPlus.sql(),
+        queryPlus.sqlNode(),
+        queryContext,
+        hook
+    );
+  }
+
   @Override
   protected Function<Set<ResourceAction>, AuthorizationResult> authorizer()
   {
