@@ -57,7 +57,7 @@ public class DruidJdbcStatement extends AbstractDruidJdbcStatement
   public synchronized void execute(SqlQueryPlus queryPlus, long maxRowCount)
   {
     closeResultSet();
-    this.sqlQuery = queryPlus.withContext(queryContext);
+    this.sqlQuery = queryPlus.withContext(queryContext).freshCopy();
     DirectStatement stmt = lifecycleFactory.directStatement(this.sqlQuery);
     resultSet = new DruidJdbcResultSet(this, stmt, Long.MAX_VALUE, fetcherFactory);
     try {
