@@ -22,6 +22,7 @@ package org.apache.druid.sql.calcite.expression.builtin;
 import org.apache.calcite.rex.RexCall;
 import org.apache.calcite.rex.RexNode;
 import org.apache.calcite.sql.SqlFunction;
+import org.apache.calcite.sql.SqlOperator;
 import org.apache.calcite.sql.type.OperandTypes;
 import org.apache.calcite.sql.type.ReturnTypes;
 import org.apache.calcite.sql.type.SqlTypeFamily;
@@ -72,7 +73,12 @@ public class ArrayContainsOperatorConversion extends BaseExpressionDimFilterOper
 
   public ArrayContainsOperatorConversion()
   {
-    super(SQL_FUNCTION, EXPR_FUNCTION);
+    this(SQL_FUNCTION, EXPR_FUNCTION);
+  }
+
+  protected ArrayContainsOperatorConversion(SqlOperator operator, String druidFunctionName)
+  {
+    super(operator, druidFunctionName);
   }
 
   @Nullable
@@ -149,8 +155,10 @@ public class ArrayContainsOperatorConversion extends BaseExpressionDimFilterOper
         }
       }
     }
-    // if the input is a direct array column, we can use sweet array filter
-    if (leftExpr.isDirectColumnAccess() && leftExpr.isArray()) {
+    // if this is really ARRAY_CONTAINS (not the subclass MV_CONTAINS), and the input is a direct array column,
+    // we can use sweet array filter
+    final boolean isArrayContains = SQL_FUNCTION.equals(calciteOperator());
+    if (isArrayContains && leftExpr.isDirectColumnAccess() && leftExpr.isArray()) {
       Expr expr = plannerContext.parseExpression(rightExpr.getExpression());
       // To convert this expression filter into an AND of ArrayContainsElement filters, we need to extract all array
       // elements. For now, we can optimize only when rightExpr is a literal because there is no way to extract the
