@@ -60,6 +60,7 @@ import org.apache.calcite.tools.Program;
 import org.apache.calcite.tools.RelBuilder;
 import org.apache.calcite.tools.ValidationException;
 import org.apache.calcite.util.Pair;
+import org.apache.druid.sql.calcite.parser.DruidSqlParser;
 
 import javax.annotation.Nullable;
 import java.io.Reader;
@@ -227,11 +228,30 @@ public class CalcitePlanner implements Planner, ViewExpander
     }
     ensure(CalcitePlanner.State.STATE_2_READY);
     SqlParser parser = SqlParser.create(reader, parserConfig);
-    SqlNode sqlNode = parser.parseStmt();
+    SqlNode sqlNode = parser.parseStmtList();
     state = CalcitePlanner.State.STATE_3_PARSED;
     return sqlNode;
   }
 
+  /**
+   * Skip parsing, moving state along to a {@link State#STATE_3_PARSED}. We have this because we
+   * parse before the planner is even created, using {@link DruidSqlParser}, in order to capture
+   * SET statements as early as possible.
+   */
+  public void skipParse()
+  {
+    switch (state) {
+      case STATE_0_CLOSED:
+      case STATE_1_RESET:
+        ready();
+        break;
+      default:
+        break;
+    }
+    ensure(CalcitePlanner.State.STATE_2_READY);
+    state = CalcitePlanner.State.STATE_3_PARSED;
+  }
+
   @Override
   public SqlNode validate(SqlNode sqlNode) throws ValidationException
   {
