@@ -29,6 +29,8 @@ import org.apache.calcite.rel.core.Window;
 import org.apache.calcite.rex.RexBuilder;
 import org.apache.calcite.rex.RexInputRef;
 import org.apache.calcite.rex.RexNode;
+import org.apache.commons.lang3.builder.ToStringBuilder;
+import org.apache.commons.lang3.builder.ToStringStyle;
 import org.apache.druid.error.DruidException;
 import org.apache.druid.query.DataSource;
 import org.apache.druid.query.FilteredDataSource;
@@ -73,7 +75,7 @@ public class DruidQueryGenerator
    * Its main purpose is to provide access to parent nodes;
    * so that context sensitive logics can be formalized with it.
    */
-  static class DruidNodeStack
+  public static class DruidNodeStack
   {
     static class Entry
     {
@@ -85,9 +87,20 @@ public class DruidQueryGenerator
         this.node = node;
         this.operandIndex = operandIndex;
       }
-    }
 
+      @Override
+      public String toString()
+      {
+        return ToStringBuilder.reflectionToString(this, ToStringStyle.NO_CLASS_NAME_STYLE);
+      }
+    }
     Stack<Entry> stack = new Stack<>();
+    PlannerContext plannerContext;
+
+    public DruidNodeStack(PlannerContext plannerContext)
+    {
+      this.plannerContext = plannerContext;
+    }
 
     public void push(DruidLogicalNode item)
     {
@@ -109,7 +122,7 @@ public class DruidQueryGenerator
       return stack.size();
     }
 
-    public DruidLogicalNode peekNode()
+    public DruidLogicalNode getNode()
     {
       return stack.peek().node;
     }
@@ -128,11 +141,16 @@ public class DruidQueryGenerator
     {
       return stack.peek().operandIndex;
     }
+
+    public PlannerContext getPlannerContext()
+    {
+      return plannerContext;
+    }
   }
 
   public DruidQuery buildQuery()
   {
-    DruidNodeStack stack = new DruidNodeStack();
+    DruidNodeStack stack = new DruidNodeStack(vertexFactory.plannerContext);
     stack.push(relRoot);
     Vertex vertex = buildVertexFor(stack);
     return vertex.buildQuery(true);
@@ -142,7 +160,7 @@ public class DruidQueryGenerator
   {
     List<Vertex> newInputs = new ArrayList<>();
 
-    for (RelNode input : stack.peekNode().getInputs()) {
+    for (RelNode input : stack.getNode().getInputs()) {
       stack.push((DruidLogicalNode) input, newInputs.size());
       newInputs.add(buildVertexFor(stack));
       stack.pop();
@@ -153,7 +171,7 @@ public class DruidQueryGenerator
 
   private Vertex processNodeWithInputs(DruidNodeStack stack, List<Vertex> newInputs)
   {
-    DruidLogicalNode node = stack.peekNode();
+    DruidLogicalNode node = stack.getNode();
     if (node instanceof SourceDescProducer) {
       return vertexFactory.createVertex(stack, PartialDruidQuery.create(node), newInputs);
     }
@@ -386,7 +404,7 @@ public class DruidQueryGenerator
        */
       private Optional<PartialDruidQuery> extendPartialDruidQuery(DruidNodeStack stack)
       {
-        DruidLogicalNode parentNode = stack.peekNode();
+        DruidLogicalNode parentNode = stack.getNode();
         if (accepts(stack, Stage.WHERE_FILTER, Filter.class)) {
           PartialDruidQuery newPartialQuery = partialDruidQuery.withWhereFilter((Filter) parentNode);
           return Optional.of(newPartialQuery);
@@ -428,7 +446,7 @@ public class DruidQueryGenerator
 
       private boolean accepts(DruidNodeStack stack, Stage stage, Class<? extends RelNode> clazz)
       {
-        DruidLogicalNode currentNode = stack.peekNode();
+        DruidLogicalNode currentNode = stack.getNode();
         if (Project.class == clazz && stack.size() >= 2) {
           // peek at parent and postpone project for next query stage
           DruidLogicalNode parentNode = stack.parentNode();
