@@ -20,6 +20,7 @@
 package org.apache.druid.sql.calcite.run;
 
 import com.fasterxml.jackson.databind.ObjectMapper;
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.ImmutableSet;
 import com.google.inject.Inject;
 import org.apache.calcite.rel.RelRoot;
@@ -27,10 +28,14 @@ import org.apache.calcite.rel.type.RelDataType;
 import org.apache.calcite.rel.type.RelDataTypeFactory;
 import org.apache.druid.error.InvalidSqlInput;
 import org.apache.druid.guice.LazySingleton;
+import org.apache.druid.java.util.common.logger.Logger;
 import org.apache.druid.query.JoinAlgorithm;
 import org.apache.druid.query.groupby.GroupByQuery;
 import org.apache.druid.query.timeboundary.TimeBoundaryQuery;
 import org.apache.druid.server.QueryLifecycleFactory;
+import org.apache.druid.server.QueryScheduler;
+import org.apache.druid.sql.SqlStatementFactory;
+import org.apache.druid.sql.SqlToolbox;
 import org.apache.druid.sql.calcite.parser.DruidSqlInsert;
 import org.apache.druid.sql.calcite.parser.DruidSqlReplace;
 import org.apache.druid.sql.calcite.planner.PlannerContext;
@@ -39,10 +44,13 @@ import org.apache.druid.sql.destination.IngestDestination;
 
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.CopyOnWriteArrayList;
 
 @LazySingleton
 public class NativeSqlEngine implements SqlEngine
 {
+  private static final Logger LOG = new Logger(NativeSqlEngine.class);
+
   public static final Set<String> SYSTEM_CONTEXT_PARAMETERS = ImmutableSet.of(
       TimeBoundaryQuery.MAX_TIME_ARRAY_OUTPUT_NAME,
       TimeBoundaryQuery.MIN_TIME_ARRAY_OUTPUT_NAME,
@@ -54,19 +62,34 @@ public class NativeSqlEngine implements SqlEngine
       DruidSqlReplace.SQL_REPLACE_TIME_CHUNKS
   );
 
-  private static final String NAME = "native";
+  public static final String NAME = "native";
 
   private final QueryLifecycleFactory queryLifecycleFactory;
   private final ObjectMapper jsonMapper;
+  private final SqlStatementFactory sqlStatementFactory;
 
   @Inject
   public NativeSqlEngine(
       final QueryLifecycleFactory queryLifecycleFactory,
-      final ObjectMapper jsonMapper
+      final ObjectMapper jsonMapper,
+      final SqlToolbox toolbox
   )
   {
     this.queryLifecycleFactory = queryLifecycleFactory;
     this.jsonMapper = jsonMapper;
+    this.sqlStatementFactory = new SqlStatementFactory(toolbox.withEngine(this));
+  }
+
+  @VisibleForTesting
+  public NativeSqlEngine(
+      final QueryLifecycleFactory queryLifecycleFactory,
+      final ObjectMapper jsonMapper,
+      final SqlStatementFactory sqlStatementFactory
+  )
+  {
+    this.queryLifecycleFactory = queryLifecycleFactory;
+    this.jsonMapper = jsonMapper;
+    this.sqlStatementFactory = sqlStatementFactory;
   }
 
   @Override
@@ -164,4 +187,21 @@ public class NativeSqlEngine implements SqlEngine
       throw InvalidSqlInput.exception("Join algorithm [%s] is not supported by engine [%s]", joinAlgorithm, NAME);
     }
   }
+
+  @Override
+  public SqlStatementFactory getSqlStatementFactory()
+  {
+    return sqlStatementFactory;
+  }
+
+  @Override
+  public void cancelQuery(PlannerContext plannerContext, QueryScheduler queryScheduler)
+  {
+    final CopyOnWriteArrayList<String> nativeQueryIds = plannerContext.getNativeQueryIds();
+
+    for (String nativeQueryId : nativeQueryIds) {
+      LOG.debug("Canceling native query [%s]", nativeQueryId);
+      queryScheduler.cancelQuery(nativeQueryId);
+    }
+  }
 }
