@@ -207,6 +207,9 @@ public class BaseCalciteQueryTest extends CalciteTestBase
       QueryContexts.MAX_SCATTER_GATHER_BYTES_KEY, Long.MAX_VALUE
   );
 
+  public static final Map<String, Object> QUERY_CONTEXT_LEXICOGRAPHIC_TOPN =
+      QueryContexts.override(QUERY_CONTEXT_DEFAULT, PlannerConfig.CTX_KEY_USE_LEXICOGRAPHIC_TOPN, true);
+
   public static final Map<String, Object> QUERY_CONTEXT_NO_TOPN = ImmutableMap.of(
       QueryContexts.CTX_SQL_QUERY_ID, DUMMY_SQL_ID,
       PlannerContext.CTX_SQL_CURRENT_TIMESTAMP, PRETEND_CURRENT_TIME,
@@ -1047,12 +1050,13 @@ public class BaseCalciteQueryTest extends CalciteTestBase
             i,
             types.get(i),
             expectedCell,
-            resultCell);
+            resultCell
+        );
       }
     }
   }
 
-  public void assertResultsEquals(String sql, List<Object[]> expectedResults, List<Object[]> results)
+  public static void assertResultsEquals(String sql, List<Object[]> expectedResults, List<Object[]> results)
   {
     int minSize = Math.min(results.size(), expectedResults.size());
     for (int i = 0; i < minSize; i++) {
@@ -1498,7 +1502,7 @@ public class BaseCalciteQueryTest extends CalciteTestBase
           outprint(col);
           outprint("F");
         } else if (col instanceof Object[]) {
-          printArray(array);
+          printArray((Object[]) col);
         } else if (col instanceof List) {
           printList((List<?>) col);
         } else {
