@@ -263,10 +263,6 @@ public class QueryTestRunner
       BaseCalciteQueryTest.log.info("SQL: %s", builder.sql);
 
       final SqlStatementFactory sqlStatementFactory = builder.statementFactory();
-      final SqlQueryPlus sqlQuery = SqlQueryPlus.builder(builder.sql)
-                                                .sqlParameters(builder.parameters)
-                                                .auth(builder.authenticationResult)
-                                                .build();
 
       final List<String> vectorizeValues = new ArrayList<>();
       vectorizeValues.add("false");
@@ -275,7 +271,14 @@ public class QueryTestRunner
       }
 
       for (final String vectorize : vectorizeValues) {
-        final Map<String, Object> theQueryContext = new HashMap<>(builder.queryContext);
+        // Need to create sqlQuery inside the loop, because SqlQueryPlus can only be used once
+        final SqlQueryPlus sqlQuery = SqlQueryPlus.builder(builder.sql)
+                                                  .sqlParameters(builder.parameters)
+                                                  .auth(builder.authenticationResult)
+                                                  .context(builder.queryContext)
+                                                  .build();
+
+        final Map<String, Object> theQueryContext = new HashMap<>(sqlQuery.context());
         theQueryContext.put(QueryContexts.VECTORIZE_KEY, vectorize);
         theQueryContext.put(QueryContexts.VECTORIZE_VIRTUAL_COLUMNS_KEY, vectorize);
 
