@@ -33,24 +33,24 @@ import org.apache.calcite.rel.logical.LogicalTableScan;
 import org.apache.calcite.rel.rules.ProjectMergeRule;
 import org.apache.calcite.schema.Schema;
 import org.apache.druid.guice.LazySingleton;
+import org.apache.druid.guice.security.PolicyModule;
 import org.apache.druid.jackson.DefaultObjectMapper;
 import org.apache.druid.jackson.JacksonModule;
 import org.apache.druid.math.expr.ExprMacroTable;
 import org.apache.druid.segment.join.JoinableFactoryWrapper;
 import org.apache.druid.server.QueryLifecycleFactory;
-import org.apache.druid.server.security.AuthConfig;
 import org.apache.druid.server.security.AuthorizerMapper;
 import org.apache.druid.server.security.ResourceType;
+import org.apache.druid.sql.SqlStatementFactory;
 import org.apache.druid.sql.calcite.aggregation.SqlAggregator;
 import org.apache.druid.sql.calcite.expression.SqlOperatorConversion;
+import org.apache.druid.sql.calcite.parser.DruidSqlParser;
 import org.apache.druid.sql.calcite.rule.ExtensionCalciteRuleProvider;
 import org.apache.druid.sql.calcite.run.NativeSqlEngine;
 import org.apache.druid.sql.calcite.schema.DruidSchemaCatalog;
 import org.apache.druid.sql.calcite.schema.DruidSchemaName;
 import org.apache.druid.sql.calcite.schema.NamedSchema;
 import org.apache.druid.sql.calcite.util.CalciteTestBase;
-import org.apache.druid.sql.calcite.util.CalciteTests;
-import org.apache.druid.sql.hook.DruidHookDispatcher;
 import org.easymock.EasyMock;
 import org.easymock.EasyMockExtension;
 import org.easymock.Mock;
@@ -127,6 +127,7 @@ public class CalcitePlannerModuleTest extends CalciteTestBase
     };
     injector = Guice.createInjector(
         new JacksonModule(),
+        new PolicyModule(),
         binder -> {
           binder.bind(Validator.class).toInstance(Validation.buildDefaultValidatorFactory().getValidator());
           binder.bindScope(LazySingleton.class, Scopes.SINGLETON);
@@ -182,25 +183,14 @@ public class CalcitePlannerModuleTest extends CalciteTestBase
   public void testExtensionCalciteRule()
   {
     ObjectMapper mapper = new DefaultObjectMapper();
-    PlannerToolbox toolbox = new PlannerToolbox(
-        injector.getInstance(DruidOperatorTable.class),
-        macroTable,
-        mapper,
-        injector.getInstance(PlannerConfig.class),
-        rootSchema,
-        joinableFactoryWrapper,
-        CatalogResolver.NULL_RESOLVER,
-        "druid",
-        new CalciteRulesManager(ImmutableSet.of()),
-        CalciteTests.TEST_AUTHORIZER_MAPPER,
-        AuthConfig.newBuilder().build(),
-        new DruidHookDispatcher()
-    );
+    PlannerToolbox toolbox = injector.getInstance(PlannerFactory.class);
 
+    final String sql = "SELECT 1";
     PlannerContext context = PlannerContext.create(
         toolbox,
-        "SELECT 1",
-        new NativeSqlEngine(queryLifecycleFactory, mapper),
+        sql,
+        DruidSqlParser.parse(sql, false).getMainStatement(),
+        new NativeSqlEngine(queryLifecycleFactory, mapper, (SqlStatementFactory) null),
         Collections.emptyMap(),
         null
     );
@@ -215,33 +205,23 @@ public class CalcitePlannerModuleTest extends CalciteTestBase
   public void testConfigurableBloat()
   {
     ObjectMapper mapper = new DefaultObjectMapper();
-    PlannerToolbox toolbox = new PlannerToolbox(
-            injector.getInstance(DruidOperatorTable.class),
-            macroTable,
-            mapper,
-            injector.getInstance(PlannerConfig.class),
-            rootSchema,
-            joinableFactoryWrapper,
-            CatalogResolver.NULL_RESOLVER,
-            "druid",
-            new CalciteRulesManager(ImmutableSet.of()),
-            CalciteTests.TEST_AUTHORIZER_MAPPER,
-            AuthConfig.newBuilder().build(),
-            new DruidHookDispatcher()
-    );
+    PlannerToolbox toolbox = injector.getInstance(PlannerFactory.class);
 
+    final String sql = "SELECT 1";
     PlannerContext contextWithBloat = PlannerContext.create(
             toolbox,
-            "SELECT 1",
-            new NativeSqlEngine(queryLifecycleFactory, mapper),
+            sql,
+            DruidSqlParser.parse(sql, false).getMainStatement(),
+            new NativeSqlEngine(queryLifecycleFactory, mapper, (SqlStatementFactory) null),
             Collections.singletonMap(BLOAT_PROPERTY, BLOAT),
             null
     );
 
     PlannerContext contextWithoutBloat = PlannerContext.create(
             toolbox,
-            "SELECT 1",
-            new NativeSqlEngine(queryLifecycleFactory, mapper),
+            sql,
+            DruidSqlParser.parse(sql, false).getMainStatement(),
+            new NativeSqlEngine(queryLifecycleFactory, mapper, (SqlStatementFactory) null),
             Collections.emptyMap(),
             null
     );
