@@ -25,7 +25,6 @@ import com.fasterxml.jackson.databind.ObjectMapper;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
-import com.google.common.collect.Iterables;
 import com.google.common.util.concurrent.Futures;
 import junitparams.converters.Nullable;
 import org.apache.calcite.DataContext;
@@ -45,6 +44,8 @@ import org.apache.druid.client.InternalQueryConfig;
 import org.apache.druid.client.TimelineServerView;
 import org.apache.druid.client.coordinator.NoopCoordinatorClient;
 import org.apache.druid.data.input.InputRow;
+import org.apache.druid.data.input.impl.DimensionsSpec;
+import org.apache.druid.data.input.impl.StringDimensionSchema;
 import org.apache.druid.discovery.DataNodeService;
 import org.apache.druid.discovery.DiscoveryDruidNode;
 import org.apache.druid.discovery.DruidLeaderClient;
@@ -69,7 +70,6 @@ import org.apache.druid.java.util.http.client.response.StringFullResponseHolder;
 import org.apache.druid.query.QueryRunnerFactoryConglomerate;
 import org.apache.druid.query.aggregation.CountAggregatorFactory;
 import org.apache.druid.query.aggregation.DoubleSumAggregatorFactory;
-import org.apache.druid.query.aggregation.LongSumAggregatorFactory;
 import org.apache.druid.query.aggregation.hyperloglog.HyperUniquesAggregatorFactory;
 import org.apache.druid.rpc.indexing.OverlordClient;
 import org.apache.druid.segment.IndexBuilder;
@@ -109,6 +109,7 @@ import org.apache.druid.timeline.DataSegment;
 import org.apache.druid.timeline.SegmentId;
 import org.apache.druid.timeline.SegmentStatusInCluster;
 import org.apache.druid.timeline.partition.NumberedShardSpec;
+import org.apache.druid.timeline.partition.ShardSpec;
 import org.easymock.EasyMock;
 import org.jboss.netty.handler.codec.http.HttpResponse;
 import org.joda.time.DateTime;
@@ -155,6 +156,17 @@ public class SystemSchemaTest extends CalciteTestBase
       TestDataBuilder.createRow(ImmutableMap.of("t", "2001-01-02", "m1", "8.0", "dim3", ImmutableList.of("xyz")))
   );
 
+  private static final ShardSpec SHARD_SPEC = new NumberedShardSpec(0, 1);
+  private static final IncrementalIndexSchema SCHEMA = new IncrementalIndexSchema.Builder()
+      .withDimensionsSpec(new DimensionsSpec(ImmutableList.of(new StringDimensionSchema("dim1"))))
+      .withMetrics(
+          new CountAggregatorFactory("cnt"),
+          new DoubleSumAggregatorFactory("m1", "m1"),
+          new HyperUniquesAggregatorFactory("unique_dim1", "dim1")
+      )
+      .withRollup(false)
+      .build();
+
   private SystemSchema schema;
   private SpecificSegmentsQuerySegmentWalker walker;
   private DruidLeaderClient client;
@@ -208,46 +220,27 @@ public class SystemSchemaTest extends CalciteTestBase
     final QueryableIndex index1 = IndexBuilder.create()
                                               .tmpDir(new File(tmpDir, "1"))
                                               .segmentWriteOutMediumFactory(OffHeapMemorySegmentWriteOutMediumFactory.instance())
-                                              .schema(
-                                                  new IncrementalIndexSchema.Builder()
-                                                      .withMetrics(
-                                                          new CountAggregatorFactory("cnt"),
-                                                          new DoubleSumAggregatorFactory("m1", "m1"),
-                                                          new HyperUniquesAggregatorFactory("unique_dim1", "dim1")
-                                                      )
-                                                      .withRollup(false)
-                                                      .build()
-                                              )
+                                              .schema(SCHEMA)
                                               .rows(ROWS1)
                                               .buildMMappedIndex();
 
     final QueryableIndex index2 = IndexBuilder.create()
                                               .tmpDir(new File(tmpDir, "2"))
                                               .segmentWriteOutMediumFactory(OffHeapMemorySegmentWriteOutMediumFactory.instance())
-                                              .schema(
-                                                  new IncrementalIndexSchema.Builder()
-                                                      .withMetrics(new LongSumAggregatorFactory("m1", "m1"))
-                                                      .withRollup(false)
-                                                      .build()
-                                              )
+                                              .schema(SCHEMA)
                                               .rows(ROWS2)
                                               .buildMMappedIndex();
     final QueryableIndex index3 = IndexBuilder.create()
                                               .tmpDir(new File(tmpDir, "3"))
                                               .segmentWriteOutMediumFactory(OffHeapMemorySegmentWriteOutMediumFactory.instance())
-                                              .schema(
-                                                  new IncrementalIndexSchema.Builder()
-                                                      .withMetrics(new LongSumAggregatorFactory("m1", "m1"))
-                                                      .withRollup(false)
-                                                      .build()
-                                              )
+                                              .schema(SCHEMA)
                                               .rows(ROWS3)
                                               .buildMMappedIndex();
 
     walker = SpecificSegmentsQuerySegmentWalker.createWalker(conglomerate)
-        .add(segment1, index1)
-        .add(segment2, index2)
-        .add(segment3, index3);
+                                               .add(segment1, index1)
+                                               .add(segment2, index2)
+                                               .add(segment3, index3);
 
     BrokerSegmentMetadataCache cache = new BrokerSegmentMetadataCache(
         CalciteTests.createMockQueryLifecycleFactory(walker, conglomerate),
@@ -316,30 +309,31 @@ public class SystemSchemaTest extends CalciteTestBase
       1,
       83000L
   );
-  private final DataSegment publishedUncompactedSegment3 = new DataSegment(
-      "wikipedia3",
-      Intervals.of("2009/2010"),
-      "version3",
-      null,
-      ImmutableList.of("dim1", "dim2"),
-      ImmutableList.of("met1", "met2"),
-      null,
-      null,
-      1,
-      47000L
-  );
+  private final DataSegment publishedUncompactedSegment3 = DataSegment.builder(SegmentId.of(
+                                                                          "wikipedia3",
+                                                                          Intervals.of("2009/2010"),
+                                                                          "version3",
+                                                                          null
+                                                                      ))
+                                                                      .dimensions(ImmutableList.of("dim1", "dim2"))
+                                                                      .metrics(ImmutableList.of("met1", "met2"))
+                                                                      .projections(ImmutableList.of())
+                                                                      .binaryVersion(1)
+                                                                      .size(47000L)
+                                                                      .build();
 
-  private final DataSegment segment1 = new DataSegment(
-      "test1",
-      Intervals.of("2010/2011"),
-      "version1",
-      null,
-      ImmutableList.of("dim1", "dim2"),
-      ImmutableList.of("met1", "met2"),
-      null,
-      1,
-      100L
-  );
+  private final DataSegment segment1 = DataSegment.builder(SegmentId.of(
+                                                      "test1",
+                                                      Intervals.of("2010/2011"),
+                                                      "version1",
+                                                      null
+                                                  ))
+                                                  .dimensions(ImmutableList.of("dim1", "dim2"))
+                                                  .metrics(ImmutableList.of("met1", "met2"))
+                                                  .projections(ImmutableList.of("proj1", "proj2"))
+                                                  .binaryVersion(1)
+                                                  .size(100L)
+                                                  .build();
   private final DataSegment segment2 = new DataSegment(
       "test2",
       Intervals.of("2011/2012"),
@@ -547,7 +541,7 @@ public class SystemSchemaTest extends CalciteTestBase
     final RelDataType rowType = segmentsTable.getRowType(new JavaTypeFactoryImpl());
     final List<RelDataTypeField> fields = rowType.getFieldList();
 
-    Assert.assertEquals(19, fields.size());
+    Assert.assertEquals(20, fields.size());
 
     final SystemSchema.TasksTable tasksTable = (SystemSchema.TasksTable) schema.getTableMap().get("tasks");
     final RelDataType sysRowType = tasksTable.getRowType(new JavaTypeFactoryImpl());
@@ -585,138 +579,104 @@ public class SystemSchemaTest extends CalciteTestBase
     rows.sort((Object[] row1, Object[] row2) -> ((Comparable) row1[0]).compareTo(row2[0]));
 
     // total segments = 8
-    // segments test1, test2  are published and available
-    // segment test3 is served by historical but unpublished or unused
-    // segments test4, test5 are not published but available (realtime segments)
-    // segment test2 is both published and served by a realtime server.
-
     Assert.assertEquals(8, rows.size());
-
-    verifyRow(
-        rows.get(0),
-        "test1_2010-01-01T00:00:00.000Z_2011-01-01T00:00:00.000Z_version1",
-        100L,
-        0L, //partition_num
-        1L, //num_replicas
-        3L, //numRows
-        1L, //is_published
-        1L, //is_available
-        0L, //is_realtime
-        1L, //is_overshadowed
-        null, //is_compacted
-        2L  // replication_factor
-    );
-
-    verifyRow(
-        rows.get(1),
-        "test2_2011-01-01T00:00:00.000Z_2012-01-01T00:00:00.000Z_version2",
-        100L,
-        0L, //partition_num
-        2L, //x§segment test2 is served by historical and realtime servers
-        3L, //numRows
-        1L, //is_published
-        1L, //is_available
-        0L, //is_realtime
-        0L, //is_overshadowed,
-        null, //is_compacted
-        0L  // replication_factor
-    );
-
-    //segment test3 is unpublished and has a NumberedShardSpec with partitionNum = 2
-    verifyRow(
-        rows.get(2),
-        "test3_2012-01-01T00:00:00.000Z_2013-01-01T00:00:00.000Z_version3_2",
-        100L,
-        2L, //partition_num
-        1L, //num_replicas
-        2L, //numRows
-        0L, //is_published
-        1L, //is_available
-        0L, //is_realtime
-        0L, //is_overshadowed
-        null, //is_compacted
-        -1L   // replication_factor
-    );
-
-    verifyRow(
-        rows.get(3),
-        "test4_2014-01-01T00:00:00.000Z_2015-01-01T00:00:00.000Z_version4",
-        100L,
-        0L, //partition_num
-        1L, //num_replicas
-        0L, //numRows
-        0L, //is_published
-        1L, //is_available
-        1L, //is_realtime
-        0L, //is_overshadowed
-        null, //is_compacted
-        -1L  // replication_factor
-    );
-
-    verifyRow(
-        rows.get(4),
-        "test5_2015-01-01T00:00:00.000Z_2016-01-01T00:00:00.000Z_version5",
-        100L,
-        0L, //partition_num
-        1L, //num_replicas
-        0L, //numRows
-        0L, //is_published
-        1L, //is_available
-        1L, //is_realtime
-        0L, //is_overshadowed
-        null, //is_compacted
-        -1L  // replication_factor
-    );
-
-    // wikipedia segments are published and unavailable, num_replicas is 0
-    // wikipedia segment 1 and 2 are compacted while 3 are not compacted
-    verifyRow(
-        rows.get(5),
-        "wikipedia1_2007-01-01T00:00:00.000Z_2008-01-01T00:00:00.000Z_version1",
-        53000L,
-        0L, //partition_num
-        0L, //num_replicas
-        0L, //numRows
-        1L, //is_published
-        0L, //is_available
-        0L, //is_realtime
-        1L, //is_overshadowed
-        expectedCompactionState, //is_compacted
-        2L  // replication_factor
-    );
-
-    verifyRow(
-        rows.get(6),
-        "wikipedia2_2008-01-01T00:00:00.000Z_2009-01-01T00:00:00.000Z_version2",
-        83000L,
-        0L, //partition_num
-        0L, //num_replicas
-        0L, //numRows
-        1L, //is_published
-        0L, //is_available
-        0L, //is_realtime
-        0L, //is_overshadowed
-        expectedCompactionState, //is_compacted
-        0L  // replication_factor
-    );
-
-    verifyRow(
-        rows.get(7),
-        "wikipedia3_2009-01-01T00:00:00.000Z_2010-01-01T00:00:00.000Z_version3",
-        47000L,
-        0L, //partition_num
-        0L, //num_replicas
-        0L, //numRows
-        1L, //is_published
-        0L, //is_available
-        0L, //is_realtime
-        0L, //is_overshadowed
-        null, //is_compacted
-        2L  // replication_factor
-    );
-
     // Verify value types.
     verifyTypes(rows, SystemSchema.SEGMENTS_SIGNATURE);
+
+    // segments test1, test2  are published and available.
+    Object[] segment1Expected = new Object[]{
+        // segment_id, datasource
+        "test1_2010-01-01T00:00:00.000Z_2011-01-01T00:00:00.000Z_version1", "test1",
+        // start, end, size, version, partition_num, num_replicas, numRows
+        "2010-01-01T00:00:00.000Z", "2011-01-01T00:00:00.000Z", 100L, "version1", 0L, 1L, 3L,
+        //  is_active, is_published, is_available, is_realtime, is_overshadowed, shard_spec
+        0L, 1L, 1L, 0L, 1L, MAPPER.writeValueAsString(SHARD_SPEC),
+        // dimensions, metrics, projections, last_compaction_state, replication_factor
+        "[\"dim1\",\"dim2\"]", "[\"met1\",\"met2\"]", "[\"proj1\",\"proj2\"]", null, 2L
+    };
+    Assert.assertArrayEquals(segment1Expected, rows.get(0));
+    Object[] segment2Expected = new Object[]{
+        // segment_id, datasource
+        "test2_2011-01-01T00:00:00.000Z_2012-01-01T00:00:00.000Z_version2", "test2",
+        // start, end, size, version, partition_num, num_replicas, numRows
+        "2011-01-01T00:00:00.000Z", "2012-01-01T00:00:00.000Z", 100L, "version2", 0L, 2L, 3L,
+        //  is_active, is_published, is_available, is_realtime, is_overshadowed, shard_spec
+        1L, 1L, 1L, 0L, 0L, MAPPER.writeValueAsString(SHARD_SPEC),
+        // dimensions, metrics, projections, last_compaction_state, replication_factor
+        "[\"dim1\",\"dim2\"]", "[\"met1\",\"met2\"]", null, null, 0L
+    };
+    Assert.assertArrayEquals(segment2Expected, rows.get(1));
+    //segment test3 is unpublished and has a NumberedShardSpec with partitionNum = 2, is served by historical but unpublished or unused
+    Object[] segment3Expected = new Object[]{
+        // segment_id, datasource
+        "test3_2012-01-01T00:00:00.000Z_2013-01-01T00:00:00.000Z_version3_2", "test3",
+        // start, end, size, version, partition_num, num_replicas, numRows
+        "2012-01-01T00:00:00.000Z", "2013-01-01T00:00:00.000Z", 100L, "version3", 2L, 1L, 2L,
+        //  is_active, is_published, is_available, is_realtime, is_overshadowed, shard_spec
+        0L, 0L, 1L, 0L, 0L, MAPPER.writeValueAsString(new NumberedShardSpec(2, 3)),
+        // dimensions, metrics, projections, last_compaction_state, replication_factor
+        "[\"dim1\",\"dim2\"]", "[\"met1\",\"met2\"]", null, null, -1L
+    };
+    Assert.assertArrayEquals(segment3Expected, rows.get(2));
+    // segments test4, test5 are not published but available (realtime segments)
+    Object[] segment4Expected = new Object[]{
+        // segment_id, datasource
+        "test4_2014-01-01T00:00:00.000Z_2015-01-01T00:00:00.000Z_version4", "test4",
+        // start, end, size, version, partition_num, num_replicas, numRows
+        "2014-01-01T00:00:00.000Z", "2015-01-01T00:00:00.000Z", 100L, "version4", 0L, 1L, 0L,
+        //  is_active, is_published, is_available, is_realtime, is_overshadowed, shard_spec
+        1L, 0L, 1L, 1L, 0L, MAPPER.writeValueAsString(SHARD_SPEC),
+        // dimensions, metrics, projections, last_compaction_state, replication_factor
+        "[\"dim1\",\"dim2\"]", "[\"met1\",\"met2\"]", null, null, -1L
+    };
+    Assert.assertArrayEquals(segment4Expected, rows.get(3));
+    Object[] segment5Expected = new Object[]{
+        // segment_id, datasource
+        "test5_2015-01-01T00:00:00.000Z_2016-01-01T00:00:00.000Z_version5", "test5",
+        // start, end, size, version, partition_num, num_replicas, numRows
+        "2015-01-01T00:00:00.000Z", "2016-01-01T00:00:00.000Z", 100L, "version5", 0L, 1L, 0L,
+        //  is_active, is_published, is_available, is_realtime, is_overshadowed, shard_spec
+        1L, 0L, 1L, 1L, 0L, MAPPER.writeValueAsString(SHARD_SPEC),
+        // dimensions, metrics, projections, last_compaction_state, replication_factor
+        "[\"dim1\",\"dim2\"]", "[\"met1\",\"met2\"]", null, null, -1L
+    };
+    Assert.assertArrayEquals(segment5Expected, rows.get(4));
+
+    // wikipedia segment 1 and segment 2 are published and unavailable and compacted, num_replicas is 0
+    Object[] wikiSegment1Expected = new Object[]{
+        // segment_id, datasource
+        "wikipedia1_2007-01-01T00:00:00.000Z_2008-01-01T00:00:00.000Z_version1", "wikipedia1",
+        // start, end, size, version, partition_num, num_replicas, numRows
+        "2007-01-01T00:00:00.000Z", "2008-01-01T00:00:00.000Z", 53_000L, "version1", 0L, 0L, 0L,
+        //  is_active, is_published, is_available, is_realtime, is_overshadowed, shard_spec
+        0L, 1L, 0L, 0L, 1L, MAPPER.writeValueAsString(SHARD_SPEC),
+        // dimensions, metrics, projections, last_compaction_state, replication_factor
+        "[\"dim1\",\"dim2\"]", "[\"met1\",\"met2\"]", null, MAPPER.writeValueAsString(expectedCompactionState), 2L
+    };
+    Assert.assertArrayEquals(wikiSegment1Expected, rows.get(5));
+    Object[] wikiSegment2Expected = new Object[]{
+        // segment_id, datasource
+        "wikipedia2_2008-01-01T00:00:00.000Z_2009-01-01T00:00:00.000Z_version2", "wikipedia2",
+        // start, end, size, version, partition_num, num_replicas, numRows
+        "2008-01-01T00:00:00.000Z", "2009-01-01T00:00:00.000Z", 83_000L, "version2", 0L, 0L, 0L,
+        //  is_active, is_published, is_available, is_realtime, is_overshadowed, shard_spec
+        1L, 1L, 0L, 0L, 0L, MAPPER.writeValueAsString(SHARD_SPEC),
+        // dimensions, metrics, projections, last_compaction_state, replication_factor
+        "[\"dim1\",\"dim2\"]", "[\"met1\",\"met2\"]", null, MAPPER.writeValueAsString(expectedCompactionState), 0L
+    };
+    Assert.assertArrayEquals(wikiSegment2Expected, rows.get(6));
+    // wikipedia segment 3 are not compacted, and is projection aware.
+    Object[] wikiSegment3Expected = new Object[]{
+        // segment_id, datasource
+        "wikipedia3_2009-01-01T00:00:00.000Z_2010-01-01T00:00:00.000Z_version3", "wikipedia3",
+        // start, end, size, version, partition_num, num_replicas, numRows
+        "2009-01-01T00:00:00.000Z", "2010-01-01T00:00:00.000Z", 47_000L, "version3", 0L, 0L, 0L,
+        //  is_active, is_published, is_available, is_realtime, is_overshadowed, shard_spec
+        1L, 1L, 0L, 0L, 0L, MAPPER.writeValueAsString(SHARD_SPEC),
+        // dimensions, metrics, projections, last_compaction_state, replication_factor
+        "[\"dim1\",\"dim2\"]", "[\"met1\",\"met2\"]", "[]", null, 2L
+    };
+    Assert.assertArrayEquals(wikiSegment3Expected, rows.get(7));
   }
 
   @Test
@@ -787,44 +747,6 @@ public class SystemSchemaTest extends CalciteTestBase
     );
   }
 
-  private void verifyRow(
-      Object[] row,
-      String segmentId,
-      long size,
-      long partitionNum,
-      long numReplicas,
-      long numRows,
-      long isPublished,
-      long isAvailable,
-      long isRealtime,
-      long isOvershadowed,
-      CompactionState compactionState,
-      long replicationFactor
-  ) throws Exception
-  {
-    Assert.assertEquals(segmentId, row[0].toString());
-    SegmentId id = Iterables.get(SegmentId.iterateAllPossibleParsings(segmentId), 0);
-    Assert.assertEquals(id.getDataSource(), row[1]);
-    Assert.assertEquals(id.getIntervalStart().toString(), row[2]);
-    Assert.assertEquals(id.getIntervalEnd().toString(), row[3]);
-    Assert.assertEquals(size, row[4]);
-    Assert.assertEquals(id.getVersion(), row[5]);
-    Assert.assertEquals(partitionNum, row[6]);
-    Assert.assertEquals(numReplicas, row[7]);
-    Assert.assertEquals(numRows, row[8]);
-    Assert.assertEquals((((isPublished == 1) && (isOvershadowed == 0)) || (isRealtime == 1)) ? 1L : 0L, row[9]);
-    Assert.assertEquals(isPublished, row[10]);
-    Assert.assertEquals(isAvailable, row[11]);
-    Assert.assertEquals(isRealtime, row[12]);
-    Assert.assertEquals(isOvershadowed, row[13]);
-    if (compactionState == null) {
-      Assert.assertNull(row[17]);
-    } else {
-      Assert.assertEquals(MAPPER.writeValueAsString(compactionState), row[17]);
-    }
-    Assert.assertEquals(replicationFactor, row[18]);
-  }
-
   @Test
   public void testServersTable() throws URISyntaxException
   {
@@ -1391,7 +1313,8 @@ public class SystemSchemaTest extends CalciteTestBase
     EasyMock.replay(supervisorTable);
 
     String json = "[{\n"
-                  + "\t\"id\": \"wikipedia\",\n"
+                  + "\t\"id\": \"wikipedia_supervisor\",\n"
+                  + "\t\"dataSource\": \"wikipedia\",\n"
                   + "\t\"state\": \"UNHEALTHY_SUPERVISOR\",\n"
                   + "\t\"detailedState\": \"UNABLE_TO_CONNECT_TO_STREAM\",\n"
                   + "\t\"healthy\": false,\n"
@@ -1415,16 +1338,17 @@ public class SystemSchemaTest extends CalciteTestBase
     final List<Object[]> rows = supervisorTable.scan(dataContext).toList();
 
     Object[] row0 = rows.get(0);
-    Assert.assertEquals("wikipedia", row0[0].toString());
-    Assert.assertEquals("UNHEALTHY_SUPERVISOR", row0[1].toString());
-    Assert.assertEquals("UNABLE_TO_CONNECT_TO_STREAM", row0[2].toString());
-    Assert.assertEquals(0L, row0[3]);
-    Assert.assertEquals("kafka", row0[4].toString());
-    Assert.assertEquals("wikipedia", row0[5].toString());
-    Assert.assertEquals(0L, row0[6]);
+    Assert.assertEquals("wikipedia_supervisor", row0[0].toString());
+    Assert.assertEquals("wikipedia", row0[1].toString());
+    Assert.assertEquals("UNHEALTHY_SUPERVISOR", row0[2].toString());
+    Assert.assertEquals("UNABLE_TO_CONNECT_TO_STREAM", row0[3].toString());
+    Assert.assertEquals(0L, row0[4]);
+    Assert.assertEquals("kafka", row0[5].toString());
+    Assert.assertEquals("wikipedia", row0[6].toString());
+    Assert.assertEquals(0L, row0[7]);
     Assert.assertEquals(
         "{\"type\":\"kafka\",\"dataSchema\":{\"dataSource\":\"wikipedia\"},\"context\":null,\"suspended\":false}",
-        row0[7].toString()
+        row0[8].toString()
     );
 
     // Verify value types.
