@@ -48,6 +48,7 @@ import org.apache.druid.guice.StorageNodeModule;
 import org.apache.druid.guice.annotations.Global;
 import org.apache.druid.guice.annotations.Merging;
 import org.apache.druid.guice.annotations.Self;
+import org.apache.druid.guice.security.PolicyModule;
 import org.apache.druid.initialization.CoreInjectorBuilder;
 import org.apache.druid.initialization.DruidModule;
 import org.apache.druid.initialization.ServiceInjectorBuilder;
@@ -56,6 +57,7 @@ import org.apache.druid.java.util.common.io.Closer;
 import org.apache.druid.java.util.emitter.service.ServiceEmitter;
 import org.apache.druid.java.util.http.client.HttpClient;
 import org.apache.druid.math.expr.ExprMacroTable;
+import org.apache.druid.query.DefaultGenericQueryMetricsFactory;
 import org.apache.druid.query.DruidProcessingConfig;
 import org.apache.druid.query.GlobalTableDataSource;
 import org.apache.druid.query.QueryRunnerFactoryConglomerate;
@@ -72,6 +74,7 @@ import org.apache.druid.query.groupby.GroupByStatsProvider;
 import org.apache.druid.query.groupby.GroupingEngine;
 import org.apache.druid.query.groupby.TestGroupByBuffers;
 import org.apache.druid.query.lookup.LookupExtractorFactoryContainerProvider;
+import org.apache.druid.query.policy.NoopPolicyEnforcer;
 import org.apache.druid.query.topn.TopNQueryConfig;
 import org.apache.druid.quidem.ProjectPathUtils;
 import org.apache.druid.quidem.TestSqlModule;
@@ -402,6 +405,7 @@ public class SqlTestFramework
     public DruidModule getCoreModule()
     {
       return DruidModuleCollection.of(
+          new PolicyModule(),
           new LookylooModule(),
           new SegmentWranglerModule(),
           new ExpressionModule(),
@@ -849,14 +853,11 @@ public class SqlTestFramework
           framework.injector.getInstance(JoinableFactoryWrapper.class),
           framework.builder.catalogResolver,
           authConfig != null ? authConfig : new AuthConfig(),
+          NoopPolicyEnforcer.instance(),
           new DruidHookDispatcher()
       );
       componentSupplier.finalizePlanner(this);
-      this.statementFactory = QueryFrameworkUtils.createSqlStatementFactory(
-          framework.engine,
-          plannerFactory,
-          authConfig
-      );
+      this.statementFactory = QueryFrameworkUtils.createSqlMultiStatementFactory(framework.engine, plannerFactory);
       componentSupplier.populateViews(viewManager, plannerFactory);
     }
 
@@ -1145,7 +1146,8 @@ public class SqlTestFramework
           injector.getInstance(Cache.class),
           injector.getInstance(CacheConfig.class),
           new SubqueryGuardrailHelper(null, JvmUtils.getRuntimeInfo().getMaxHeapSizeBytes(), 1),
-          new SubqueryCountStatsProvider()
+          new SubqueryCountStatsProvider(),
+          new DefaultGenericQueryMetricsFactory()
       );
     }
 
