@@ -255,8 +255,6 @@ interface EditorColumn {
 export interface SchemaStepProps {
   queryString: string;
   onQueryStringChange(queryString: string): void;
-  forceSegmentSortByTime: boolean;
-  changeForceSegmentSortByTime(forceSegmentSortByTime: boolean): void;
   enableAnalyze: boolean;
   goToQuery: () => void;
   onBack(): void;
@@ -268,8 +266,6 @@ export const SchemaStep = function SchemaStep(props: SchemaStepProps) {
   const {
     queryString,
     onQueryStringChange,
-    forceSegmentSortByTime,
-    changeForceSegmentSortByTime,
     enableAnalyze,
     goToQuery,
     onBack,
@@ -674,7 +670,9 @@ export const SchemaStep = function SchemaStep(props: SchemaStepProps) {
                         <MenuItem
                           key={i}
                           text={outputName}
-                          disabled={outputName === TIME_COLUMN && forceSegmentSortByTime}
+                          disabled={
+                            outputName === TIME_COLUMN && ingestQueryPattern.forceSegmentSortByTime
+                          }
                           onClick={() =>
                             updatePattern({
                               ...ingestQueryPattern,
@@ -690,8 +688,10 @@ export const SchemaStep = function SchemaStep(props: SchemaStepProps) {
                   <MenuBoolean
                     icon={IconNames.GEOTIME}
                     text="Force segment sort by time"
-                    value={forceSegmentSortByTime}
-                    onValueChange={v => changeForceSegmentSortByTime(Boolean(v))}
+                    value={ingestQueryPattern.forceSegmentSortByTime}
+                    onValueChange={v =>
+                      updatePattern({ ...ingestQueryPattern, forceSegmentSortByTime: Boolean(v) })
+                    }
                     optionsText={ENABLED_DISABLED_OPTIONS_TEXT}
                     optionsLabelElement={{ false: EXPERIMENTAL_ICON }}
                   />
