@@ -189,7 +189,7 @@ export const QueryTab = React.memo(function QueryTab(props: QueryTabProps) {
     return Boolean(queryDuration && queryDuration < 10000);
   }
 
-  const queryInputRef = useRef<FlexibleQueryInput | null>(null);
+  const queryInputRef = useRef<{ goToPosition: (rowColumn: RowColumn) => void } | null>(null);
 
   const cachedExecutionState = ExecutionStateCache.getState(id);
   const currentRunningPromise = WorkbenchRunningPromises.getPromise(id);
@@ -278,7 +278,7 @@ export const QueryTab = React.memo(function QueryTab(props: QueryTabProps) {
               void cancelToken.promise
                 .then(cancel => {
                   if (cancel.message === QueryManager.TERMINATION_MESSAGE) return;
-                  return Api.instance.delete(`/druid/v2/sql/dart/${Api.encodePath(cancelQueryId)}`);
+                  return Api.instance.delete(`/druid/v2/sql/${Api.encodePath(cancelQueryId)}`);
                 })
                 .catch(() => {});
             }
@@ -286,7 +286,7 @@ export const QueryTab = React.memo(function QueryTab(props: QueryTabProps) {
             onQueryChange(props.query.changeLastExecution(undefined));
 
             const executionPromise = Api.instance
-              .post(`/druid/v2/sql/dart`, query, {
+              .post(`/druid/v2/sql`, query, {
                 cancelToken: new axios.CancelToken(cancelFn => {
                   nativeQueryCancelFnRef.current = cancelFn;
                 }),
@@ -436,7 +436,7 @@ export const QueryTab = React.memo(function QueryTab(props: QueryTabProps) {
 
       const capacityInfo = await getClusterCapacity?.();
 
-      const effectiveMaxNumTasks = effectiveQuery.queryContext.maxNumTasks ?? 2;
+      const effectiveMaxNumTasks = effectiveQuery.getMaxNumTasks() ?? 2;
       if (capacityInfo && capacityInfo.availableTaskSlots < effectiveMaxNumTasks) {
         setAlertElement(
           <CapacityAlert
